#nullable enable
using Canon.Core.Abstractions;
using Canon.Core.GrammarParser;
using Canon.Core.Enums;
namespace Canon.Core.GrammarParser;

public class GeneratedTransformer : ITransformer
{
    private IDictionary<TerminatorBase, string> _shiftPointers;

    public string Name { get; }

    public IDictionary<Terminator, ReduceInformation> ReduceTable { get; }

    public IDictionary<TerminatorBase, ITransformer> ShiftTable { get; }

    public GeneratedTransformer(Dictionary<TerminatorBase, string> shiftTable,
        Dictionary<Terminator, ReduceInformation> reduceTable, string name)
    {
        ReduceTable = reduceTable;
        ShiftTable = new Dictionary<TerminatorBase, ITransformer>();
        _shiftPointers = shiftTable;
        Name = name;
    }

    public GeneratedTransformer()
    {
        ReduceTable = new Dictionary<Terminator, ReduceInformation>();
        ShiftTable = new Dictionary<TerminatorBase, ITransformer>();
        _shiftPointers = new Dictionary<TerminatorBase, string>();
        Name = Guid.NewGuid().ToString();
    }

    public void ConstructShiftTable(Dictionary<string, GeneratedTransformer> transformers)
    {
        foreach (KeyValuePair<TerminatorBase,string> pair in _shiftPointers)
        {
            ShiftTable.Add(pair.Key, transformers[pair.Value]);
        }
    }

    public override bool Equals(object? obj)
    {
        if (obj is not GeneratedTransformer other)
        {
            return false;
        }

        return Name == other.Name;
    }

    public override int GetHashCode() => Name.GetHashCode();
}
public class GeneratedGrammarParser : IGrammarParser
{
	private static readonly Dictionary<string, GeneratedTransformer> s_transformers = new()
	{
		{ "42b4bf15-b0ab-4e7a-86db-947c31c11f30", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ProgramStruct), "d8d21174-f64c-4560-a541-d505330e434c"}, { new NonTerminator(NonTerminatorType.ProgramHead), "5b2102c8-afaf-4da8-b703-6a26fc87e199"}, { new Terminator(KeywordType.Program), "8819303a-5fcb-484b-b2fa-3122244808ee"},}, new Dictionary<Terminator, ReduceInformation>{ }, "42b4bf15-b0ab-4e7a-86db-947c31c11f30") },
		{ "d8d21174-f64c-4560-a541-d505330e434c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.EndTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.StartNonTerminator))}, }, "d8d21174-f64c-4560-a541-d505330e434c") },
		{ "5b2102c8-afaf-4da8-b703-6a26fc87e199", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Semicolon), "89fb603c-d204-4e25-9c0f-222cbb333e30"},}, new Dictionary<Terminator, ReduceInformation>{ }, "5b2102c8-afaf-4da8-b703-6a26fc87e199") },
		{ "8819303a-5fcb-484b-b2fa-3122244808ee", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "02e9caf9-51d5-4d16-bfd6-08bbd712ee77"},}, new Dictionary<Terminator, ReduceInformation>{ }, "8819303a-5fcb-484b-b2fa-3122244808ee") },
		{ "89fb603c-d204-4e25-9c0f-222cbb333e30", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ProgramBody), "96e3c907-57ab-4b63-ae93-3698cdf12d84"}, { new NonTerminator(NonTerminatorType.ConstDeclarations), "f1e54d0b-5b90-4d87-a3a2-ce1eef70194c"}, { new Terminator(KeywordType.Const), "40fc8b9c-1cf8-4eb8-8de9-08e18ffeb309"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ConstDeclarations))}, { new Terminator(KeywordType.Procedure), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ConstDeclarations))}, { new Terminator(KeywordType.Function), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ConstDeclarations))}, { new Terminator(KeywordType.Var), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ConstDeclarations))}, }, "89fb603c-d204-4e25-9c0f-222cbb333e30") },
		{ "02e9caf9-51d5-4d16-bfd6-08bbd712ee77", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "51a89292-6025-41a0-b737-f7ea0d74430f"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.ProgramHead))}, }, "02e9caf9-51d5-4d16-bfd6-08bbd712ee77") },
		{ "96e3c907-57ab-4b63-ae93-3698cdf12d84", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Period), "e88d7175-ec33-48b2-8b39-3dbe129bc1cc"},}, new Dictionary<Terminator, ReduceInformation>{ }, "96e3c907-57ab-4b63-ae93-3698cdf12d84") },
		{ "f1e54d0b-5b90-4d87-a3a2-ce1eef70194c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.VarDeclarations), "1f86bd0e-ef29-48c0-b48f-306c74d62fe8"}, { new Terminator(KeywordType.Var), "f60de607-1fed-4507-8b54-b3860d6e31ce"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(0, new NonTerminator(NonTerminatorType.VarDeclarations))}, { new Terminator(KeywordType.Procedure), new ReduceInformation(0, new NonTerminator(NonTerminatorType.VarDeclarations))}, { new Terminator(KeywordType.Function), new ReduceInformation(0, new NonTerminator(NonTerminatorType.VarDeclarations))}, }, "f1e54d0b-5b90-4d87-a3a2-ce1eef70194c") },
		{ "40fc8b9c-1cf8-4eb8-8de9-08e18ffeb309", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ConstDeclaration), "127565dd-ff0a-4433-a0af-219c37ef2421"}, { Terminator.IdentifierTerminator, "ea240188-f7bf-4060-a07a-964690763117"},}, new Dictionary<Terminator, ReduceInformation>{ }, "40fc8b9c-1cf8-4eb8-8de9-08e18ffeb309") },
		{ "51a89292-6025-41a0-b737-f7ea0d74430f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.IdentifierList), "3fdbb3e8-674c-4c51-8ac2-97992bce7708"}, { new Terminator(DelimiterType.Comma), "8dd5274a-66ad-4fbd-a2dc-e42109392f32"}, { new Terminator(DelimiterType.Colon), "221cb9d5-d5eb-40ce-8e83-0e49ba208b25"},}, new Dictionary<Terminator, ReduceInformation>{ }, "51a89292-6025-41a0-b737-f7ea0d74430f") },
		{ "e88d7175-ec33-48b2-8b39-3dbe129bc1cc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.EndTerminator, new ReduceInformation(4, new NonTerminator(NonTerminatorType.ProgramStruct))}, }, "e88d7175-ec33-48b2-8b39-3dbe129bc1cc") },
		{ "1f86bd0e-ef29-48c0-b48f-306c74d62fe8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SubprogramDeclarations), "6e53124d-1a54-40f8-b555-2beeeebea5b1"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(0, new NonTerminator(NonTerminatorType.SubprogramDeclarations))}, { new Terminator(KeywordType.Procedure), new ReduceInformation(0, new NonTerminator(NonTerminatorType.SubprogramDeclarations))}, { new Terminator(KeywordType.Function), new ReduceInformation(0, new NonTerminator(NonTerminatorType.SubprogramDeclarations))}, }, "1f86bd0e-ef29-48c0-b48f-306c74d62fe8") },
		{ "f60de607-1fed-4507-8b54-b3860d6e31ce", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.VarDeclaration), "98fc8981-d941-404e-a12c-be870dbff214"}, { Terminator.IdentifierTerminator, "a4a4eb25-4859-4776-ad08-ce9a255e3e47"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f60de607-1fed-4507-8b54-b3860d6e31ce") },
		{ "127565dd-ff0a-4433-a0af-219c37ef2421", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Semicolon), "f6ac443c-94a4-4d68-9452-ea2aa38892da"},}, new Dictionary<Terminator, ReduceInformation>{ }, "127565dd-ff0a-4433-a0af-219c37ef2421") },
		{ "ea240188-f7bf-4060-a07a-964690763117", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(OperatorType.Equal), "db1dab4b-2897-4e3d-a5ef-8a53cdca360a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ea240188-f7bf-4060-a07a-964690763117") },
		{ "3fdbb3e8-674c-4c51-8ac2-97992bce7708", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "cc23ec72-3ada-4c37-8021-5ac5e5375577"},}, new Dictionary<Terminator, ReduceInformation>{ }, "3fdbb3e8-674c-4c51-8ac2-97992bce7708") },
		{ "8dd5274a-66ad-4fbd-a2dc-e42109392f32", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "cfaf12ad-107d-454d-8fd7-0ec5c15d4758"},}, new Dictionary<Terminator, ReduceInformation>{ }, "8dd5274a-66ad-4fbd-a2dc-e42109392f32") },
		{ "221cb9d5-d5eb-40ce-8e83-0e49ba208b25", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Type), "560c7f56-9b49-4c76-aacc-c73556be2e01"}, { new NonTerminator(NonTerminatorType.BasicType), "743791cc-6cc6-4bb0-ba56-2413529ba4da"}, { new Terminator(KeywordType.Array), "f66d3b8b-3a05-4ae6-86ae-f7246da790d7"}, { new Terminator(KeywordType.Integer), "b94dae85-8e9c-4115-97f4-34e052bde163"}, { new Terminator(KeywordType.Real), "3052501b-2122-4544-80d8-457e242a600c"}, { new Terminator(KeywordType.Boolean), "a0856ce5-e6fb-4370-9190-af36caf2f75f"}, { new Terminator(KeywordType.Character), "cf014b4e-1286-43ca-8fc2-abe48e6dbfe7"},}, new Dictionary<Terminator, ReduceInformation>{ }, "221cb9d5-d5eb-40ce-8e83-0e49ba208b25") },
		{ "6e53124d-1a54-40f8-b555-2beeeebea5b1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.CompoundStatement), "94bc948f-e596-4dfa-9a45-46cffa2a0250"}, { new Terminator(KeywordType.Begin), "923128ab-0aaa-4006-92ec-f4c45ba7e3a5"}, { new NonTerminator(NonTerminatorType.Subprogram), "17b892d5-1445-4a21-94dc-5719d33a23d5"}, { new NonTerminator(NonTerminatorType.SubprogramHead), "e0a5a8e1-acba-44c1-a4c0-905a2ee5d4a6"}, { new Terminator(KeywordType.Procedure), "3de2302a-3c41-483d-8bfb-a213d0f8f22a"}, { new Terminator(KeywordType.Function), "045cdc40-6a2d-4f24-8897-ad66bfc33761"},}, new Dictionary<Terminator, ReduceInformation>{ }, "6e53124d-1a54-40f8-b555-2beeeebea5b1") },
		{ "98fc8981-d941-404e-a12c-be870dbff214", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Semicolon), "80fb288b-0582-4a0b-b3dd-5d8b2bba70ce"},}, new Dictionary<Terminator, ReduceInformation>{ }, "98fc8981-d941-404e-a12c-be870dbff214") },
		{ "a4a4eb25-4859-4776-ad08-ce9a255e3e47", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.IdentifierList), "34fa406a-6eb6-4a4b-a862-faa32e8efd19"}, { new Terminator(DelimiterType.Comma), "f986d4b0-0bd1-4c91-a419-a72005b9f403"}, { new Terminator(DelimiterType.Colon), "a7778d9a-db87-41b8-9434-f2cf7b810bcf"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a4a4eb25-4859-4776-ad08-ce9a255e3e47") },
		{ "f6ac443c-94a4-4d68-9452-ea2aa38892da", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "b550be58-4517-415d-b55d-5cb7151be29a"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ConstDeclarations))}, { new Terminator(KeywordType.Procedure), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ConstDeclarations))}, { new Terminator(KeywordType.Function), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ConstDeclarations))}, { new Terminator(KeywordType.Var), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ConstDeclarations))}, }, "f6ac443c-94a4-4d68-9452-ea2aa38892da") },
		{ "db1dab4b-2897-4e3d-a5ef-8a53cdca360a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ConstValue), "0499a00b-5dcb-4e3e-a197-920bebda9e0e"}, { new Terminator(OperatorType.Plus), "c6d6b909-feb1-49b7-80da-65204d6f4fd4"}, { new Terminator(OperatorType.Minus), "69d82d53-9081-48a2-947e-e21e531c1c1d"}, { Terminator.NumberTerminator, "18fe0a28-7708-4805-9ceb-1909ab8e173d"}, { Terminator.CharacterTerminator, "17d023ce-a851-4656-ad8e-8265eecb7cd8"},}, new Dictionary<Terminator, ReduceInformation>{ }, "db1dab4b-2897-4e3d-a5ef-8a53cdca360a") },
		{ "cc23ec72-3ada-4c37-8021-5ac5e5375577", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(5, new NonTerminator(NonTerminatorType.ProgramHead))}, }, "cc23ec72-3ada-4c37-8021-5ac5e5375577") },
		{ "cfaf12ad-107d-454d-8fd7-0ec5c15d4758", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.IdentifierList), "ee0dfff9-5a61-449c-805a-b43300419d4f"}, { new Terminator(DelimiterType.Comma), "8dd5274a-66ad-4fbd-a2dc-e42109392f32"}, { new Terminator(DelimiterType.Colon), "221cb9d5-d5eb-40ce-8e83-0e49ba208b25"},}, new Dictionary<Terminator, ReduceInformation>{ }, "cfaf12ad-107d-454d-8fd7-0ec5c15d4758") },
		{ "560c7f56-9b49-4c76-aacc-c73556be2e01", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.IdentifierList))}, }, "560c7f56-9b49-4c76-aacc-c73556be2e01") },
		{ "743791cc-6cc6-4bb0-ba56-2413529ba4da", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Type))}, }, "743791cc-6cc6-4bb0-ba56-2413529ba4da") },
		{ "f66d3b8b-3a05-4ae6-86ae-f7246da790d7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftSquareBracket), "735b72d3-5e96-4611-b9c9-26228c4b6880"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f66d3b8b-3a05-4ae6-86ae-f7246da790d7") },
		{ "b94dae85-8e9c-4115-97f4-34e052bde163", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "b94dae85-8e9c-4115-97f4-34e052bde163") },
		{ "3052501b-2122-4544-80d8-457e242a600c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "3052501b-2122-4544-80d8-457e242a600c") },
		{ "a0856ce5-e6fb-4370-9190-af36caf2f75f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "a0856ce5-e6fb-4370-9190-af36caf2f75f") },
		{ "cf014b4e-1286-43ca-8fc2-abe48e6dbfe7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "cf014b4e-1286-43ca-8fc2-abe48e6dbfe7") },
		{ "94bc948f-e596-4dfa-9a45-46cffa2a0250", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Period), new ReduceInformation(4, new NonTerminator(NonTerminatorType.ProgramBody))}, }, "94bc948f-e596-4dfa-9a45-46cffa2a0250") },
		{ "923128ab-0aaa-4006-92ec-f4c45ba7e3a5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.StatementList), "44f64833-526e-4a1c-ae37-f0ddaaa304dc"}, { new NonTerminator(NonTerminatorType.Statement), "c4bea712-ff83-427d-bf36-f8d83c3ed530"}, { new NonTerminator(NonTerminatorType.Variable), "be92d52d-f5cd-445d-b485-bd33a70d96d0"}, { Terminator.IdentifierTerminator, "b2246d94-54c6-4d0a-b568-08ce2b05aed2"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "54a3f285-f7dd-4bc1-bea4-a895b234ba48"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "430eb033-8171-4e0b-a572-c7fc61d11c56"}, { new Terminator(KeywordType.If), "5a82796b-2d92-4130-9660-38c01054dba4"}, { new Terminator(KeywordType.For), "6f8cff58-95d7-47f8-8456-787a36fc1c87"}, { new Terminator(KeywordType.Begin), "e9129679-d275-4d11-a66e-30fda7679656"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "923128ab-0aaa-4006-92ec-f4c45ba7e3a5") },
		{ "17b892d5-1445-4a21-94dc-5719d33a23d5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Semicolon), "16c07b92-c75f-4d16-8e20-0fef656c6fc3"},}, new Dictionary<Terminator, ReduceInformation>{ }, "17b892d5-1445-4a21-94dc-5719d33a23d5") },
		{ "e0a5a8e1-acba-44c1-a4c0-905a2ee5d4a6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Semicolon), "279e8612-d94e-4b67-9465-5922819daad4"},}, new Dictionary<Terminator, ReduceInformation>{ }, "e0a5a8e1-acba-44c1-a4c0-905a2ee5d4a6") },
		{ "3de2302a-3c41-483d-8bfb-a213d0f8f22a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "ffe09817-8554-42b2-b862-e382768d33cc"},}, new Dictionary<Terminator, ReduceInformation>{ }, "3de2302a-3c41-483d-8bfb-a213d0f8f22a") },
		{ "045cdc40-6a2d-4f24-8897-ad66bfc33761", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "57416168-42b2-4627-99b8-3849093e01e8"},}, new Dictionary<Terminator, ReduceInformation>{ }, "045cdc40-6a2d-4f24-8897-ad66bfc33761") },
		{ "80fb288b-0582-4a0b-b3dd-5d8b2bba70ce", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "935d6c9f-d4d4-445b-9154-26664a79ca9a"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(3, new NonTerminator(NonTerminatorType.VarDeclarations))}, { new Terminator(KeywordType.Procedure), new ReduceInformation(3, new NonTerminator(NonTerminatorType.VarDeclarations))}, { new Terminator(KeywordType.Function), new ReduceInformation(3, new NonTerminator(NonTerminatorType.VarDeclarations))}, }, "80fb288b-0582-4a0b-b3dd-5d8b2bba70ce") },
		{ "34fa406a-6eb6-4a4b-a862-faa32e8efd19", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.VarDeclaration))}, }, "34fa406a-6eb6-4a4b-a862-faa32e8efd19") },
		{ "f986d4b0-0bd1-4c91-a419-a72005b9f403", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "3e5fea94-5de4-4227-ae6f-ac60218a795c"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f986d4b0-0bd1-4c91-a419-a72005b9f403") },
		{ "a7778d9a-db87-41b8-9434-f2cf7b810bcf", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Type), "32b52204-167f-466c-b639-88eef60447aa"}, { new NonTerminator(NonTerminatorType.BasicType), "cffcf892-f9cf-426f-a533-6f9f2430b550"}, { new Terminator(KeywordType.Array), "07021fd8-9fc4-49ba-a94e-df6a2ae7ac16"}, { new Terminator(KeywordType.Integer), "ebe37bec-c209-4195-a140-f57d3f2763ea"}, { new Terminator(KeywordType.Real), "f3352fc3-8224-4718-b0cc-5afcdbcea304"}, { new Terminator(KeywordType.Boolean), "7942b503-91a1-408d-bdf4-f3f76c9a33a0"}, { new Terminator(KeywordType.Character), "ad9d0031-e9ad-485c-bbee-619e17a3a472"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a7778d9a-db87-41b8-9434-f2cf7b810bcf") },
		{ "b550be58-4517-415d-b55d-5cb7151be29a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(OperatorType.Equal), "654d22cd-b856-4b69-b1f8-d8e8d1909ea5"},}, new Dictionary<Terminator, ReduceInformation>{ }, "b550be58-4517-415d-b55d-5cb7151be29a") },
		{ "0499a00b-5dcb-4e3e-a197-920bebda9e0e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ConstDeclaration))}, }, "0499a00b-5dcb-4e3e-a197-920bebda9e0e") },
		{ "c6d6b909-feb1-49b7-80da-65204d6f4fd4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.NumberTerminator, "d0001074-a463-4021-8208-c844ee9ad83d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c6d6b909-feb1-49b7-80da-65204d6f4fd4") },
		{ "69d82d53-9081-48a2-947e-e21e531c1c1d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.NumberTerminator, "7c131815-f61d-40c2-a4ea-78f43bd51358"},}, new Dictionary<Terminator, ReduceInformation>{ }, "69d82d53-9081-48a2-947e-e21e531c1c1d") },
		{ "18fe0a28-7708-4805-9ceb-1909ab8e173d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ConstValue))}, }, "18fe0a28-7708-4805-9ceb-1909ab8e173d") },
		{ "17d023ce-a851-4656-ad8e-8265eecb7cd8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ConstValue))}, }, "17d023ce-a851-4656-ad8e-8265eecb7cd8") },
		{ "ee0dfff9-5a61-449c-805a-b43300419d4f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdentifierList))}, }, "ee0dfff9-5a61-449c-805a-b43300419d4f") },
		{ "735b72d3-5e96-4611-b9c9-26228c4b6880", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Period), "5a045313-fd27-48f6-ad35-103f76158649"}, { Terminator.NumberTerminator, "9c9213aa-db0c-400f-a84f-f3bf743ec7d9"},}, new Dictionary<Terminator, ReduceInformation>{ }, "735b72d3-5e96-4611-b9c9-26228c4b6880") },
		{ "44f64833-526e-4a1c-ae37-f0ddaaa304dc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.End), "2330f2ca-176e-4395-9cee-233b07437388"}, { new Terminator(DelimiterType.Semicolon), "655817d0-6fda-4a1b-a6fb-1075ad52b093"},}, new Dictionary<Terminator, ReduceInformation>{ }, "44f64833-526e-4a1c-ae37-f0ddaaa304dc") },
		{ "c4bea712-ff83-427d-bf36-f8d83c3ed530", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.StatementList))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.StatementList))}, }, "c4bea712-ff83-427d-bf36-f8d83c3ed530") },
		{ "be92d52d-f5cd-445d-b485-bd33a70d96d0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(OperatorType.Assign), "caf5c998-ae57-4328-addf-2ce3a871b318"},}, new Dictionary<Terminator, ReduceInformation>{ }, "be92d52d-f5cd-445d-b485-bd33a70d96d0") },
		{ "b2246d94-54c6-4d0a-b568-08ce2b05aed2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(OperatorType.Assign), "6868fa63-2bc6-4d03-8410-fd7b64fd4919"}, { new NonTerminator(NonTerminatorType.IdVarPart), "103ddd16-3ad7-45cc-a651-791a092a020b"}, { new Terminator(DelimiterType.LeftSquareBracket), "2b0d7e71-adca-43de-a708-6c2f8c5613ae"}, { new Terminator(DelimiterType.LeftParenthesis), "47ecca50-56e9-420a-b41c-162f48e7a850"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(OperatorType.Assign), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ProcedureCall))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ProcedureCall))}, }, "b2246d94-54c6-4d0a-b568-08ce2b05aed2") },
		{ "54a3f285-f7dd-4bc1-bea4-a895b234ba48", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, }, "54a3f285-f7dd-4bc1-bea4-a895b234ba48") },
		{ "430eb033-8171-4e0b-a572-c7fc61d11c56", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, }, "430eb033-8171-4e0b-a572-c7fc61d11c56") },
		{ "5a82796b-2d92-4130-9660-38c01054dba4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "fffa3768-ea73-45cb-9a77-204e92522278"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "d0b871cb-b302-4a53-856a-d5bd7f4e7d3c"}, { new NonTerminator(NonTerminatorType.Term), "5fd6c89b-6edd-4f4f-8d72-ce68a8f5d407"}, { new NonTerminator(NonTerminatorType.Factor), "67dd75bf-89f3-4ad3-8209-1d3a4d3f3303"}, { Terminator.NumberTerminator, "c90f9bea-bfaa-4211-8ea7-8c89c19751ad"}, { new NonTerminator(NonTerminatorType.Variable), "02430536-65bb-44a6-86f4-f3c102956340"}, { new Terminator(DelimiterType.LeftParenthesis), "cd305913-18dd-45cd-aebf-84bb50475ed5"}, { Terminator.IdentifierTerminator, "f41a8bc1-8fea-4591-a205-dd02472cc9e6"}, { new Terminator(KeywordType.Not), "21d6e202-9092-4593-a903-5b7ce59a3a44"}, { new Terminator(OperatorType.Minus), "d2f171ed-e2fb-4f7d-af92-d427c2f599b9"},}, new Dictionary<Terminator, ReduceInformation>{ }, "5a82796b-2d92-4130-9660-38c01054dba4") },
		{ "6f8cff58-95d7-47f8-8456-787a36fc1c87", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "ee010803-4c9a-4082-bf8d-d0b5ccfcbbd4"},}, new Dictionary<Terminator, ReduceInformation>{ }, "6f8cff58-95d7-47f8-8456-787a36fc1c87") },
		{ "e9129679-d275-4d11-a66e-30fda7679656", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.StatementList), "2c52c119-5fc7-40bc-b02a-516ec9121c43"}, { new NonTerminator(NonTerminatorType.Statement), "c4bea712-ff83-427d-bf36-f8d83c3ed530"}, { new NonTerminator(NonTerminatorType.Variable), "be92d52d-f5cd-445d-b485-bd33a70d96d0"}, { Terminator.IdentifierTerminator, "b2246d94-54c6-4d0a-b568-08ce2b05aed2"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "54a3f285-f7dd-4bc1-bea4-a895b234ba48"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "430eb033-8171-4e0b-a572-c7fc61d11c56"}, { new Terminator(KeywordType.If), "5a82796b-2d92-4130-9660-38c01054dba4"}, { new Terminator(KeywordType.For), "6f8cff58-95d7-47f8-8456-787a36fc1c87"}, { new Terminator(KeywordType.Begin), "e9129679-d275-4d11-a66e-30fda7679656"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "e9129679-d275-4d11-a66e-30fda7679656") },
		{ "16c07b92-c75f-4d16-8e20-0fef656c6fc3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SubprogramDeclarations))}, { new Terminator(KeywordType.Procedure), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SubprogramDeclarations))}, { new Terminator(KeywordType.Function), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SubprogramDeclarations))}, }, "16c07b92-c75f-4d16-8e20-0fef656c6fc3") },
		{ "279e8612-d94e-4b67-9465-5922819daad4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SubprogramBody), "7bd2877e-0f4d-42ba-a9c9-0f76a5732c4b"}, { new NonTerminator(NonTerminatorType.ConstDeclarations), "0bd495cf-e0a2-444d-b5af-322022d1310e"}, { new Terminator(KeywordType.Const), "13157f83-6b01-4a74-b70d-a374f4cc294b"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ConstDeclarations))}, { new Terminator(KeywordType.Var), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ConstDeclarations))}, }, "279e8612-d94e-4b67-9465-5922819daad4") },
		{ "ffe09817-8554-42b2-b862-e382768d33cc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.FormalParameter), "ef2e9739-b0fb-4b0e-89bb-4968a7a31a77"}, { new Terminator(DelimiterType.LeftParenthesis), "e20a85d5-a7a1-428f-8b63-b0a909a3225e"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.FormalParameter))}, }, "ffe09817-8554-42b2-b862-e382768d33cc") },
		{ "57416168-42b2-4627-99b8-3849093e01e8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.FormalParameter), "ab644005-e44f-4aff-a653-e38bfd5e4260"}, { new Terminator(DelimiterType.LeftParenthesis), "e9766404-a352-4a50-83b7-ccca76c02d0c"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Colon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.FormalParameter))}, }, "57416168-42b2-4627-99b8-3849093e01e8") },
		{ "935d6c9f-d4d4-445b-9154-26664a79ca9a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.IdentifierList), "a7090436-6db2-4c21-b89d-021878aa45d0"}, { new Terminator(DelimiterType.Comma), "f986d4b0-0bd1-4c91-a419-a72005b9f403"}, { new Terminator(DelimiterType.Colon), "a7778d9a-db87-41b8-9434-f2cf7b810bcf"},}, new Dictionary<Terminator, ReduceInformation>{ }, "935d6c9f-d4d4-445b-9154-26664a79ca9a") },
		{ "3e5fea94-5de4-4227-ae6f-ac60218a795c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.IdentifierList), "8eec6fa3-a419-4e42-8f12-f700af89224a"}, { new Terminator(DelimiterType.Comma), "f986d4b0-0bd1-4c91-a419-a72005b9f403"}, { new Terminator(DelimiterType.Colon), "a7778d9a-db87-41b8-9434-f2cf7b810bcf"},}, new Dictionary<Terminator, ReduceInformation>{ }, "3e5fea94-5de4-4227-ae6f-ac60218a795c") },
		{ "32b52204-167f-466c-b639-88eef60447aa", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.IdentifierList))}, }, "32b52204-167f-466c-b639-88eef60447aa") },
		{ "cffcf892-f9cf-426f-a533-6f9f2430b550", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Type))}, }, "cffcf892-f9cf-426f-a533-6f9f2430b550") },
		{ "07021fd8-9fc4-49ba-a94e-df6a2ae7ac16", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftSquareBracket), "d02e2812-27a6-40b9-8b5d-5894e9b76bda"},}, new Dictionary<Terminator, ReduceInformation>{ }, "07021fd8-9fc4-49ba-a94e-df6a2ae7ac16") },
		{ "ebe37bec-c209-4195-a140-f57d3f2763ea", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "ebe37bec-c209-4195-a140-f57d3f2763ea") },
		{ "f3352fc3-8224-4718-b0cc-5afcdbcea304", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "f3352fc3-8224-4718-b0cc-5afcdbcea304") },
		{ "7942b503-91a1-408d-bdf4-f3f76c9a33a0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "7942b503-91a1-408d-bdf4-f3f76c9a33a0") },
		{ "ad9d0031-e9ad-485c-bbee-619e17a3a472", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "ad9d0031-e9ad-485c-bbee-619e17a3a472") },
		{ "654d22cd-b856-4b69-b1f8-d8e8d1909ea5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ConstValue), "8bfd1368-b2c1-4b35-8fcd-22f65f624d6c"}, { new Terminator(OperatorType.Plus), "c6d6b909-feb1-49b7-80da-65204d6f4fd4"}, { new Terminator(OperatorType.Minus), "69d82d53-9081-48a2-947e-e21e531c1c1d"}, { Terminator.NumberTerminator, "18fe0a28-7708-4805-9ceb-1909ab8e173d"}, { Terminator.CharacterTerminator, "17d023ce-a851-4656-ad8e-8265eecb7cd8"},}, new Dictionary<Terminator, ReduceInformation>{ }, "654d22cd-b856-4b69-b1f8-d8e8d1909ea5") },
		{ "d0001074-a463-4021-8208-c844ee9ad83d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.ConstValue))}, }, "d0001074-a463-4021-8208-c844ee9ad83d") },
		{ "7c131815-f61d-40c2-a4ea-78f43bd51358", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.ConstValue))}, }, "7c131815-f61d-40c2-a4ea-78f43bd51358") },
		{ "5a045313-fd27-48f6-ad35-103f76158649", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "bdd3a698-2eb3-4ced-91e1-cdb4b542295e"}, { new Terminator(DelimiterType.Comma), "16576908-4ae2-4aea-a555-907676e5b539"},}, new Dictionary<Terminator, ReduceInformation>{ }, "5a045313-fd27-48f6-ad35-103f76158649") },
		{ "9c9213aa-db0c-400f-a84f-f3bf743ec7d9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.DoubleDots), "0716c4bf-7f7d-4bf0-89f8-63703420822b"},}, new Dictionary<Terminator, ReduceInformation>{ }, "9c9213aa-db0c-400f-a84f-f3bf743ec7d9") },
		{ "2330f2ca-176e-4395-9cee-233b07437388", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Period), new ReduceInformation(3, new NonTerminator(NonTerminatorType.CompoundStatement))}, }, "2330f2ca-176e-4395-9cee-233b07437388") },
		{ "655817d0-6fda-4a1b-a6fb-1075ad52b093", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Statement), "4e628d3b-279c-4db0-9031-e62e64e72951"}, { new NonTerminator(NonTerminatorType.Variable), "be92d52d-f5cd-445d-b485-bd33a70d96d0"}, { Terminator.IdentifierTerminator, "b2246d94-54c6-4d0a-b568-08ce2b05aed2"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "54a3f285-f7dd-4bc1-bea4-a895b234ba48"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "430eb033-8171-4e0b-a572-c7fc61d11c56"}, { new Terminator(KeywordType.If), "5a82796b-2d92-4130-9660-38c01054dba4"}, { new Terminator(KeywordType.For), "6f8cff58-95d7-47f8-8456-787a36fc1c87"}, { new Terminator(KeywordType.Begin), "e9129679-d275-4d11-a66e-30fda7679656"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "655817d0-6fda-4a1b-a6fb-1075ad52b093") },
		{ "caf5c998-ae57-4328-addf-2ce3a871b318", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "0ba3aa16-bf5e-4e4e-900c-fe7b5003c129"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "5d2880aa-711d-43f9-81e6-7ddac2ae137b"}, { new NonTerminator(NonTerminatorType.Term), "5b386270-0769-4603-b5d4-38c90f32038c"}, { new NonTerminator(NonTerminatorType.Factor), "d0773f8f-3e0b-47b4-b9a5-33b173774588"}, { Terminator.NumberTerminator, "55ee839d-2c75-424b-87bd-a6dcfaed247a"}, { new NonTerminator(NonTerminatorType.Variable), "4dddee46-3cbe-40d8-8c3b-aa0e74d21b01"}, { new Terminator(DelimiterType.LeftParenthesis), "4e680b0b-45e5-4000-8b6e-36205a12432e"}, { Terminator.IdentifierTerminator, "8e321900-a883-4a45-b396-a6f26b6b341c"}, { new Terminator(KeywordType.Not), "0e41064a-e0af-48d7-80d1-e0e263116cfd"}, { new Terminator(OperatorType.Minus), "cc970159-9448-4bfb-9d6d-75aed317fae9"},}, new Dictionary<Terminator, ReduceInformation>{ }, "caf5c998-ae57-4328-addf-2ce3a871b318") },
		{ "6868fa63-2bc6-4d03-8410-fd7b64fd4919", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "ebfb4524-8918-4fcf-9126-1eb87ad726da"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "5d2880aa-711d-43f9-81e6-7ddac2ae137b"}, { new NonTerminator(NonTerminatorType.Term), "5b386270-0769-4603-b5d4-38c90f32038c"}, { new NonTerminator(NonTerminatorType.Factor), "d0773f8f-3e0b-47b4-b9a5-33b173774588"}, { Terminator.NumberTerminator, "55ee839d-2c75-424b-87bd-a6dcfaed247a"}, { new NonTerminator(NonTerminatorType.Variable), "4dddee46-3cbe-40d8-8c3b-aa0e74d21b01"}, { new Terminator(DelimiterType.LeftParenthesis), "4e680b0b-45e5-4000-8b6e-36205a12432e"}, { Terminator.IdentifierTerminator, "8e321900-a883-4a45-b396-a6f26b6b341c"}, { new Terminator(KeywordType.Not), "0e41064a-e0af-48d7-80d1-e0e263116cfd"}, { new Terminator(OperatorType.Minus), "cc970159-9448-4bfb-9d6d-75aed317fae9"},}, new Dictionary<Terminator, ReduceInformation>{ }, "6868fa63-2bc6-4d03-8410-fd7b64fd4919") },
		{ "103ddd16-3ad7-45cc-a651-791a092a020b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(OperatorType.Assign), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "103ddd16-3ad7-45cc-a651-791a092a020b") },
		{ "2b0d7e71-adca-43de-a708-6c2f8c5613ae", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "fe55e52a-c653-4ba5-91b6-868fa82ab021"}, { new NonTerminator(NonTerminatorType.Expression), "2e88b57e-7c35-4aee-9387-ff1c061cb7ab"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "089240b9-2be9-4199-97a8-41a6a9ac2f54"}, { new NonTerminator(NonTerminatorType.Term), "f46a091e-b991-43bd-877a-3c2eef645ab7"}, { new NonTerminator(NonTerminatorType.Factor), "5cb8d05c-bce3-47cc-8728-d7c037f2e9b1"}, { Terminator.NumberTerminator, "3928c790-d1f1-4d6a-8e98-6a0d0a3182e7"}, { new NonTerminator(NonTerminatorType.Variable), "08f29b35-75b5-4fe9-b677-ed18c2caa51b"}, { new Terminator(DelimiterType.LeftParenthesis), "7bb9f1fe-edd2-4812-9091-30ae6a61e59d"}, { Terminator.IdentifierTerminator, "ad126546-60a0-4982-8f68-c9a727b1c223"}, { new Terminator(KeywordType.Not), "d17a2e13-31b3-4e22-a0a6-5cf5fa910500"}, { new Terminator(OperatorType.Minus), "e8e4e781-e847-4c03-b1c8-fc377a1a86f7"},}, new Dictionary<Terminator, ReduceInformation>{ }, "2b0d7e71-adca-43de-a708-6c2f8c5613ae") },
		{ "47ecca50-56e9-420a-b41c-162f48e7a850", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "cf973399-71d1-445a-8a7e-87ce30786c51"}, { new NonTerminator(NonTerminatorType.Expression), "2e7bde52-48a5-420d-a6e3-9f664510da68"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "17428d1e-ed4e-461b-ad85-a5f1d8f006eb"}, { new NonTerminator(NonTerminatorType.Term), "bab97185-47af-4ddd-8275-1863e6c37e23"}, { new NonTerminator(NonTerminatorType.Factor), "f5d56db2-25f1-4001-abce-4572f92e534a"}, { Terminator.NumberTerminator, "35cb5529-4f39-4b7c-a937-233f6efe6cdb"}, { new NonTerminator(NonTerminatorType.Variable), "ca1b9fbe-0a27-4b7d-ad6a-ee1ea8f20f5f"}, { new Terminator(DelimiterType.LeftParenthesis), "5f4ba076-b3cf-4472-848c-74d2a618eacb"}, { Terminator.IdentifierTerminator, "99071775-1827-45c4-b161-826ababf0384"}, { new Terminator(KeywordType.Not), "ed5ffdaa-b277-4c22-ad93-e4b1ed293ae4"}, { new Terminator(OperatorType.Minus), "15dc2c62-c938-4d0a-b0aa-1f0a0b73661f"},}, new Dictionary<Terminator, ReduceInformation>{ }, "47ecca50-56e9-420a-b41c-162f48e7a850") },
		{ "fffa3768-ea73-45cb-9a77-204e92522278", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.Then), "65e3267b-26dc-4d80-b1d5-b780d630540c"},}, new Dictionary<Terminator, ReduceInformation>{ }, "fffa3768-ea73-45cb-9a77-204e92522278") },
		{ "d0b871cb-b302-4a53-856a-d5bd7f4e7d3c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.RelationOperator), "d6def487-3d19-4582-a2e7-0b2db83c40b7"}, { new Terminator(OperatorType.Equal), "5cba3bbd-2f4e-4072-ab57-6933c47cf686"}, { new Terminator(OperatorType.NotEqual), "f19c1f01-646c-448a-a706-47091a7f6df7"}, { new Terminator(OperatorType.Less), "94b6c574-1915-47b0-b2de-d2c73eb5c1c2"}, { new Terminator(OperatorType.LessEqual), "0f2fec62-57bd-4061-9dff-9bcc1114b3a4"}, { new Terminator(OperatorType.Greater), "14be83c5-2fdf-4057-b5ed-b5bb521a20a2"}, { new Terminator(OperatorType.GreaterEqual), "83ce7741-272c-4749-8008-fcc200294ed8"}, { new NonTerminator(NonTerminatorType.AddOperator), "eaa27b88-ea92-4056-95b3-c02ed7925161"}, { new Terminator(OperatorType.Plus), "2cf28f7e-6888-4e54-9825-8a80b00c7a56"}, { new Terminator(OperatorType.Minus), "1804ef7c-da92-431e-b725-ea7823b69327"}, { new Terminator(KeywordType.Or), "e777b812-5ca1-4317-9fd4-cc37fbcb2af0"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, }, "d0b871cb-b302-4a53-856a-d5bd7f4e7d3c") },
		{ "5fd6c89b-6edd-4f4f-8d72-ce68a8f5d407", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "1d5fd969-7d23-4c9c-a74b-2af81c0d3a86"}, { new Terminator(OperatorType.Multiply), "4d79eeb4-6b1a-4ca0-97f6-1f6675aed09f"}, { new Terminator(OperatorType.Divide), "fecb6832-79a2-4f95-95c4-fc698109ba3e"}, { new Terminator(KeywordType.Divide), "784cd050-c0ae-4844-8706-7f532d156c70"}, { new Terminator(KeywordType.Mod), "f3fdd777-da4f-4c74-9196-b8c4d7b7bace"}, { new Terminator(KeywordType.And), "376ab083-df94-433e-bcac-f300687907b1"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "5fd6c89b-6edd-4f4f-8d72-ce68a8f5d407") },
		{ "67dd75bf-89f3-4ad3-8209-1d3a4d3f3303", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "67dd75bf-89f3-4ad3-8209-1d3a4d3f3303") },
		{ "c90f9bea-bfaa-4211-8ea7-8c89c19751ad", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "c90f9bea-bfaa-4211-8ea7-8c89c19751ad") },
		{ "02430536-65bb-44a6-86f4-f3c102956340", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "02430536-65bb-44a6-86f4-f3c102956340") },
		{ "cd305913-18dd-45cd-aebf-84bb50475ed5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "c61395eb-9627-4cca-a009-4e20649df578"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "550fbb43-0150-468f-9eb4-7bf5912daa0a"}, { new NonTerminator(NonTerminatorType.Term), "f0cad299-28f5-4e84-b9e7-2bc048f33840"}, { new NonTerminator(NonTerminatorType.Factor), "43e04c0c-8568-4c6b-ab2b-bceedcc9fa32"}, { Terminator.NumberTerminator, "c04070de-7d08-4aad-b5d7-41002b95f79d"}, { new NonTerminator(NonTerminatorType.Variable), "448fb14e-b117-467e-94e2-0ffad798acf2"}, { new Terminator(DelimiterType.LeftParenthesis), "61941794-1fbf-4cdd-a20f-059c95559fcc"}, { Terminator.IdentifierTerminator, "e932245d-2b23-4f7c-b6a4-c6fbd2b3dd9c"}, { new Terminator(KeywordType.Not), "8fcb5991-4a57-4bc9-b6ed-c1c6b22af95e"}, { new Terminator(OperatorType.Minus), "7312a539-9334-411e-b2f6-dc420da87c55"},}, new Dictionary<Terminator, ReduceInformation>{ }, "cd305913-18dd-45cd-aebf-84bb50475ed5") },
		{ "f41a8bc1-8fea-4591-a205-dd02472cc9e6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "14badc00-6719-4664-a41a-6bdfb7683a0b"}, { new NonTerminator(NonTerminatorType.IdVarPart), "f7a50f01-c4e9-4521-b49c-e970268149ec"}, { new Terminator(DelimiterType.LeftSquareBracket), "737cc1da-f2b5-43d9-b245-3708f879bd29"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "f41a8bc1-8fea-4591-a205-dd02472cc9e6") },
		{ "21d6e202-9092-4593-a903-5b7ce59a3a44", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "364e9c33-c4f7-4ee5-a31f-c3a2df0de0b1"}, { Terminator.NumberTerminator, "c90f9bea-bfaa-4211-8ea7-8c89c19751ad"}, { new NonTerminator(NonTerminatorType.Variable), "02430536-65bb-44a6-86f4-f3c102956340"}, { new Terminator(DelimiterType.LeftParenthesis), "cd305913-18dd-45cd-aebf-84bb50475ed5"}, { Terminator.IdentifierTerminator, "f41a8bc1-8fea-4591-a205-dd02472cc9e6"}, { new Terminator(KeywordType.Not), "21d6e202-9092-4593-a903-5b7ce59a3a44"}, { new Terminator(OperatorType.Minus), "d2f171ed-e2fb-4f7d-af92-d427c2f599b9"},}, new Dictionary<Terminator, ReduceInformation>{ }, "21d6e202-9092-4593-a903-5b7ce59a3a44") },
		{ "d2f171ed-e2fb-4f7d-af92-d427c2f599b9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "75371fa8-effc-4461-a224-36cbe5093d28"}, { Terminator.NumberTerminator, "c90f9bea-bfaa-4211-8ea7-8c89c19751ad"}, { new NonTerminator(NonTerminatorType.Variable), "02430536-65bb-44a6-86f4-f3c102956340"}, { new Terminator(DelimiterType.LeftParenthesis), "cd305913-18dd-45cd-aebf-84bb50475ed5"}, { Terminator.IdentifierTerminator, "f41a8bc1-8fea-4591-a205-dd02472cc9e6"}, { new Terminator(KeywordType.Not), "21d6e202-9092-4593-a903-5b7ce59a3a44"}, { new Terminator(OperatorType.Minus), "d2f171ed-e2fb-4f7d-af92-d427c2f599b9"},}, new Dictionary<Terminator, ReduceInformation>{ }, "d2f171ed-e2fb-4f7d-af92-d427c2f599b9") },
		{ "ee010803-4c9a-4082-bf8d-d0b5ccfcbbd4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(OperatorType.Assign), "a74fe10c-8da0-4f66-939e-9f21b53de09e"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ee010803-4c9a-4082-bf8d-d0b5ccfcbbd4") },
		{ "2c52c119-5fc7-40bc-b02a-516ec9121c43", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.End), "48816eb3-4b3a-42d6-bcb2-d5211cd77a49"}, { new Terminator(DelimiterType.Semicolon), "655817d0-6fda-4a1b-a6fb-1075ad52b093"},}, new Dictionary<Terminator, ReduceInformation>{ }, "2c52c119-5fc7-40bc-b02a-516ec9121c43") },
		{ "7bd2877e-0f4d-42ba-a9c9-0f76a5732c4b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Subprogram))}, }, "7bd2877e-0f4d-42ba-a9c9-0f76a5732c4b") },
		{ "0bd495cf-e0a2-444d-b5af-322022d1310e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.VarDeclarations), "e09f806e-4867-4364-ba85-7ba6a15703de"}, { new Terminator(KeywordType.Var), "ba832b24-5e7e-47e4-843b-0a78d71ff1d7"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(0, new NonTerminator(NonTerminatorType.VarDeclarations))}, }, "0bd495cf-e0a2-444d-b5af-322022d1310e") },
		{ "13157f83-6b01-4a74-b70d-a374f4cc294b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ConstDeclaration), "4ff54eba-edd7-4de7-953b-fa790a51c969"}, { Terminator.IdentifierTerminator, "ea240188-f7bf-4060-a07a-964690763117"},}, new Dictionary<Terminator, ReduceInformation>{ }, "13157f83-6b01-4a74-b70d-a374f4cc294b") },
		{ "ef2e9739-b0fb-4b0e-89bb-4968a7a31a77", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SubprogramHead))}, }, "ef2e9739-b0fb-4b0e-89bb-4968a7a31a77") },
		{ "e20a85d5-a7a1-428f-8b63-b0a909a3225e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ParameterList), "b25a12da-0f51-4c20-beeb-f482375d1042"}, { new NonTerminator(NonTerminatorType.Parameter), "58803553-901b-4f68-9e76-dbcab2d440d3"}, { new NonTerminator(NonTerminatorType.VarParameter), "3b2120f9-1746-4d68-9805-d552c9073b1a"}, { new NonTerminator(NonTerminatorType.ValueParameter), "88ddcbe8-4835-4b0b-939a-7fe235ac52d9"}, { new Terminator(KeywordType.Var), "05d3e85d-786a-4f42-85e2-dd90338f0594"}, { Terminator.IdentifierTerminator, "8266ea0c-3873-4a6f-b7d3-1cf665e02669"},}, new Dictionary<Terminator, ReduceInformation>{ }, "e20a85d5-a7a1-428f-8b63-b0a909a3225e") },
		{ "ab644005-e44f-4aff-a653-e38bfd5e4260", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Colon), "fa5b2bb8-7fc6-4685-8694-c6c15d1ccadc"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ab644005-e44f-4aff-a653-e38bfd5e4260") },
		{ "e9766404-a352-4a50-83b7-ccca76c02d0c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ParameterList), "e14f87f2-dd7f-4d70-af5c-e0bf1943dd3c"}, { new NonTerminator(NonTerminatorType.Parameter), "58803553-901b-4f68-9e76-dbcab2d440d3"}, { new NonTerminator(NonTerminatorType.VarParameter), "3b2120f9-1746-4d68-9805-d552c9073b1a"}, { new NonTerminator(NonTerminatorType.ValueParameter), "88ddcbe8-4835-4b0b-939a-7fe235ac52d9"}, { new Terminator(KeywordType.Var), "05d3e85d-786a-4f42-85e2-dd90338f0594"}, { Terminator.IdentifierTerminator, "8266ea0c-3873-4a6f-b7d3-1cf665e02669"},}, new Dictionary<Terminator, ReduceInformation>{ }, "e9766404-a352-4a50-83b7-ccca76c02d0c") },
		{ "a7090436-6db2-4c21-b89d-021878aa45d0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(4, new NonTerminator(NonTerminatorType.VarDeclaration))}, }, "a7090436-6db2-4c21-b89d-021878aa45d0") },
		{ "8eec6fa3-a419-4e42-8f12-f700af89224a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdentifierList))}, }, "8eec6fa3-a419-4e42-8f12-f700af89224a") },
		{ "d02e2812-27a6-40b9-8b5d-5894e9b76bda", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Period), "433b1418-c1c7-46b2-92eb-18d1490e4d39"}, { Terminator.NumberTerminator, "9c9213aa-db0c-400f-a84f-f3bf743ec7d9"},}, new Dictionary<Terminator, ReduceInformation>{ }, "d02e2812-27a6-40b9-8b5d-5894e9b76bda") },
		{ "8bfd1368-b2c1-4b35-8fcd-22f65f624d6c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(5, new NonTerminator(NonTerminatorType.ConstDeclaration))}, }, "8bfd1368-b2c1-4b35-8fcd-22f65f624d6c") },
		{ "bdd3a698-2eb3-4ced-91e1-cdb4b542295e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.Of), "5b383f5d-009f-42d9-939d-66ad6f6c98c5"},}, new Dictionary<Terminator, ReduceInformation>{ }, "bdd3a698-2eb3-4ced-91e1-cdb4b542295e") },
		{ "16576908-4ae2-4aea-a555-907676e5b539", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.NumberTerminator, "766b9396-4bd4-47c9-8b45-e4f03b7ef150"},}, new Dictionary<Terminator, ReduceInformation>{ }, "16576908-4ae2-4aea-a555-907676e5b539") },
		{ "0716c4bf-7f7d-4bf0-89f8-63703420822b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.NumberTerminator, "f4baa733-c8b5-4263-a9e5-5391244dd04e"},}, new Dictionary<Terminator, ReduceInformation>{ }, "0716c4bf-7f7d-4bf0-89f8-63703420822b") },
		{ "4e628d3b-279c-4db0-9031-e62e64e72951", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.StatementList))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.StatementList))}, }, "4e628d3b-279c-4db0-9031-e62e64e72951") },
		{ "0ba3aa16-bf5e-4e4e-900c-fe7b5003c129", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Statement))}, }, "0ba3aa16-bf5e-4e4e-900c-fe7b5003c129") },
		{ "5d2880aa-711d-43f9-81e6-7ddac2ae137b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.RelationOperator), "479cedfb-f097-490e-aacd-7a1a246ca335"}, { new Terminator(OperatorType.Equal), "5cba3bbd-2f4e-4072-ab57-6933c47cf686"}, { new Terminator(OperatorType.NotEqual), "f19c1f01-646c-448a-a706-47091a7f6df7"}, { new Terminator(OperatorType.Less), "94b6c574-1915-47b0-b2de-d2c73eb5c1c2"}, { new Terminator(OperatorType.LessEqual), "0f2fec62-57bd-4061-9dff-9bcc1114b3a4"}, { new Terminator(OperatorType.Greater), "14be83c5-2fdf-4057-b5ed-b5bb521a20a2"}, { new Terminator(OperatorType.GreaterEqual), "83ce7741-272c-4749-8008-fcc200294ed8"}, { new NonTerminator(NonTerminatorType.AddOperator), "891b841f-61c1-428b-a933-79d01590d236"}, { new Terminator(OperatorType.Plus), "2cf28f7e-6888-4e54-9825-8a80b00c7a56"}, { new Terminator(OperatorType.Minus), "1804ef7c-da92-431e-b725-ea7823b69327"}, { new Terminator(KeywordType.Or), "e777b812-5ca1-4317-9fd4-cc37fbcb2af0"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, }, "5d2880aa-711d-43f9-81e6-7ddac2ae137b") },
		{ "5b386270-0769-4603-b5d4-38c90f32038c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "045ec7dd-2546-4063-ac79-3c172c197c7d"}, { new Terminator(OperatorType.Multiply), "4d79eeb4-6b1a-4ca0-97f6-1f6675aed09f"}, { new Terminator(OperatorType.Divide), "fecb6832-79a2-4f95-95c4-fc698109ba3e"}, { new Terminator(KeywordType.Divide), "784cd050-c0ae-4844-8706-7f532d156c70"}, { new Terminator(KeywordType.Mod), "f3fdd777-da4f-4c74-9196-b8c4d7b7bace"}, { new Terminator(KeywordType.And), "376ab083-df94-433e-bcac-f300687907b1"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "5b386270-0769-4603-b5d4-38c90f32038c") },
		{ "d0773f8f-3e0b-47b4-b9a5-33b173774588", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "d0773f8f-3e0b-47b4-b9a5-33b173774588") },
		{ "55ee839d-2c75-424b-87bd-a6dcfaed247a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "55ee839d-2c75-424b-87bd-a6dcfaed247a") },
		{ "4dddee46-3cbe-40d8-8c3b-aa0e74d21b01", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "4dddee46-3cbe-40d8-8c3b-aa0e74d21b01") },
		{ "4e680b0b-45e5-4000-8b6e-36205a12432e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "f232ccb1-25da-4e0b-9848-36e1cbff66ae"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "550fbb43-0150-468f-9eb4-7bf5912daa0a"}, { new NonTerminator(NonTerminatorType.Term), "f0cad299-28f5-4e84-b9e7-2bc048f33840"}, { new NonTerminator(NonTerminatorType.Factor), "43e04c0c-8568-4c6b-ab2b-bceedcc9fa32"}, { Terminator.NumberTerminator, "c04070de-7d08-4aad-b5d7-41002b95f79d"}, { new NonTerminator(NonTerminatorType.Variable), "448fb14e-b117-467e-94e2-0ffad798acf2"}, { new Terminator(DelimiterType.LeftParenthesis), "61941794-1fbf-4cdd-a20f-059c95559fcc"}, { Terminator.IdentifierTerminator, "e932245d-2b23-4f7c-b6a4-c6fbd2b3dd9c"}, { new Terminator(KeywordType.Not), "8fcb5991-4a57-4bc9-b6ed-c1c6b22af95e"}, { new Terminator(OperatorType.Minus), "7312a539-9334-411e-b2f6-dc420da87c55"},}, new Dictionary<Terminator, ReduceInformation>{ }, "4e680b0b-45e5-4000-8b6e-36205a12432e") },
		{ "8e321900-a883-4a45-b396-a6f26b6b341c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "f8a93d3b-64cd-4775-b445-2bdd334e0923"}, { new NonTerminator(NonTerminatorType.IdVarPart), "ecfd8596-4873-4c10-afe1-a8ba5072def8"}, { new Terminator(DelimiterType.LeftSquareBracket), "f470eb29-000d-4aaf-a7d9-01db90f6a8b6"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "8e321900-a883-4a45-b396-a6f26b6b341c") },
		{ "0e41064a-e0af-48d7-80d1-e0e263116cfd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "9c6bab2a-59f4-4a01-8316-bb7026d4072f"}, { Terminator.NumberTerminator, "55ee839d-2c75-424b-87bd-a6dcfaed247a"}, { new NonTerminator(NonTerminatorType.Variable), "4dddee46-3cbe-40d8-8c3b-aa0e74d21b01"}, { new Terminator(DelimiterType.LeftParenthesis), "4e680b0b-45e5-4000-8b6e-36205a12432e"}, { Terminator.IdentifierTerminator, "8e321900-a883-4a45-b396-a6f26b6b341c"}, { new Terminator(KeywordType.Not), "0e41064a-e0af-48d7-80d1-e0e263116cfd"}, { new Terminator(OperatorType.Minus), "cc970159-9448-4bfb-9d6d-75aed317fae9"},}, new Dictionary<Terminator, ReduceInformation>{ }, "0e41064a-e0af-48d7-80d1-e0e263116cfd") },
		{ "cc970159-9448-4bfb-9d6d-75aed317fae9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "e474684a-3a11-4edc-8e0d-b345d969e9ab"}, { Terminator.NumberTerminator, "55ee839d-2c75-424b-87bd-a6dcfaed247a"}, { new NonTerminator(NonTerminatorType.Variable), "4dddee46-3cbe-40d8-8c3b-aa0e74d21b01"}, { new Terminator(DelimiterType.LeftParenthesis), "4e680b0b-45e5-4000-8b6e-36205a12432e"}, { Terminator.IdentifierTerminator, "8e321900-a883-4a45-b396-a6f26b6b341c"}, { new Terminator(KeywordType.Not), "0e41064a-e0af-48d7-80d1-e0e263116cfd"}, { new Terminator(OperatorType.Minus), "cc970159-9448-4bfb-9d6d-75aed317fae9"},}, new Dictionary<Terminator, ReduceInformation>{ }, "cc970159-9448-4bfb-9d6d-75aed317fae9") },
		{ "ebfb4524-8918-4fcf-9126-1eb87ad726da", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Statement))}, }, "ebfb4524-8918-4fcf-9126-1eb87ad726da") },
		{ "fe55e52a-c653-4ba5-91b6-868fa82ab021", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "49aa3fa9-4251-4f87-a158-6d8244b168a4"}, { new Terminator(DelimiterType.Comma), "61999b62-48c1-4d86-8277-f2ec86cd3141"},}, new Dictionary<Terminator, ReduceInformation>{ }, "fe55e52a-c653-4ba5-91b6-868fa82ab021") },
		{ "2e88b57e-7c35-4aee-9387-ff1c061cb7ab", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ExpressionList))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ExpressionList))}, }, "2e88b57e-7c35-4aee-9387-ff1c061cb7ab") },
		{ "089240b9-2be9-4199-97a8-41a6a9ac2f54", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.RelationOperator), "673698b9-7bfb-484e-963e-1027ae25cd98"}, { new Terminator(OperatorType.Equal), "5cba3bbd-2f4e-4072-ab57-6933c47cf686"}, { new Terminator(OperatorType.NotEqual), "f19c1f01-646c-448a-a706-47091a7f6df7"}, { new Terminator(OperatorType.Less), "94b6c574-1915-47b0-b2de-d2c73eb5c1c2"}, { new Terminator(OperatorType.LessEqual), "0f2fec62-57bd-4061-9dff-9bcc1114b3a4"}, { new Terminator(OperatorType.Greater), "14be83c5-2fdf-4057-b5ed-b5bb521a20a2"}, { new Terminator(OperatorType.GreaterEqual), "83ce7741-272c-4749-8008-fcc200294ed8"}, { new NonTerminator(NonTerminatorType.AddOperator), "a5520e8b-3b5e-4396-8cc2-b10fa9f00c58"}, { new Terminator(OperatorType.Plus), "2cf28f7e-6888-4e54-9825-8a80b00c7a56"}, { new Terminator(OperatorType.Minus), "1804ef7c-da92-431e-b725-ea7823b69327"}, { new Terminator(KeywordType.Or), "e777b812-5ca1-4317-9fd4-cc37fbcb2af0"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, }, "089240b9-2be9-4199-97a8-41a6a9ac2f54") },
		{ "f46a091e-b991-43bd-877a-3c2eef645ab7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "87a430b5-3e68-44a2-a5ce-31d848e8f156"}, { new Terminator(OperatorType.Multiply), "4d79eeb4-6b1a-4ca0-97f6-1f6675aed09f"}, { new Terminator(OperatorType.Divide), "fecb6832-79a2-4f95-95c4-fc698109ba3e"}, { new Terminator(KeywordType.Divide), "784cd050-c0ae-4844-8706-7f532d156c70"}, { new Terminator(KeywordType.Mod), "f3fdd777-da4f-4c74-9196-b8c4d7b7bace"}, { new Terminator(KeywordType.And), "376ab083-df94-433e-bcac-f300687907b1"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "f46a091e-b991-43bd-877a-3c2eef645ab7") },
		{ "5cb8d05c-bce3-47cc-8728-d7c037f2e9b1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "5cb8d05c-bce3-47cc-8728-d7c037f2e9b1") },
		{ "3928c790-d1f1-4d6a-8e98-6a0d0a3182e7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "3928c790-d1f1-4d6a-8e98-6a0d0a3182e7") },
		{ "08f29b35-75b5-4fe9-b677-ed18c2caa51b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "08f29b35-75b5-4fe9-b677-ed18c2caa51b") },
		{ "7bb9f1fe-edd2-4812-9091-30ae6a61e59d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "99ca754c-e46f-44ad-a8e4-3141f6887122"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "550fbb43-0150-468f-9eb4-7bf5912daa0a"}, { new NonTerminator(NonTerminatorType.Term), "f0cad299-28f5-4e84-b9e7-2bc048f33840"}, { new NonTerminator(NonTerminatorType.Factor), "43e04c0c-8568-4c6b-ab2b-bceedcc9fa32"}, { Terminator.NumberTerminator, "c04070de-7d08-4aad-b5d7-41002b95f79d"}, { new NonTerminator(NonTerminatorType.Variable), "448fb14e-b117-467e-94e2-0ffad798acf2"}, { new Terminator(DelimiterType.LeftParenthesis), "61941794-1fbf-4cdd-a20f-059c95559fcc"}, { Terminator.IdentifierTerminator, "e932245d-2b23-4f7c-b6a4-c6fbd2b3dd9c"}, { new Terminator(KeywordType.Not), "8fcb5991-4a57-4bc9-b6ed-c1c6b22af95e"}, { new Terminator(OperatorType.Minus), "7312a539-9334-411e-b2f6-dc420da87c55"},}, new Dictionary<Terminator, ReduceInformation>{ }, "7bb9f1fe-edd2-4812-9091-30ae6a61e59d") },
		{ "ad126546-60a0-4982-8f68-c9a727b1c223", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "31c99fb4-8248-4b8e-a6cf-a46f1624d6f1"}, { new NonTerminator(NonTerminatorType.IdVarPart), "13bceee6-fc96-44ff-a084-192a2173224c"}, { new Terminator(DelimiterType.LeftSquareBracket), "57965ac8-a7b8-462c-a073-32f12501457f"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "ad126546-60a0-4982-8f68-c9a727b1c223") },
		{ "d17a2e13-31b3-4e22-a0a6-5cf5fa910500", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "ee596998-5c40-4e2d-b302-538b12b60141"}, { Terminator.NumberTerminator, "3928c790-d1f1-4d6a-8e98-6a0d0a3182e7"}, { new NonTerminator(NonTerminatorType.Variable), "08f29b35-75b5-4fe9-b677-ed18c2caa51b"}, { new Terminator(DelimiterType.LeftParenthesis), "7bb9f1fe-edd2-4812-9091-30ae6a61e59d"}, { Terminator.IdentifierTerminator, "ad126546-60a0-4982-8f68-c9a727b1c223"}, { new Terminator(KeywordType.Not), "d17a2e13-31b3-4e22-a0a6-5cf5fa910500"}, { new Terminator(OperatorType.Minus), "e8e4e781-e847-4c03-b1c8-fc377a1a86f7"},}, new Dictionary<Terminator, ReduceInformation>{ }, "d17a2e13-31b3-4e22-a0a6-5cf5fa910500") },
		{ "e8e4e781-e847-4c03-b1c8-fc377a1a86f7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "1e2c05ca-183d-4501-b6ff-a053115ff436"}, { Terminator.NumberTerminator, "3928c790-d1f1-4d6a-8e98-6a0d0a3182e7"}, { new NonTerminator(NonTerminatorType.Variable), "08f29b35-75b5-4fe9-b677-ed18c2caa51b"}, { new Terminator(DelimiterType.LeftParenthesis), "7bb9f1fe-edd2-4812-9091-30ae6a61e59d"}, { Terminator.IdentifierTerminator, "ad126546-60a0-4982-8f68-c9a727b1c223"}, { new Terminator(KeywordType.Not), "d17a2e13-31b3-4e22-a0a6-5cf5fa910500"}, { new Terminator(OperatorType.Minus), "e8e4e781-e847-4c03-b1c8-fc377a1a86f7"},}, new Dictionary<Terminator, ReduceInformation>{ }, "e8e4e781-e847-4c03-b1c8-fc377a1a86f7") },
		{ "cf973399-71d1-445a-8a7e-87ce30786c51", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "99528598-aca5-437f-8146-f0c79a91351a"}, { new Terminator(DelimiterType.Comma), "f8c6158b-c7b5-4520-8a37-49ad141ff150"},}, new Dictionary<Terminator, ReduceInformation>{ }, "cf973399-71d1-445a-8a7e-87ce30786c51") },
		{ "2e7bde52-48a5-420d-a6e3-9f664510da68", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ExpressionList))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ExpressionList))}, }, "2e7bde52-48a5-420d-a6e3-9f664510da68") },
		{ "17428d1e-ed4e-461b-ad85-a5f1d8f006eb", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.RelationOperator), "4fc347f9-6b87-41c5-87a7-dfca73a52291"}, { new Terminator(OperatorType.Equal), "5cba3bbd-2f4e-4072-ab57-6933c47cf686"}, { new Terminator(OperatorType.NotEqual), "f19c1f01-646c-448a-a706-47091a7f6df7"}, { new Terminator(OperatorType.Less), "94b6c574-1915-47b0-b2de-d2c73eb5c1c2"}, { new Terminator(OperatorType.LessEqual), "0f2fec62-57bd-4061-9dff-9bcc1114b3a4"}, { new Terminator(OperatorType.Greater), "14be83c5-2fdf-4057-b5ed-b5bb521a20a2"}, { new Terminator(OperatorType.GreaterEqual), "83ce7741-272c-4749-8008-fcc200294ed8"}, { new NonTerminator(NonTerminatorType.AddOperator), "e672341e-ca36-4d12-9752-c8ed2a1e61a4"}, { new Terminator(OperatorType.Plus), "2cf28f7e-6888-4e54-9825-8a80b00c7a56"}, { new Terminator(OperatorType.Minus), "1804ef7c-da92-431e-b725-ea7823b69327"}, { new Terminator(KeywordType.Or), "e777b812-5ca1-4317-9fd4-cc37fbcb2af0"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, }, "17428d1e-ed4e-461b-ad85-a5f1d8f006eb") },
		{ "bab97185-47af-4ddd-8275-1863e6c37e23", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "c85c13e8-5099-4466-a6ad-914cf3ae1717"}, { new Terminator(OperatorType.Multiply), "4d79eeb4-6b1a-4ca0-97f6-1f6675aed09f"}, { new Terminator(OperatorType.Divide), "fecb6832-79a2-4f95-95c4-fc698109ba3e"}, { new Terminator(KeywordType.Divide), "784cd050-c0ae-4844-8706-7f532d156c70"}, { new Terminator(KeywordType.Mod), "f3fdd777-da4f-4c74-9196-b8c4d7b7bace"}, { new Terminator(KeywordType.And), "376ab083-df94-433e-bcac-f300687907b1"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "bab97185-47af-4ddd-8275-1863e6c37e23") },
		{ "f5d56db2-25f1-4001-abce-4572f92e534a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "f5d56db2-25f1-4001-abce-4572f92e534a") },
		{ "35cb5529-4f39-4b7c-a937-233f6efe6cdb", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "35cb5529-4f39-4b7c-a937-233f6efe6cdb") },
		{ "ca1b9fbe-0a27-4b7d-ad6a-ee1ea8f20f5f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "ca1b9fbe-0a27-4b7d-ad6a-ee1ea8f20f5f") },
		{ "5f4ba076-b3cf-4472-848c-74d2a618eacb", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "111da932-9fcf-452c-b21e-1132eff01cb4"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "550fbb43-0150-468f-9eb4-7bf5912daa0a"}, { new NonTerminator(NonTerminatorType.Term), "f0cad299-28f5-4e84-b9e7-2bc048f33840"}, { new NonTerminator(NonTerminatorType.Factor), "43e04c0c-8568-4c6b-ab2b-bceedcc9fa32"}, { Terminator.NumberTerminator, "c04070de-7d08-4aad-b5d7-41002b95f79d"}, { new NonTerminator(NonTerminatorType.Variable), "448fb14e-b117-467e-94e2-0ffad798acf2"}, { new Terminator(DelimiterType.LeftParenthesis), "61941794-1fbf-4cdd-a20f-059c95559fcc"}, { Terminator.IdentifierTerminator, "e932245d-2b23-4f7c-b6a4-c6fbd2b3dd9c"}, { new Terminator(KeywordType.Not), "8fcb5991-4a57-4bc9-b6ed-c1c6b22af95e"}, { new Terminator(OperatorType.Minus), "7312a539-9334-411e-b2f6-dc420da87c55"},}, new Dictionary<Terminator, ReduceInformation>{ }, "5f4ba076-b3cf-4472-848c-74d2a618eacb") },
		{ "99071775-1827-45c4-b161-826ababf0384", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "e850ac14-0820-4312-bc39-5e3041cae261"}, { new NonTerminator(NonTerminatorType.IdVarPart), "49b90856-ebe3-4970-95a9-fbf68e3ab032"}, { new Terminator(DelimiterType.LeftSquareBracket), "91da5d30-41c8-4ca6-ad3a-e06fc6ebc121"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "99071775-1827-45c4-b161-826ababf0384") },
		{ "ed5ffdaa-b277-4c22-ad93-e4b1ed293ae4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "09ccd708-a6b2-4dea-9328-786bebe811fc"}, { Terminator.NumberTerminator, "35cb5529-4f39-4b7c-a937-233f6efe6cdb"}, { new NonTerminator(NonTerminatorType.Variable), "ca1b9fbe-0a27-4b7d-ad6a-ee1ea8f20f5f"}, { new Terminator(DelimiterType.LeftParenthesis), "5f4ba076-b3cf-4472-848c-74d2a618eacb"}, { Terminator.IdentifierTerminator, "99071775-1827-45c4-b161-826ababf0384"}, { new Terminator(KeywordType.Not), "ed5ffdaa-b277-4c22-ad93-e4b1ed293ae4"}, { new Terminator(OperatorType.Minus), "15dc2c62-c938-4d0a-b0aa-1f0a0b73661f"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ed5ffdaa-b277-4c22-ad93-e4b1ed293ae4") },
		{ "15dc2c62-c938-4d0a-b0aa-1f0a0b73661f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "95d89dbe-bb7e-415d-84fa-d9f4393f2a87"}, { Terminator.NumberTerminator, "35cb5529-4f39-4b7c-a937-233f6efe6cdb"}, { new NonTerminator(NonTerminatorType.Variable), "ca1b9fbe-0a27-4b7d-ad6a-ee1ea8f20f5f"}, { new Terminator(DelimiterType.LeftParenthesis), "5f4ba076-b3cf-4472-848c-74d2a618eacb"}, { Terminator.IdentifierTerminator, "99071775-1827-45c4-b161-826ababf0384"}, { new Terminator(KeywordType.Not), "ed5ffdaa-b277-4c22-ad93-e4b1ed293ae4"}, { new Terminator(OperatorType.Minus), "15dc2c62-c938-4d0a-b0aa-1f0a0b73661f"},}, new Dictionary<Terminator, ReduceInformation>{ }, "15dc2c62-c938-4d0a-b0aa-1f0a0b73661f") },
		{ "65e3267b-26dc-4d80-b1d5-b780d630540c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Statement), "d10b614a-f743-4cc2-82a7-a7a9644cc5b4"}, { new NonTerminator(NonTerminatorType.Variable), "ea208c56-799b-4542-aab9-97041337f8cd"}, { Terminator.IdentifierTerminator, "9a4e95ea-b154-4175-98e3-a2691d4eb20f"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "14f3970e-268e-4251-aa12-eaa8effa762a"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "b1815343-33db-47b4-8679-35c240abe9b0"}, { new Terminator(KeywordType.If), "e3fc592c-2252-423a-8c29-eb6efd3e239e"}, { new Terminator(KeywordType.For), "35582002-f9e8-40c5-b5fe-930d8a63d94a"}, { new Terminator(KeywordType.Begin), "09269d2d-ccde-4852-8c11-5c03f8f1f9f2"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "65e3267b-26dc-4d80-b1d5-b780d630540c") },
		{ "d6def487-3d19-4582-a2e7-0b2db83c40b7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SimpleExpression), "73d52634-0511-41fb-8811-32a583080972"}, { new NonTerminator(NonTerminatorType.Term), "e691605a-fd15-437d-bfab-0ef31722d3c1"}, { new NonTerminator(NonTerminatorType.Factor), "aee43856-25e2-4f03-9e53-df91ae81e154"}, { Terminator.NumberTerminator, "cefd8fd4-7bfb-4296-b351-cf55cdf6df21"}, { new NonTerminator(NonTerminatorType.Variable), "84d3de93-f5f0-4bc3-80c5-34a876c227e9"}, { new Terminator(DelimiterType.LeftParenthesis), "0354f240-08fd-4673-96f5-75f3b0f9a1b8"}, { Terminator.IdentifierTerminator, "432a891c-9cfc-4720-aa49-66c77dc4da69"}, { new Terminator(KeywordType.Not), "79165cf0-6db5-4b11-b390-2e376bdf6296"}, { new Terminator(OperatorType.Minus), "3c0d12a5-5808-4427-ac54-415f251911e7"},}, new Dictionary<Terminator, ReduceInformation>{ }, "d6def487-3d19-4582-a2e7-0b2db83c40b7") },
		{ "5cba3bbd-2f4e-4072-ab57-6933c47cf686", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, }, "5cba3bbd-2f4e-4072-ab57-6933c47cf686") },
		{ "f19c1f01-646c-448a-a706-47091a7f6df7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, }, "f19c1f01-646c-448a-a706-47091a7f6df7") },
		{ "94b6c574-1915-47b0-b2de-d2c73eb5c1c2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, }, "94b6c574-1915-47b0-b2de-d2c73eb5c1c2") },
		{ "0f2fec62-57bd-4061-9dff-9bcc1114b3a4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, }, "0f2fec62-57bd-4061-9dff-9bcc1114b3a4") },
		{ "14be83c5-2fdf-4057-b5ed-b5bb521a20a2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, }, "14be83c5-2fdf-4057-b5ed-b5bb521a20a2") },
		{ "83ce7741-272c-4749-8008-fcc200294ed8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, }, "83ce7741-272c-4749-8008-fcc200294ed8") },
		{ "eaa27b88-ea92-4056-95b3-c02ed7925161", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "c20ba73e-961a-44c4-b277-6be87879b7a9"}, { new NonTerminator(NonTerminatorType.Factor), "67dd75bf-89f3-4ad3-8209-1d3a4d3f3303"}, { Terminator.NumberTerminator, "c90f9bea-bfaa-4211-8ea7-8c89c19751ad"}, { new NonTerminator(NonTerminatorType.Variable), "02430536-65bb-44a6-86f4-f3c102956340"}, { new Terminator(DelimiterType.LeftParenthesis), "cd305913-18dd-45cd-aebf-84bb50475ed5"}, { Terminator.IdentifierTerminator, "f41a8bc1-8fea-4591-a205-dd02472cc9e6"}, { new Terminator(KeywordType.Not), "21d6e202-9092-4593-a903-5b7ce59a3a44"}, { new Terminator(OperatorType.Minus), "d2f171ed-e2fb-4f7d-af92-d427c2f599b9"},}, new Dictionary<Terminator, ReduceInformation>{ }, "eaa27b88-ea92-4056-95b3-c02ed7925161") },
		{ "2cf28f7e-6888-4e54-9825-8a80b00c7a56", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, }, "2cf28f7e-6888-4e54-9825-8a80b00c7a56") },
		{ "1804ef7c-da92-431e-b725-ea7823b69327", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, }, "1804ef7c-da92-431e-b725-ea7823b69327") },
		{ "e777b812-5ca1-4317-9fd4-cc37fbcb2af0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, }, "e777b812-5ca1-4317-9fd4-cc37fbcb2af0") },
		{ "1d5fd969-7d23-4c9c-a74b-2af81c0d3a86", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "9138a41a-6056-4826-b2ca-2ced6f7e2f94"}, { Terminator.NumberTerminator, "c90f9bea-bfaa-4211-8ea7-8c89c19751ad"}, { new NonTerminator(NonTerminatorType.Variable), "02430536-65bb-44a6-86f4-f3c102956340"}, { new Terminator(DelimiterType.LeftParenthesis), "cd305913-18dd-45cd-aebf-84bb50475ed5"}, { Terminator.IdentifierTerminator, "f41a8bc1-8fea-4591-a205-dd02472cc9e6"}, { new Terminator(KeywordType.Not), "21d6e202-9092-4593-a903-5b7ce59a3a44"}, { new Terminator(OperatorType.Minus), "d2f171ed-e2fb-4f7d-af92-d427c2f599b9"},}, new Dictionary<Terminator, ReduceInformation>{ }, "1d5fd969-7d23-4c9c-a74b-2af81c0d3a86") },
		{ "4d79eeb4-6b1a-4ca0-97f6-1f6675aed09f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, }, "4d79eeb4-6b1a-4ca0-97f6-1f6675aed09f") },
		{ "fecb6832-79a2-4f95-95c4-fc698109ba3e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, }, "fecb6832-79a2-4f95-95c4-fc698109ba3e") },
		{ "784cd050-c0ae-4844-8706-7f532d156c70", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, }, "784cd050-c0ae-4844-8706-7f532d156c70") },
		{ "f3fdd777-da4f-4c74-9196-b8c4d7b7bace", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, }, "f3fdd777-da4f-4c74-9196-b8c4d7b7bace") },
		{ "376ab083-df94-433e-bcac-f300687907b1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, }, "376ab083-df94-433e-bcac-f300687907b1") },
		{ "c61395eb-9627-4cca-a009-4e20649df578", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "58752d46-753f-4ed1-8888-64534c80f38b"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c61395eb-9627-4cca-a009-4e20649df578") },
		{ "550fbb43-0150-468f-9eb4-7bf5912daa0a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.RelationOperator), "0f91d141-6a65-4d4f-adf6-861609564784"}, { new Terminator(OperatorType.Equal), "5cba3bbd-2f4e-4072-ab57-6933c47cf686"}, { new Terminator(OperatorType.NotEqual), "f19c1f01-646c-448a-a706-47091a7f6df7"}, { new Terminator(OperatorType.Less), "94b6c574-1915-47b0-b2de-d2c73eb5c1c2"}, { new Terminator(OperatorType.LessEqual), "0f2fec62-57bd-4061-9dff-9bcc1114b3a4"}, { new Terminator(OperatorType.Greater), "14be83c5-2fdf-4057-b5ed-b5bb521a20a2"}, { new Terminator(OperatorType.GreaterEqual), "83ce7741-272c-4749-8008-fcc200294ed8"}, { new NonTerminator(NonTerminatorType.AddOperator), "73f8d40c-6684-45d4-8b16-c7c1ec6369cb"}, { new Terminator(OperatorType.Plus), "2cf28f7e-6888-4e54-9825-8a80b00c7a56"}, { new Terminator(OperatorType.Minus), "1804ef7c-da92-431e-b725-ea7823b69327"}, { new Terminator(KeywordType.Or), "e777b812-5ca1-4317-9fd4-cc37fbcb2af0"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, }, "550fbb43-0150-468f-9eb4-7bf5912daa0a") },
		{ "f0cad299-28f5-4e84-b9e7-2bc048f33840", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "2bdea77e-b2e7-4243-a30d-f025bdb7cc68"}, { new Terminator(OperatorType.Multiply), "4d79eeb4-6b1a-4ca0-97f6-1f6675aed09f"}, { new Terminator(OperatorType.Divide), "fecb6832-79a2-4f95-95c4-fc698109ba3e"}, { new Terminator(KeywordType.Divide), "784cd050-c0ae-4844-8706-7f532d156c70"}, { new Terminator(KeywordType.Mod), "f3fdd777-da4f-4c74-9196-b8c4d7b7bace"}, { new Terminator(KeywordType.And), "376ab083-df94-433e-bcac-f300687907b1"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "f0cad299-28f5-4e84-b9e7-2bc048f33840") },
		{ "43e04c0c-8568-4c6b-ab2b-bceedcc9fa32", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "43e04c0c-8568-4c6b-ab2b-bceedcc9fa32") },
		{ "c04070de-7d08-4aad-b5d7-41002b95f79d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "c04070de-7d08-4aad-b5d7-41002b95f79d") },
		{ "448fb14e-b117-467e-94e2-0ffad798acf2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "448fb14e-b117-467e-94e2-0ffad798acf2") },
		{ "61941794-1fbf-4cdd-a20f-059c95559fcc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "6fab3ce2-edfd-4ece-8e52-de80029ea48d"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "550fbb43-0150-468f-9eb4-7bf5912daa0a"}, { new NonTerminator(NonTerminatorType.Term), "f0cad299-28f5-4e84-b9e7-2bc048f33840"}, { new NonTerminator(NonTerminatorType.Factor), "43e04c0c-8568-4c6b-ab2b-bceedcc9fa32"}, { Terminator.NumberTerminator, "c04070de-7d08-4aad-b5d7-41002b95f79d"}, { new NonTerminator(NonTerminatorType.Variable), "448fb14e-b117-467e-94e2-0ffad798acf2"}, { new Terminator(DelimiterType.LeftParenthesis), "61941794-1fbf-4cdd-a20f-059c95559fcc"}, { Terminator.IdentifierTerminator, "e932245d-2b23-4f7c-b6a4-c6fbd2b3dd9c"}, { new Terminator(KeywordType.Not), "8fcb5991-4a57-4bc9-b6ed-c1c6b22af95e"}, { new Terminator(OperatorType.Minus), "7312a539-9334-411e-b2f6-dc420da87c55"},}, new Dictionary<Terminator, ReduceInformation>{ }, "61941794-1fbf-4cdd-a20f-059c95559fcc") },
		{ "e932245d-2b23-4f7c-b6a4-c6fbd2b3dd9c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "d7246cda-d2bf-455b-9181-3d1f813ecda6"}, { new NonTerminator(NonTerminatorType.IdVarPart), "c30cf66b-9bde-48f0-9552-b7de88707d26"}, { new Terminator(DelimiterType.LeftSquareBracket), "38aa5d26-7153-444a-a3fe-88df3e94fa95"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "e932245d-2b23-4f7c-b6a4-c6fbd2b3dd9c") },
		{ "8fcb5991-4a57-4bc9-b6ed-c1c6b22af95e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "13f05a96-f077-4d17-a345-6f7655720b85"}, { Terminator.NumberTerminator, "c04070de-7d08-4aad-b5d7-41002b95f79d"}, { new NonTerminator(NonTerminatorType.Variable), "448fb14e-b117-467e-94e2-0ffad798acf2"}, { new Terminator(DelimiterType.LeftParenthesis), "61941794-1fbf-4cdd-a20f-059c95559fcc"}, { Terminator.IdentifierTerminator, "e932245d-2b23-4f7c-b6a4-c6fbd2b3dd9c"}, { new Terminator(KeywordType.Not), "8fcb5991-4a57-4bc9-b6ed-c1c6b22af95e"}, { new Terminator(OperatorType.Minus), "7312a539-9334-411e-b2f6-dc420da87c55"},}, new Dictionary<Terminator, ReduceInformation>{ }, "8fcb5991-4a57-4bc9-b6ed-c1c6b22af95e") },
		{ "7312a539-9334-411e-b2f6-dc420da87c55", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "35fafaa1-3a99-4617-9b3c-f59f02684b97"}, { Terminator.NumberTerminator, "c04070de-7d08-4aad-b5d7-41002b95f79d"}, { new NonTerminator(NonTerminatorType.Variable), "448fb14e-b117-467e-94e2-0ffad798acf2"}, { new Terminator(DelimiterType.LeftParenthesis), "61941794-1fbf-4cdd-a20f-059c95559fcc"}, { Terminator.IdentifierTerminator, "e932245d-2b23-4f7c-b6a4-c6fbd2b3dd9c"}, { new Terminator(KeywordType.Not), "8fcb5991-4a57-4bc9-b6ed-c1c6b22af95e"}, { new Terminator(OperatorType.Minus), "7312a539-9334-411e-b2f6-dc420da87c55"},}, new Dictionary<Terminator, ReduceInformation>{ }, "7312a539-9334-411e-b2f6-dc420da87c55") },
		{ "14badc00-6719-4664-a41a-6bdfb7683a0b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "e4efaa23-4566-447a-900b-bcdd6b1ba20b"}, { new NonTerminator(NonTerminatorType.Expression), "2e7bde52-48a5-420d-a6e3-9f664510da68"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "17428d1e-ed4e-461b-ad85-a5f1d8f006eb"}, { new NonTerminator(NonTerminatorType.Term), "bab97185-47af-4ddd-8275-1863e6c37e23"}, { new NonTerminator(NonTerminatorType.Factor), "f5d56db2-25f1-4001-abce-4572f92e534a"}, { Terminator.NumberTerminator, "35cb5529-4f39-4b7c-a937-233f6efe6cdb"}, { new NonTerminator(NonTerminatorType.Variable), "ca1b9fbe-0a27-4b7d-ad6a-ee1ea8f20f5f"}, { new Terminator(DelimiterType.LeftParenthesis), "5f4ba076-b3cf-4472-848c-74d2a618eacb"}, { Terminator.IdentifierTerminator, "99071775-1827-45c4-b161-826ababf0384"}, { new Terminator(KeywordType.Not), "ed5ffdaa-b277-4c22-ad93-e4b1ed293ae4"}, { new Terminator(OperatorType.Minus), "15dc2c62-c938-4d0a-b0aa-1f0a0b73661f"},}, new Dictionary<Terminator, ReduceInformation>{ }, "14badc00-6719-4664-a41a-6bdfb7683a0b") },
		{ "f7a50f01-c4e9-4521-b49c-e970268149ec", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "f7a50f01-c4e9-4521-b49c-e970268149ec") },
		{ "737cc1da-f2b5-43d9-b245-3708f879bd29", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "0afbf405-4147-4ca1-a593-fcc5c74c14d0"}, { new NonTerminator(NonTerminatorType.Expression), "2e88b57e-7c35-4aee-9387-ff1c061cb7ab"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "089240b9-2be9-4199-97a8-41a6a9ac2f54"}, { new NonTerminator(NonTerminatorType.Term), "f46a091e-b991-43bd-877a-3c2eef645ab7"}, { new NonTerminator(NonTerminatorType.Factor), "5cb8d05c-bce3-47cc-8728-d7c037f2e9b1"}, { Terminator.NumberTerminator, "3928c790-d1f1-4d6a-8e98-6a0d0a3182e7"}, { new NonTerminator(NonTerminatorType.Variable), "08f29b35-75b5-4fe9-b677-ed18c2caa51b"}, { new Terminator(DelimiterType.LeftParenthesis), "7bb9f1fe-edd2-4812-9091-30ae6a61e59d"}, { Terminator.IdentifierTerminator, "ad126546-60a0-4982-8f68-c9a727b1c223"}, { new Terminator(KeywordType.Not), "d17a2e13-31b3-4e22-a0a6-5cf5fa910500"}, { new Terminator(OperatorType.Minus), "e8e4e781-e847-4c03-b1c8-fc377a1a86f7"},}, new Dictionary<Terminator, ReduceInformation>{ }, "737cc1da-f2b5-43d9-b245-3708f879bd29") },
		{ "364e9c33-c4f7-4ee5-a31f-c3a2df0de0b1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "364e9c33-c4f7-4ee5-a31f-c3a2df0de0b1") },
		{ "75371fa8-effc-4461-a224-36cbe5093d28", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "75371fa8-effc-4461-a224-36cbe5093d28") },
		{ "a74fe10c-8da0-4f66-939e-9f21b53de09e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "93ebf02a-d2a4-421f-a682-586f1545c447"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "3e55bf24-5789-49ed-a5ae-5eb7ec7347cd"}, { new NonTerminator(NonTerminatorType.Term), "08443152-f969-4da6-aa5b-97e3b3345fc9"}, { new NonTerminator(NonTerminatorType.Factor), "c901372b-e50e-432c-ac38-9f6def8240bb"}, { Terminator.NumberTerminator, "d2a4c365-b1fc-48c2-aba8-dcd35bc4a13f"}, { new NonTerminator(NonTerminatorType.Variable), "0cb8880d-7fd6-442b-97b7-5939869faac7"}, { new Terminator(DelimiterType.LeftParenthesis), "571d117e-bca5-4cec-a8aa-99b81b821045"}, { Terminator.IdentifierTerminator, "6f5f06f1-5c16-4c28-aa0b-a254c725d359"}, { new Terminator(KeywordType.Not), "c90dca67-28e6-41ad-b475-ab75f98bff0f"}, { new Terminator(OperatorType.Minus), "48762e8c-76c8-477d-8b60-3bb834e285b3"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a74fe10c-8da0-4f66-939e-9f21b53de09e") },
		{ "48816eb3-4b3a-42d6-bcb2-d5211cd77a49", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.CompoundStatement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.CompoundStatement))}, }, "48816eb3-4b3a-42d6-bcb2-d5211cd77a49") },
		{ "e09f806e-4867-4364-ba85-7ba6a15703de", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.CompoundStatement), "9466bb5e-b5e9-435e-a93a-c6f73e1303da"}, { new Terminator(KeywordType.Begin), "1a0ae9ce-0fa3-4795-b20b-26ac2b51461c"},}, new Dictionary<Terminator, ReduceInformation>{ }, "e09f806e-4867-4364-ba85-7ba6a15703de") },
		{ "ba832b24-5e7e-47e4-843b-0a78d71ff1d7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.VarDeclaration), "5ca40820-c830-4dd8-8c3b-faa9f33be727"}, { Terminator.IdentifierTerminator, "a4a4eb25-4859-4776-ad08-ce9a255e3e47"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ba832b24-5e7e-47e4-843b-0a78d71ff1d7") },
		{ "4ff54eba-edd7-4de7-953b-fa790a51c969", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Semicolon), "b8c76ca4-ca81-46e6-a697-db2be07a0306"},}, new Dictionary<Terminator, ReduceInformation>{ }, "4ff54eba-edd7-4de7-953b-fa790a51c969") },
		{ "b25a12da-0f51-4c20-beeb-f482375d1042", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "d2f0477a-42ad-4435-81e4-4acd7ae12d76"}, { new Terminator(DelimiterType.Semicolon), "ac72cf4d-3f76-4b5b-a7b5-b7883396427c"},}, new Dictionary<Terminator, ReduceInformation>{ }, "b25a12da-0f51-4c20-beeb-f482375d1042") },
		{ "58803553-901b-4f68-9e76-dbcab2d440d3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ParameterList))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ParameterList))}, }, "58803553-901b-4f68-9e76-dbcab2d440d3") },
		{ "3b2120f9-1746-4d68-9805-d552c9073b1a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Parameter))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Parameter))}, }, "3b2120f9-1746-4d68-9805-d552c9073b1a") },
		{ "88ddcbe8-4835-4b0b-939a-7fe235ac52d9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Parameter))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Parameter))}, }, "88ddcbe8-4835-4b0b-939a-7fe235ac52d9") },
		{ "05d3e85d-786a-4f42-85e2-dd90338f0594", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ValueParameter), "a7a4f3f8-03d0-4b27-bec4-2b6a42f2da67"}, { Terminator.IdentifierTerminator, "8266ea0c-3873-4a6f-b7d3-1cf665e02669"},}, new Dictionary<Terminator, ReduceInformation>{ }, "05d3e85d-786a-4f42-85e2-dd90338f0594") },
		{ "8266ea0c-3873-4a6f-b7d3-1cf665e02669", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.IdentifierList), "57d2c113-bfeb-43f5-9e37-00ef947387c5"}, { new Terminator(DelimiterType.Comma), "59ea175e-51e1-4f0e-a58a-dba5258c48a3"}, { new Terminator(DelimiterType.Colon), "63f9e1a5-607e-4602-90bc-9c1d20cbc841"},}, new Dictionary<Terminator, ReduceInformation>{ }, "8266ea0c-3873-4a6f-b7d3-1cf665e02669") },
		{ "fa5b2bb8-7fc6-4685-8694-c6c15d1ccadc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.BasicType), "52e16e08-cdf4-412f-9656-f1f051afdfbe"}, { new Terminator(KeywordType.Integer), "ebe37bec-c209-4195-a140-f57d3f2763ea"}, { new Terminator(KeywordType.Real), "f3352fc3-8224-4718-b0cc-5afcdbcea304"}, { new Terminator(KeywordType.Boolean), "7942b503-91a1-408d-bdf4-f3f76c9a33a0"}, { new Terminator(KeywordType.Character), "ad9d0031-e9ad-485c-bbee-619e17a3a472"},}, new Dictionary<Terminator, ReduceInformation>{ }, "fa5b2bb8-7fc6-4685-8694-c6c15d1ccadc") },
		{ "e14f87f2-dd7f-4d70-af5c-e0bf1943dd3c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "a53abbcc-dead-4d62-9ee6-7dfa0ce96e2f"}, { new Terminator(DelimiterType.Semicolon), "ac72cf4d-3f76-4b5b-a7b5-b7883396427c"},}, new Dictionary<Terminator, ReduceInformation>{ }, "e14f87f2-dd7f-4d70-af5c-e0bf1943dd3c") },
		{ "433b1418-c1c7-46b2-92eb-18d1490e4d39", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "aed33747-2903-44e4-b7dc-d6c3da4637af"}, { new Terminator(DelimiterType.Comma), "16576908-4ae2-4aea-a555-907676e5b539"},}, new Dictionary<Terminator, ReduceInformation>{ }, "433b1418-c1c7-46b2-92eb-18d1490e4d39") },
		{ "5b383f5d-009f-42d9-939d-66ad6f6c98c5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.BasicType), "05594cd9-2dec-4ee8-bddd-e67bb8ce693f"}, { new Terminator(KeywordType.Integer), "b94dae85-8e9c-4115-97f4-34e052bde163"}, { new Terminator(KeywordType.Real), "3052501b-2122-4544-80d8-457e242a600c"}, { new Terminator(KeywordType.Boolean), "a0856ce5-e6fb-4370-9190-af36caf2f75f"}, { new Terminator(KeywordType.Character), "cf014b4e-1286-43ca-8fc2-abe48e6dbfe7"},}, new Dictionary<Terminator, ReduceInformation>{ }, "5b383f5d-009f-42d9-939d-66ad6f6c98c5") },
		{ "766b9396-4bd4-47c9-8b45-e4f03b7ef150", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.DoubleDots), "11ab4735-af5c-4c5f-a303-84f519fdb27f"},}, new Dictionary<Terminator, ReduceInformation>{ }, "766b9396-4bd4-47c9-8b45-e4f03b7ef150") },
		{ "f4baa733-c8b5-4263-a9e5-5391244dd04e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Period))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Period))}, }, "f4baa733-c8b5-4263-a9e5-5391244dd04e") },
		{ "479cedfb-f097-490e-aacd-7a1a246ca335", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SimpleExpression), "210f20bb-d846-4551-9755-3cb2da04ddac"}, { new NonTerminator(NonTerminatorType.Term), "62602618-d5b3-48b8-98fd-c7800d4b080c"}, { new NonTerminator(NonTerminatorType.Factor), "c3d60e4d-5c66-4705-a482-2058ad1fedc1"}, { Terminator.NumberTerminator, "69de6cb0-b810-436f-8aa1-7f9cd7ffdc47"}, { new NonTerminator(NonTerminatorType.Variable), "f8d34924-6053-411a-bcf7-fbd2f2c5a5d0"}, { new Terminator(DelimiterType.LeftParenthesis), "14c29ffd-630e-465b-953a-59d607e06a6a"}, { Terminator.IdentifierTerminator, "e11b9910-5816-47fc-9455-7015c3ffd940"}, { new Terminator(KeywordType.Not), "33da2634-9ccd-439e-9316-87806b898b4c"}, { new Terminator(OperatorType.Minus), "d4dd0bca-2671-4abf-8dab-793593072da6"},}, new Dictionary<Terminator, ReduceInformation>{ }, "479cedfb-f097-490e-aacd-7a1a246ca335") },
		{ "891b841f-61c1-428b-a933-79d01590d236", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "6cf85861-3b32-41ad-982d-ccbb00c8a40c"}, { new NonTerminator(NonTerminatorType.Factor), "d0773f8f-3e0b-47b4-b9a5-33b173774588"}, { Terminator.NumberTerminator, "55ee839d-2c75-424b-87bd-a6dcfaed247a"}, { new NonTerminator(NonTerminatorType.Variable), "4dddee46-3cbe-40d8-8c3b-aa0e74d21b01"}, { new Terminator(DelimiterType.LeftParenthesis), "4e680b0b-45e5-4000-8b6e-36205a12432e"}, { Terminator.IdentifierTerminator, "8e321900-a883-4a45-b396-a6f26b6b341c"}, { new Terminator(KeywordType.Not), "0e41064a-e0af-48d7-80d1-e0e263116cfd"}, { new Terminator(OperatorType.Minus), "cc970159-9448-4bfb-9d6d-75aed317fae9"},}, new Dictionary<Terminator, ReduceInformation>{ }, "891b841f-61c1-428b-a933-79d01590d236") },
		{ "045ec7dd-2546-4063-ac79-3c172c197c7d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "e5b2b50c-7474-4328-b827-d0c25cffd3ba"}, { Terminator.NumberTerminator, "55ee839d-2c75-424b-87bd-a6dcfaed247a"}, { new NonTerminator(NonTerminatorType.Variable), "4dddee46-3cbe-40d8-8c3b-aa0e74d21b01"}, { new Terminator(DelimiterType.LeftParenthesis), "4e680b0b-45e5-4000-8b6e-36205a12432e"}, { Terminator.IdentifierTerminator, "8e321900-a883-4a45-b396-a6f26b6b341c"}, { new Terminator(KeywordType.Not), "0e41064a-e0af-48d7-80d1-e0e263116cfd"}, { new Terminator(OperatorType.Minus), "cc970159-9448-4bfb-9d6d-75aed317fae9"},}, new Dictionary<Terminator, ReduceInformation>{ }, "045ec7dd-2546-4063-ac79-3c172c197c7d") },
		{ "f232ccb1-25da-4e0b-9848-36e1cbff66ae", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "48ff5b3e-da68-4a4f-a5ed-472350ac8b56"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f232ccb1-25da-4e0b-9848-36e1cbff66ae") },
		{ "f8a93d3b-64cd-4775-b445-2bdd334e0923", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "2d8a36df-a7df-4c0f-9ac4-701ecbcb325b"}, { new NonTerminator(NonTerminatorType.Expression), "2e7bde52-48a5-420d-a6e3-9f664510da68"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "17428d1e-ed4e-461b-ad85-a5f1d8f006eb"}, { new NonTerminator(NonTerminatorType.Term), "bab97185-47af-4ddd-8275-1863e6c37e23"}, { new NonTerminator(NonTerminatorType.Factor), "f5d56db2-25f1-4001-abce-4572f92e534a"}, { Terminator.NumberTerminator, "35cb5529-4f39-4b7c-a937-233f6efe6cdb"}, { new NonTerminator(NonTerminatorType.Variable), "ca1b9fbe-0a27-4b7d-ad6a-ee1ea8f20f5f"}, { new Terminator(DelimiterType.LeftParenthesis), "5f4ba076-b3cf-4472-848c-74d2a618eacb"}, { Terminator.IdentifierTerminator, "99071775-1827-45c4-b161-826ababf0384"}, { new Terminator(KeywordType.Not), "ed5ffdaa-b277-4c22-ad93-e4b1ed293ae4"}, { new Terminator(OperatorType.Minus), "15dc2c62-c938-4d0a-b0aa-1f0a0b73661f"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f8a93d3b-64cd-4775-b445-2bdd334e0923") },
		{ "ecfd8596-4873-4c10-afe1-a8ba5072def8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "ecfd8596-4873-4c10-afe1-a8ba5072def8") },
		{ "f470eb29-000d-4aaf-a7d9-01db90f6a8b6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "ca1fbe72-6394-4195-9520-b7e6063d190e"}, { new NonTerminator(NonTerminatorType.Expression), "2e88b57e-7c35-4aee-9387-ff1c061cb7ab"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "089240b9-2be9-4199-97a8-41a6a9ac2f54"}, { new NonTerminator(NonTerminatorType.Term), "f46a091e-b991-43bd-877a-3c2eef645ab7"}, { new NonTerminator(NonTerminatorType.Factor), "5cb8d05c-bce3-47cc-8728-d7c037f2e9b1"}, { Terminator.NumberTerminator, "3928c790-d1f1-4d6a-8e98-6a0d0a3182e7"}, { new NonTerminator(NonTerminatorType.Variable), "08f29b35-75b5-4fe9-b677-ed18c2caa51b"}, { new Terminator(DelimiterType.LeftParenthesis), "7bb9f1fe-edd2-4812-9091-30ae6a61e59d"}, { Terminator.IdentifierTerminator, "ad126546-60a0-4982-8f68-c9a727b1c223"}, { new Terminator(KeywordType.Not), "d17a2e13-31b3-4e22-a0a6-5cf5fa910500"}, { new Terminator(OperatorType.Minus), "e8e4e781-e847-4c03-b1c8-fc377a1a86f7"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f470eb29-000d-4aaf-a7d9-01db90f6a8b6") },
		{ "9c6bab2a-59f4-4a01-8316-bb7026d4072f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "9c6bab2a-59f4-4a01-8316-bb7026d4072f") },
		{ "e474684a-3a11-4edc-8e0d-b345d969e9ab", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "e474684a-3a11-4edc-8e0d-b345d969e9ab") },
		{ "49aa3fa9-4251-4f87-a158-6d8244b168a4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(OperatorType.Assign), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "49aa3fa9-4251-4f87-a158-6d8244b168a4") },
		{ "61999b62-48c1-4d86-8277-f2ec86cd3141", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "557bddf8-f28b-4b8e-ab05-3eb1d60d62f4"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "089240b9-2be9-4199-97a8-41a6a9ac2f54"}, { new NonTerminator(NonTerminatorType.Term), "f46a091e-b991-43bd-877a-3c2eef645ab7"}, { new NonTerminator(NonTerminatorType.Factor), "5cb8d05c-bce3-47cc-8728-d7c037f2e9b1"}, { Terminator.NumberTerminator, "3928c790-d1f1-4d6a-8e98-6a0d0a3182e7"}, { new NonTerminator(NonTerminatorType.Variable), "08f29b35-75b5-4fe9-b677-ed18c2caa51b"}, { new Terminator(DelimiterType.LeftParenthesis), "7bb9f1fe-edd2-4812-9091-30ae6a61e59d"}, { Terminator.IdentifierTerminator, "ad126546-60a0-4982-8f68-c9a727b1c223"}, { new Terminator(KeywordType.Not), "d17a2e13-31b3-4e22-a0a6-5cf5fa910500"}, { new Terminator(OperatorType.Minus), "e8e4e781-e847-4c03-b1c8-fc377a1a86f7"},}, new Dictionary<Terminator, ReduceInformation>{ }, "61999b62-48c1-4d86-8277-f2ec86cd3141") },
		{ "673698b9-7bfb-484e-963e-1027ae25cd98", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SimpleExpression), "8144becc-3e46-43b9-9333-9a20fe517e94"}, { new NonTerminator(NonTerminatorType.Term), "06691bf4-bc4e-43f7-93b8-c56af8b73219"}, { new NonTerminator(NonTerminatorType.Factor), "e6af18a0-1b24-4fe5-b501-7bf4ba96defd"}, { Terminator.NumberTerminator, "ba493efc-5d7d-425a-ab17-3057add7633e"}, { new NonTerminator(NonTerminatorType.Variable), "e937950c-aa65-4d42-97a8-6d55502689bf"}, { new Terminator(DelimiterType.LeftParenthesis), "e3448ecf-4866-44ba-a34b-af6d031ad38d"}, { Terminator.IdentifierTerminator, "685e6fcf-000d-47c8-aee8-cc1d8fee08c3"}, { new Terminator(KeywordType.Not), "814b64e0-3336-426d-8775-15f9eddce24e"}, { new Terminator(OperatorType.Minus), "910b40b6-c2d2-4c5a-8567-9da1ff7eee9f"},}, new Dictionary<Terminator, ReduceInformation>{ }, "673698b9-7bfb-484e-963e-1027ae25cd98") },
		{ "a5520e8b-3b5e-4396-8cc2-b10fa9f00c58", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "644e4ee9-99a3-4975-a4cc-5bca42aa4242"}, { new NonTerminator(NonTerminatorType.Factor), "5cb8d05c-bce3-47cc-8728-d7c037f2e9b1"}, { Terminator.NumberTerminator, "3928c790-d1f1-4d6a-8e98-6a0d0a3182e7"}, { new NonTerminator(NonTerminatorType.Variable), "08f29b35-75b5-4fe9-b677-ed18c2caa51b"}, { new Terminator(DelimiterType.LeftParenthesis), "7bb9f1fe-edd2-4812-9091-30ae6a61e59d"}, { Terminator.IdentifierTerminator, "ad126546-60a0-4982-8f68-c9a727b1c223"}, { new Terminator(KeywordType.Not), "d17a2e13-31b3-4e22-a0a6-5cf5fa910500"}, { new Terminator(OperatorType.Minus), "e8e4e781-e847-4c03-b1c8-fc377a1a86f7"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a5520e8b-3b5e-4396-8cc2-b10fa9f00c58") },
		{ "87a430b5-3e68-44a2-a5ce-31d848e8f156", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "2d852909-7739-4f52-ad12-81dd511cfbad"}, { Terminator.NumberTerminator, "3928c790-d1f1-4d6a-8e98-6a0d0a3182e7"}, { new NonTerminator(NonTerminatorType.Variable), "08f29b35-75b5-4fe9-b677-ed18c2caa51b"}, { new Terminator(DelimiterType.LeftParenthesis), "7bb9f1fe-edd2-4812-9091-30ae6a61e59d"}, { Terminator.IdentifierTerminator, "ad126546-60a0-4982-8f68-c9a727b1c223"}, { new Terminator(KeywordType.Not), "d17a2e13-31b3-4e22-a0a6-5cf5fa910500"}, { new Terminator(OperatorType.Minus), "e8e4e781-e847-4c03-b1c8-fc377a1a86f7"},}, new Dictionary<Terminator, ReduceInformation>{ }, "87a430b5-3e68-44a2-a5ce-31d848e8f156") },
		{ "99ca754c-e46f-44ad-a8e4-3141f6887122", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "3dd126ec-924d-41fa-8e46-323a74482868"},}, new Dictionary<Terminator, ReduceInformation>{ }, "99ca754c-e46f-44ad-a8e4-3141f6887122") },
		{ "31c99fb4-8248-4b8e-a6cf-a46f1624d6f1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "e1737cb5-9154-46f2-917c-feac4a8278f1"}, { new NonTerminator(NonTerminatorType.Expression), "2e7bde52-48a5-420d-a6e3-9f664510da68"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "17428d1e-ed4e-461b-ad85-a5f1d8f006eb"}, { new NonTerminator(NonTerminatorType.Term), "bab97185-47af-4ddd-8275-1863e6c37e23"}, { new NonTerminator(NonTerminatorType.Factor), "f5d56db2-25f1-4001-abce-4572f92e534a"}, { Terminator.NumberTerminator, "35cb5529-4f39-4b7c-a937-233f6efe6cdb"}, { new NonTerminator(NonTerminatorType.Variable), "ca1b9fbe-0a27-4b7d-ad6a-ee1ea8f20f5f"}, { new Terminator(DelimiterType.LeftParenthesis), "5f4ba076-b3cf-4472-848c-74d2a618eacb"}, { Terminator.IdentifierTerminator, "99071775-1827-45c4-b161-826ababf0384"}, { new Terminator(KeywordType.Not), "ed5ffdaa-b277-4c22-ad93-e4b1ed293ae4"}, { new Terminator(OperatorType.Minus), "15dc2c62-c938-4d0a-b0aa-1f0a0b73661f"},}, new Dictionary<Terminator, ReduceInformation>{ }, "31c99fb4-8248-4b8e-a6cf-a46f1624d6f1") },
		{ "13bceee6-fc96-44ff-a084-192a2173224c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "13bceee6-fc96-44ff-a084-192a2173224c") },
		{ "57965ac8-a7b8-462c-a073-32f12501457f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "b2f144b3-1f43-491c-b2de-6e366eebc2e6"}, { new NonTerminator(NonTerminatorType.Expression), "2e88b57e-7c35-4aee-9387-ff1c061cb7ab"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "089240b9-2be9-4199-97a8-41a6a9ac2f54"}, { new NonTerminator(NonTerminatorType.Term), "f46a091e-b991-43bd-877a-3c2eef645ab7"}, { new NonTerminator(NonTerminatorType.Factor), "5cb8d05c-bce3-47cc-8728-d7c037f2e9b1"}, { Terminator.NumberTerminator, "3928c790-d1f1-4d6a-8e98-6a0d0a3182e7"}, { new NonTerminator(NonTerminatorType.Variable), "08f29b35-75b5-4fe9-b677-ed18c2caa51b"}, { new Terminator(DelimiterType.LeftParenthesis), "7bb9f1fe-edd2-4812-9091-30ae6a61e59d"}, { Terminator.IdentifierTerminator, "ad126546-60a0-4982-8f68-c9a727b1c223"}, { new Terminator(KeywordType.Not), "d17a2e13-31b3-4e22-a0a6-5cf5fa910500"}, { new Terminator(OperatorType.Minus), "e8e4e781-e847-4c03-b1c8-fc377a1a86f7"},}, new Dictionary<Terminator, ReduceInformation>{ }, "57965ac8-a7b8-462c-a073-32f12501457f") },
		{ "ee596998-5c40-4e2d-b302-538b12b60141", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "ee596998-5c40-4e2d-b302-538b12b60141") },
		{ "1e2c05ca-183d-4501-b6ff-a053115ff436", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "1e2c05ca-183d-4501-b6ff-a053115ff436") },
		{ "99528598-aca5-437f-8146-f0c79a91351a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(4, new NonTerminator(NonTerminatorType.ProcedureCall))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(4, new NonTerminator(NonTerminatorType.ProcedureCall))}, }, "99528598-aca5-437f-8146-f0c79a91351a") },
		{ "f8c6158b-c7b5-4520-8a37-49ad141ff150", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "ff510cc9-d631-41d9-9295-5ff93f00529b"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "17428d1e-ed4e-461b-ad85-a5f1d8f006eb"}, { new NonTerminator(NonTerminatorType.Term), "bab97185-47af-4ddd-8275-1863e6c37e23"}, { new NonTerminator(NonTerminatorType.Factor), "f5d56db2-25f1-4001-abce-4572f92e534a"}, { Terminator.NumberTerminator, "35cb5529-4f39-4b7c-a937-233f6efe6cdb"}, { new NonTerminator(NonTerminatorType.Variable), "ca1b9fbe-0a27-4b7d-ad6a-ee1ea8f20f5f"}, { new Terminator(DelimiterType.LeftParenthesis), "5f4ba076-b3cf-4472-848c-74d2a618eacb"}, { Terminator.IdentifierTerminator, "99071775-1827-45c4-b161-826ababf0384"}, { new Terminator(KeywordType.Not), "ed5ffdaa-b277-4c22-ad93-e4b1ed293ae4"}, { new Terminator(OperatorType.Minus), "15dc2c62-c938-4d0a-b0aa-1f0a0b73661f"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f8c6158b-c7b5-4520-8a37-49ad141ff150") },
		{ "4fc347f9-6b87-41c5-87a7-dfca73a52291", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SimpleExpression), "d23120c8-422d-484d-b8a5-49bab4fa8b01"}, { new NonTerminator(NonTerminatorType.Term), "51b7b5be-9201-40c3-9d6e-fc8bf9bf7dbc"}, { new NonTerminator(NonTerminatorType.Factor), "08be8a9e-6d3e-471f-96fd-089d3fd390aa"}, { Terminator.NumberTerminator, "cd6de66d-c8ca-4eb3-8314-72445a11c5d4"}, { new NonTerminator(NonTerminatorType.Variable), "61f16867-4d7b-413e-b862-5468cafac64b"}, { new Terminator(DelimiterType.LeftParenthesis), "d08f77fa-a766-40ac-9825-0f7480e8c974"}, { Terminator.IdentifierTerminator, "1483a6f8-0714-4bb8-93f7-d0445f241849"}, { new Terminator(KeywordType.Not), "65db3e18-ac36-4517-98e5-6d7090d47fe9"}, { new Terminator(OperatorType.Minus), "8880f166-e99b-47c7-8de6-d2c669de9dc4"},}, new Dictionary<Terminator, ReduceInformation>{ }, "4fc347f9-6b87-41c5-87a7-dfca73a52291") },
		{ "e672341e-ca36-4d12-9752-c8ed2a1e61a4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "ca08da60-76aa-4421-9e70-ca15e204ffb7"}, { new NonTerminator(NonTerminatorType.Factor), "f5d56db2-25f1-4001-abce-4572f92e534a"}, { Terminator.NumberTerminator, "35cb5529-4f39-4b7c-a937-233f6efe6cdb"}, { new NonTerminator(NonTerminatorType.Variable), "ca1b9fbe-0a27-4b7d-ad6a-ee1ea8f20f5f"}, { new Terminator(DelimiterType.LeftParenthesis), "5f4ba076-b3cf-4472-848c-74d2a618eacb"}, { Terminator.IdentifierTerminator, "99071775-1827-45c4-b161-826ababf0384"}, { new Terminator(KeywordType.Not), "ed5ffdaa-b277-4c22-ad93-e4b1ed293ae4"}, { new Terminator(OperatorType.Minus), "15dc2c62-c938-4d0a-b0aa-1f0a0b73661f"},}, new Dictionary<Terminator, ReduceInformation>{ }, "e672341e-ca36-4d12-9752-c8ed2a1e61a4") },
		{ "c85c13e8-5099-4466-a6ad-914cf3ae1717", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "ef7b5804-394e-4175-9407-f0631f20b5dc"}, { Terminator.NumberTerminator, "35cb5529-4f39-4b7c-a937-233f6efe6cdb"}, { new NonTerminator(NonTerminatorType.Variable), "ca1b9fbe-0a27-4b7d-ad6a-ee1ea8f20f5f"}, { new Terminator(DelimiterType.LeftParenthesis), "5f4ba076-b3cf-4472-848c-74d2a618eacb"}, { Terminator.IdentifierTerminator, "99071775-1827-45c4-b161-826ababf0384"}, { new Terminator(KeywordType.Not), "ed5ffdaa-b277-4c22-ad93-e4b1ed293ae4"}, { new Terminator(OperatorType.Minus), "15dc2c62-c938-4d0a-b0aa-1f0a0b73661f"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c85c13e8-5099-4466-a6ad-914cf3ae1717") },
		{ "111da932-9fcf-452c-b21e-1132eff01cb4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "11159801-194d-4524-9dfe-0e0dea3c4c43"},}, new Dictionary<Terminator, ReduceInformation>{ }, "111da932-9fcf-452c-b21e-1132eff01cb4") },
		{ "e850ac14-0820-4312-bc39-5e3041cae261", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "b5a0c726-5e62-4f5e-8cb3-d44bf5dd10a7"}, { new NonTerminator(NonTerminatorType.Expression), "2e7bde52-48a5-420d-a6e3-9f664510da68"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "17428d1e-ed4e-461b-ad85-a5f1d8f006eb"}, { new NonTerminator(NonTerminatorType.Term), "bab97185-47af-4ddd-8275-1863e6c37e23"}, { new NonTerminator(NonTerminatorType.Factor), "f5d56db2-25f1-4001-abce-4572f92e534a"}, { Terminator.NumberTerminator, "35cb5529-4f39-4b7c-a937-233f6efe6cdb"}, { new NonTerminator(NonTerminatorType.Variable), "ca1b9fbe-0a27-4b7d-ad6a-ee1ea8f20f5f"}, { new Terminator(DelimiterType.LeftParenthesis), "5f4ba076-b3cf-4472-848c-74d2a618eacb"}, { Terminator.IdentifierTerminator, "99071775-1827-45c4-b161-826ababf0384"}, { new Terminator(KeywordType.Not), "ed5ffdaa-b277-4c22-ad93-e4b1ed293ae4"}, { new Terminator(OperatorType.Minus), "15dc2c62-c938-4d0a-b0aa-1f0a0b73661f"},}, new Dictionary<Terminator, ReduceInformation>{ }, "e850ac14-0820-4312-bc39-5e3041cae261") },
		{ "49b90856-ebe3-4970-95a9-fbf68e3ab032", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "49b90856-ebe3-4970-95a9-fbf68e3ab032") },
		{ "91da5d30-41c8-4ca6-ad3a-e06fc6ebc121", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "e2351150-31b1-4938-95e8-64434d03df78"}, { new NonTerminator(NonTerminatorType.Expression), "2e88b57e-7c35-4aee-9387-ff1c061cb7ab"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "089240b9-2be9-4199-97a8-41a6a9ac2f54"}, { new NonTerminator(NonTerminatorType.Term), "f46a091e-b991-43bd-877a-3c2eef645ab7"}, { new NonTerminator(NonTerminatorType.Factor), "5cb8d05c-bce3-47cc-8728-d7c037f2e9b1"}, { Terminator.NumberTerminator, "3928c790-d1f1-4d6a-8e98-6a0d0a3182e7"}, { new NonTerminator(NonTerminatorType.Variable), "08f29b35-75b5-4fe9-b677-ed18c2caa51b"}, { new Terminator(DelimiterType.LeftParenthesis), "7bb9f1fe-edd2-4812-9091-30ae6a61e59d"}, { Terminator.IdentifierTerminator, "ad126546-60a0-4982-8f68-c9a727b1c223"}, { new Terminator(KeywordType.Not), "d17a2e13-31b3-4e22-a0a6-5cf5fa910500"}, { new Terminator(OperatorType.Minus), "e8e4e781-e847-4c03-b1c8-fc377a1a86f7"},}, new Dictionary<Terminator, ReduceInformation>{ }, "91da5d30-41c8-4ca6-ad3a-e06fc6ebc121") },
		{ "09ccd708-a6b2-4dea-9328-786bebe811fc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "09ccd708-a6b2-4dea-9328-786bebe811fc") },
		{ "95d89dbe-bb7e-415d-84fa-d9f4393f2a87", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "95d89dbe-bb7e-415d-84fa-d9f4393f2a87") },
		{ "d10b614a-f743-4cc2-82a7-a7a9644cc5b4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ElsePart), "1a9db394-1c7b-4c1f-84c7-35381f1fc444"}, { new Terminator(KeywordType.Else), "71b8c7ca-8cfc-4672-b67d-1539564b08ee"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ElsePart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ElsePart))}, }, "d10b614a-f743-4cc2-82a7-a7a9644cc5b4") },
		{ "ea208c56-799b-4542-aab9-97041337f8cd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(OperatorType.Assign), "aa62e00d-51e2-4441-8eef-92b42c06300b"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ea208c56-799b-4542-aab9-97041337f8cd") },
		{ "9a4e95ea-b154-4175-98e3-a2691d4eb20f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(OperatorType.Assign), "7bdd3fd9-2f7a-4a2e-bd74-971d2ae26907"}, { new NonTerminator(NonTerminatorType.IdVarPart), "103ddd16-3ad7-45cc-a651-791a092a020b"}, { new Terminator(DelimiterType.LeftSquareBracket), "2b0d7e71-adca-43de-a708-6c2f8c5613ae"}, { new Terminator(DelimiterType.LeftParenthesis), "ad1510b1-a594-412b-9993-a59e4a06effa"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(OperatorType.Assign), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ProcedureCall))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ProcedureCall))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ProcedureCall))}, }, "9a4e95ea-b154-4175-98e3-a2691d4eb20f") },
		{ "14f3970e-268e-4251-aa12-eaa8effa762a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, }, "14f3970e-268e-4251-aa12-eaa8effa762a") },
		{ "b1815343-33db-47b4-8679-35c240abe9b0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, }, "b1815343-33db-47b4-8679-35c240abe9b0") },
		{ "e3fc592c-2252-423a-8c29-eb6efd3e239e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "fbf0622e-4f3e-43e6-82e9-71d3cbf7a770"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "d0b871cb-b302-4a53-856a-d5bd7f4e7d3c"}, { new NonTerminator(NonTerminatorType.Term), "5fd6c89b-6edd-4f4f-8d72-ce68a8f5d407"}, { new NonTerminator(NonTerminatorType.Factor), "67dd75bf-89f3-4ad3-8209-1d3a4d3f3303"}, { Terminator.NumberTerminator, "c90f9bea-bfaa-4211-8ea7-8c89c19751ad"}, { new NonTerminator(NonTerminatorType.Variable), "02430536-65bb-44a6-86f4-f3c102956340"}, { new Terminator(DelimiterType.LeftParenthesis), "cd305913-18dd-45cd-aebf-84bb50475ed5"}, { Terminator.IdentifierTerminator, "f41a8bc1-8fea-4591-a205-dd02472cc9e6"}, { new Terminator(KeywordType.Not), "21d6e202-9092-4593-a903-5b7ce59a3a44"}, { new Terminator(OperatorType.Minus), "d2f171ed-e2fb-4f7d-af92-d427c2f599b9"},}, new Dictionary<Terminator, ReduceInformation>{ }, "e3fc592c-2252-423a-8c29-eb6efd3e239e") },
		{ "35582002-f9e8-40c5-b5fe-930d8a63d94a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "ccbd478c-b8c3-4f71-8769-3ce5bceec8d5"},}, new Dictionary<Terminator, ReduceInformation>{ }, "35582002-f9e8-40c5-b5fe-930d8a63d94a") },
		{ "09269d2d-ccde-4852-8c11-5c03f8f1f9f2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.StatementList), "08e2a0ee-82ef-41c3-901f-dd395e3e5e9c"}, { new NonTerminator(NonTerminatorType.Statement), "c4bea712-ff83-427d-bf36-f8d83c3ed530"}, { new NonTerminator(NonTerminatorType.Variable), "be92d52d-f5cd-445d-b485-bd33a70d96d0"}, { Terminator.IdentifierTerminator, "b2246d94-54c6-4d0a-b568-08ce2b05aed2"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "54a3f285-f7dd-4bc1-bea4-a895b234ba48"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "430eb033-8171-4e0b-a572-c7fc61d11c56"}, { new Terminator(KeywordType.If), "5a82796b-2d92-4130-9660-38c01054dba4"}, { new Terminator(KeywordType.For), "6f8cff58-95d7-47f8-8456-787a36fc1c87"}, { new Terminator(KeywordType.Begin), "e9129679-d275-4d11-a66e-30fda7679656"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "09269d2d-ccde-4852-8c11-5c03f8f1f9f2") },
		{ "73d52634-0511-41fb-8811-32a583080972", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.AddOperator), "9c1bf7cf-cf3e-4585-abcf-6e031d9a82a3"}, { new Terminator(OperatorType.Plus), "2cf28f7e-6888-4e54-9825-8a80b00c7a56"}, { new Terminator(OperatorType.Minus), "1804ef7c-da92-431e-b725-ea7823b69327"}, { new Terminator(KeywordType.Or), "e777b812-5ca1-4317-9fd4-cc37fbcb2af0"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, }, "73d52634-0511-41fb-8811-32a583080972") },
		{ "e691605a-fd15-437d-bfab-0ef31722d3c1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "02ea126a-ea08-4518-984c-8f376afc1b38"}, { new Terminator(OperatorType.Multiply), "4d79eeb4-6b1a-4ca0-97f6-1f6675aed09f"}, { new Terminator(OperatorType.Divide), "fecb6832-79a2-4f95-95c4-fc698109ba3e"}, { new Terminator(KeywordType.Divide), "784cd050-c0ae-4844-8706-7f532d156c70"}, { new Terminator(KeywordType.Mod), "f3fdd777-da4f-4c74-9196-b8c4d7b7bace"}, { new Terminator(KeywordType.And), "376ab083-df94-433e-bcac-f300687907b1"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "e691605a-fd15-437d-bfab-0ef31722d3c1") },
		{ "aee43856-25e2-4f03-9e53-df91ae81e154", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "aee43856-25e2-4f03-9e53-df91ae81e154") },
		{ "cefd8fd4-7bfb-4296-b351-cf55cdf6df21", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "cefd8fd4-7bfb-4296-b351-cf55cdf6df21") },
		{ "84d3de93-f5f0-4bc3-80c5-34a876c227e9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "84d3de93-f5f0-4bc3-80c5-34a876c227e9") },
		{ "0354f240-08fd-4673-96f5-75f3b0f9a1b8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "c58fa8cf-09b3-4c8a-8280-b55f8bb2d523"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "550fbb43-0150-468f-9eb4-7bf5912daa0a"}, { new NonTerminator(NonTerminatorType.Term), "f0cad299-28f5-4e84-b9e7-2bc048f33840"}, { new NonTerminator(NonTerminatorType.Factor), "43e04c0c-8568-4c6b-ab2b-bceedcc9fa32"}, { Terminator.NumberTerminator, "c04070de-7d08-4aad-b5d7-41002b95f79d"}, { new NonTerminator(NonTerminatorType.Variable), "448fb14e-b117-467e-94e2-0ffad798acf2"}, { new Terminator(DelimiterType.LeftParenthesis), "61941794-1fbf-4cdd-a20f-059c95559fcc"}, { Terminator.IdentifierTerminator, "e932245d-2b23-4f7c-b6a4-c6fbd2b3dd9c"}, { new Terminator(KeywordType.Not), "8fcb5991-4a57-4bc9-b6ed-c1c6b22af95e"}, { new Terminator(OperatorType.Minus), "7312a539-9334-411e-b2f6-dc420da87c55"},}, new Dictionary<Terminator, ReduceInformation>{ }, "0354f240-08fd-4673-96f5-75f3b0f9a1b8") },
		{ "432a891c-9cfc-4720-aa49-66c77dc4da69", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "502cdade-c22e-4968-8b28-4d7c703c2a0a"}, { new NonTerminator(NonTerminatorType.IdVarPart), "6af26c0c-8215-4877-87a6-68a21569fb78"}, { new Terminator(DelimiterType.LeftSquareBracket), "20c1ad35-feaa-4a52-af58-6cb3a15484dd"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "432a891c-9cfc-4720-aa49-66c77dc4da69") },
		{ "79165cf0-6db5-4b11-b390-2e376bdf6296", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "f2d1210b-c6d4-4cdc-9a61-6fcbefb9e594"}, { Terminator.NumberTerminator, "cefd8fd4-7bfb-4296-b351-cf55cdf6df21"}, { new NonTerminator(NonTerminatorType.Variable), "84d3de93-f5f0-4bc3-80c5-34a876c227e9"}, { new Terminator(DelimiterType.LeftParenthesis), "0354f240-08fd-4673-96f5-75f3b0f9a1b8"}, { Terminator.IdentifierTerminator, "432a891c-9cfc-4720-aa49-66c77dc4da69"}, { new Terminator(KeywordType.Not), "79165cf0-6db5-4b11-b390-2e376bdf6296"}, { new Terminator(OperatorType.Minus), "3c0d12a5-5808-4427-ac54-415f251911e7"},}, new Dictionary<Terminator, ReduceInformation>{ }, "79165cf0-6db5-4b11-b390-2e376bdf6296") },
		{ "3c0d12a5-5808-4427-ac54-415f251911e7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "2bcf4acc-c4bd-4690-8c64-bb0f16687d2d"}, { Terminator.NumberTerminator, "cefd8fd4-7bfb-4296-b351-cf55cdf6df21"}, { new NonTerminator(NonTerminatorType.Variable), "84d3de93-f5f0-4bc3-80c5-34a876c227e9"}, { new Terminator(DelimiterType.LeftParenthesis), "0354f240-08fd-4673-96f5-75f3b0f9a1b8"}, { Terminator.IdentifierTerminator, "432a891c-9cfc-4720-aa49-66c77dc4da69"}, { new Terminator(KeywordType.Not), "79165cf0-6db5-4b11-b390-2e376bdf6296"}, { new Terminator(OperatorType.Minus), "3c0d12a5-5808-4427-ac54-415f251911e7"},}, new Dictionary<Terminator, ReduceInformation>{ }, "3c0d12a5-5808-4427-ac54-415f251911e7") },
		{ "c20ba73e-961a-44c4-b277-6be87879b7a9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "1d5fd969-7d23-4c9c-a74b-2af81c0d3a86"}, { new Terminator(OperatorType.Multiply), "4d79eeb4-6b1a-4ca0-97f6-1f6675aed09f"}, { new Terminator(OperatorType.Divide), "fecb6832-79a2-4f95-95c4-fc698109ba3e"}, { new Terminator(KeywordType.Divide), "784cd050-c0ae-4844-8706-7f532d156c70"}, { new Terminator(KeywordType.Mod), "f3fdd777-da4f-4c74-9196-b8c4d7b7bace"}, { new Terminator(KeywordType.And), "376ab083-df94-433e-bcac-f300687907b1"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "c20ba73e-961a-44c4-b277-6be87879b7a9") },
		{ "9138a41a-6056-4826-b2ca-2ced6f7e2f94", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "9138a41a-6056-4826-b2ca-2ced6f7e2f94") },
		{ "58752d46-753f-4ed1-8888-64534c80f38b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "58752d46-753f-4ed1-8888-64534c80f38b") },
		{ "0f91d141-6a65-4d4f-adf6-861609564784", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SimpleExpression), "31e5f84b-03ee-4fe8-a706-410f74703e01"}, { new NonTerminator(NonTerminatorType.Term), "2c38c7f3-8fa6-40e2-97cc-476e91825784"}, { new NonTerminator(NonTerminatorType.Factor), "a10796f7-837e-4ca0-9010-6fb4ffbac4bb"}, { Terminator.NumberTerminator, "907327ce-1671-4244-ba93-3de261d8481f"}, { new NonTerminator(NonTerminatorType.Variable), "45c1cf08-1190-49ce-8ef4-f9e3b94b8c01"}, { new Terminator(DelimiterType.LeftParenthesis), "51c9eb00-0d9b-46c5-9551-93b200bcda53"}, { Terminator.IdentifierTerminator, "52164dca-2b28-47de-ab17-a0588c9bb84a"}, { new Terminator(KeywordType.Not), "9bce9b84-8a03-41d5-a1ab-ac98d336a010"}, { new Terminator(OperatorType.Minus), "dc93c5ce-0987-492a-ac7e-928dd81cfb43"},}, new Dictionary<Terminator, ReduceInformation>{ }, "0f91d141-6a65-4d4f-adf6-861609564784") },
		{ "73f8d40c-6684-45d4-8b16-c7c1ec6369cb", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "fe5d1683-713f-4eb0-a24e-10242ebfa865"}, { new NonTerminator(NonTerminatorType.Factor), "43e04c0c-8568-4c6b-ab2b-bceedcc9fa32"}, { Terminator.NumberTerminator, "c04070de-7d08-4aad-b5d7-41002b95f79d"}, { new NonTerminator(NonTerminatorType.Variable), "448fb14e-b117-467e-94e2-0ffad798acf2"}, { new Terminator(DelimiterType.LeftParenthesis), "61941794-1fbf-4cdd-a20f-059c95559fcc"}, { Terminator.IdentifierTerminator, "e932245d-2b23-4f7c-b6a4-c6fbd2b3dd9c"}, { new Terminator(KeywordType.Not), "8fcb5991-4a57-4bc9-b6ed-c1c6b22af95e"}, { new Terminator(OperatorType.Minus), "7312a539-9334-411e-b2f6-dc420da87c55"},}, new Dictionary<Terminator, ReduceInformation>{ }, "73f8d40c-6684-45d4-8b16-c7c1ec6369cb") },
		{ "2bdea77e-b2e7-4243-a30d-f025bdb7cc68", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "3318c501-976b-4993-8466-da251afd6ba3"}, { Terminator.NumberTerminator, "c04070de-7d08-4aad-b5d7-41002b95f79d"}, { new NonTerminator(NonTerminatorType.Variable), "448fb14e-b117-467e-94e2-0ffad798acf2"}, { new Terminator(DelimiterType.LeftParenthesis), "61941794-1fbf-4cdd-a20f-059c95559fcc"}, { Terminator.IdentifierTerminator, "e932245d-2b23-4f7c-b6a4-c6fbd2b3dd9c"}, { new Terminator(KeywordType.Not), "8fcb5991-4a57-4bc9-b6ed-c1c6b22af95e"}, { new Terminator(OperatorType.Minus), "7312a539-9334-411e-b2f6-dc420da87c55"},}, new Dictionary<Terminator, ReduceInformation>{ }, "2bdea77e-b2e7-4243-a30d-f025bdb7cc68") },
		{ "6fab3ce2-edfd-4ece-8e52-de80029ea48d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "0d41ea0c-f6dc-4a84-b4cd-9b9e840301fe"},}, new Dictionary<Terminator, ReduceInformation>{ }, "6fab3ce2-edfd-4ece-8e52-de80029ea48d") },
		{ "d7246cda-d2bf-455b-9181-3d1f813ecda6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "c5d9c802-7e5b-4308-9c78-d178dfede699"}, { new NonTerminator(NonTerminatorType.Expression), "2e7bde52-48a5-420d-a6e3-9f664510da68"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "17428d1e-ed4e-461b-ad85-a5f1d8f006eb"}, { new NonTerminator(NonTerminatorType.Term), "bab97185-47af-4ddd-8275-1863e6c37e23"}, { new NonTerminator(NonTerminatorType.Factor), "f5d56db2-25f1-4001-abce-4572f92e534a"}, { Terminator.NumberTerminator, "35cb5529-4f39-4b7c-a937-233f6efe6cdb"}, { new NonTerminator(NonTerminatorType.Variable), "ca1b9fbe-0a27-4b7d-ad6a-ee1ea8f20f5f"}, { new Terminator(DelimiterType.LeftParenthesis), "5f4ba076-b3cf-4472-848c-74d2a618eacb"}, { Terminator.IdentifierTerminator, "99071775-1827-45c4-b161-826ababf0384"}, { new Terminator(KeywordType.Not), "ed5ffdaa-b277-4c22-ad93-e4b1ed293ae4"}, { new Terminator(OperatorType.Minus), "15dc2c62-c938-4d0a-b0aa-1f0a0b73661f"},}, new Dictionary<Terminator, ReduceInformation>{ }, "d7246cda-d2bf-455b-9181-3d1f813ecda6") },
		{ "c30cf66b-9bde-48f0-9552-b7de88707d26", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "c30cf66b-9bde-48f0-9552-b7de88707d26") },
		{ "38aa5d26-7153-444a-a3fe-88df3e94fa95", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "dbb4007c-b0d3-43ae-8f6f-29c5b3953f57"}, { new NonTerminator(NonTerminatorType.Expression), "2e88b57e-7c35-4aee-9387-ff1c061cb7ab"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "089240b9-2be9-4199-97a8-41a6a9ac2f54"}, { new NonTerminator(NonTerminatorType.Term), "f46a091e-b991-43bd-877a-3c2eef645ab7"}, { new NonTerminator(NonTerminatorType.Factor), "5cb8d05c-bce3-47cc-8728-d7c037f2e9b1"}, { Terminator.NumberTerminator, "3928c790-d1f1-4d6a-8e98-6a0d0a3182e7"}, { new NonTerminator(NonTerminatorType.Variable), "08f29b35-75b5-4fe9-b677-ed18c2caa51b"}, { new Terminator(DelimiterType.LeftParenthesis), "7bb9f1fe-edd2-4812-9091-30ae6a61e59d"}, { Terminator.IdentifierTerminator, "ad126546-60a0-4982-8f68-c9a727b1c223"}, { new Terminator(KeywordType.Not), "d17a2e13-31b3-4e22-a0a6-5cf5fa910500"}, { new Terminator(OperatorType.Minus), "e8e4e781-e847-4c03-b1c8-fc377a1a86f7"},}, new Dictionary<Terminator, ReduceInformation>{ }, "38aa5d26-7153-444a-a3fe-88df3e94fa95") },
		{ "13f05a96-f077-4d17-a345-6f7655720b85", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "13f05a96-f077-4d17-a345-6f7655720b85") },
		{ "35fafaa1-3a99-4617-9b3c-f59f02684b97", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "35fafaa1-3a99-4617-9b3c-f59f02684b97") },
		{ "e4efaa23-4566-447a-900b-bcdd6b1ba20b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "c7f53818-ab2a-453e-a733-168baec40f8b"}, { new Terminator(DelimiterType.Comma), "f8c6158b-c7b5-4520-8a37-49ad141ff150"},}, new Dictionary<Terminator, ReduceInformation>{ }, "e4efaa23-4566-447a-900b-bcdd6b1ba20b") },
		{ "0afbf405-4147-4ca1-a593-fcc5c74c14d0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "d54210ee-86f2-4f15-bc6f-cf10e39f24e2"}, { new Terminator(DelimiterType.Comma), "61999b62-48c1-4d86-8277-f2ec86cd3141"},}, new Dictionary<Terminator, ReduceInformation>{ }, "0afbf405-4147-4ca1-a593-fcc5c74c14d0") },
		{ "93ebf02a-d2a4-421f-a682-586f1545c447", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.To), "85851ee5-eae3-437d-a8d9-1dbf794b236e"},}, new Dictionary<Terminator, ReduceInformation>{ }, "93ebf02a-d2a4-421f-a682-586f1545c447") },
		{ "3e55bf24-5789-49ed-a5ae-5eb7ec7347cd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.RelationOperator), "d4d50c3d-c8a6-4857-a212-582d7eda7163"}, { new Terminator(OperatorType.Equal), "5cba3bbd-2f4e-4072-ab57-6933c47cf686"}, { new Terminator(OperatorType.NotEqual), "f19c1f01-646c-448a-a706-47091a7f6df7"}, { new Terminator(OperatorType.Less), "94b6c574-1915-47b0-b2de-d2c73eb5c1c2"}, { new Terminator(OperatorType.LessEqual), "0f2fec62-57bd-4061-9dff-9bcc1114b3a4"}, { new Terminator(OperatorType.Greater), "14be83c5-2fdf-4057-b5ed-b5bb521a20a2"}, { new Terminator(OperatorType.GreaterEqual), "83ce7741-272c-4749-8008-fcc200294ed8"}, { new NonTerminator(NonTerminatorType.AddOperator), "c3c9dba3-d13a-4cf0-9d67-40b23a01a4d8"}, { new Terminator(OperatorType.Plus), "2cf28f7e-6888-4e54-9825-8a80b00c7a56"}, { new Terminator(OperatorType.Minus), "1804ef7c-da92-431e-b725-ea7823b69327"}, { new Terminator(KeywordType.Or), "e777b812-5ca1-4317-9fd4-cc37fbcb2af0"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, }, "3e55bf24-5789-49ed-a5ae-5eb7ec7347cd") },
		{ "08443152-f969-4da6-aa5b-97e3b3345fc9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "71324a45-2a90-476d-814b-b47ef0e7d15f"}, { new Terminator(OperatorType.Multiply), "4d79eeb4-6b1a-4ca0-97f6-1f6675aed09f"}, { new Terminator(OperatorType.Divide), "fecb6832-79a2-4f95-95c4-fc698109ba3e"}, { new Terminator(KeywordType.Divide), "784cd050-c0ae-4844-8706-7f532d156c70"}, { new Terminator(KeywordType.Mod), "f3fdd777-da4f-4c74-9196-b8c4d7b7bace"}, { new Terminator(KeywordType.And), "376ab083-df94-433e-bcac-f300687907b1"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "08443152-f969-4da6-aa5b-97e3b3345fc9") },
		{ "c901372b-e50e-432c-ac38-9f6def8240bb", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "c901372b-e50e-432c-ac38-9f6def8240bb") },
		{ "d2a4c365-b1fc-48c2-aba8-dcd35bc4a13f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "d2a4c365-b1fc-48c2-aba8-dcd35bc4a13f") },
		{ "0cb8880d-7fd6-442b-97b7-5939869faac7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "0cb8880d-7fd6-442b-97b7-5939869faac7") },
		{ "571d117e-bca5-4cec-a8aa-99b81b821045", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "4fe911e7-081d-46a5-932c-7168521bb2f5"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "550fbb43-0150-468f-9eb4-7bf5912daa0a"}, { new NonTerminator(NonTerminatorType.Term), "f0cad299-28f5-4e84-b9e7-2bc048f33840"}, { new NonTerminator(NonTerminatorType.Factor), "43e04c0c-8568-4c6b-ab2b-bceedcc9fa32"}, { Terminator.NumberTerminator, "c04070de-7d08-4aad-b5d7-41002b95f79d"}, { new NonTerminator(NonTerminatorType.Variable), "448fb14e-b117-467e-94e2-0ffad798acf2"}, { new Terminator(DelimiterType.LeftParenthesis), "61941794-1fbf-4cdd-a20f-059c95559fcc"}, { Terminator.IdentifierTerminator, "e932245d-2b23-4f7c-b6a4-c6fbd2b3dd9c"}, { new Terminator(KeywordType.Not), "8fcb5991-4a57-4bc9-b6ed-c1c6b22af95e"}, { new Terminator(OperatorType.Minus), "7312a539-9334-411e-b2f6-dc420da87c55"},}, new Dictionary<Terminator, ReduceInformation>{ }, "571d117e-bca5-4cec-a8aa-99b81b821045") },
		{ "6f5f06f1-5c16-4c28-aa0b-a254c725d359", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "12f4f138-4fc2-4133-a4b1-ffbed9b7df21"}, { new NonTerminator(NonTerminatorType.IdVarPart), "f1af3f1f-cacc-436b-a491-5409056a0450"}, { new Terminator(DelimiterType.LeftSquareBracket), "4fc267dd-06b2-4bc2-976c-0ff630da0054"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "6f5f06f1-5c16-4c28-aa0b-a254c725d359") },
		{ "c90dca67-28e6-41ad-b475-ab75f98bff0f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "f067b908-10b5-450a-ac38-85d5c0592067"}, { Terminator.NumberTerminator, "d2a4c365-b1fc-48c2-aba8-dcd35bc4a13f"}, { new NonTerminator(NonTerminatorType.Variable), "0cb8880d-7fd6-442b-97b7-5939869faac7"}, { new Terminator(DelimiterType.LeftParenthesis), "571d117e-bca5-4cec-a8aa-99b81b821045"}, { Terminator.IdentifierTerminator, "6f5f06f1-5c16-4c28-aa0b-a254c725d359"}, { new Terminator(KeywordType.Not), "c90dca67-28e6-41ad-b475-ab75f98bff0f"}, { new Terminator(OperatorType.Minus), "48762e8c-76c8-477d-8b60-3bb834e285b3"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c90dca67-28e6-41ad-b475-ab75f98bff0f") },
		{ "48762e8c-76c8-477d-8b60-3bb834e285b3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "b121a507-28c6-40b7-af6c-6944c0309c70"}, { Terminator.NumberTerminator, "d2a4c365-b1fc-48c2-aba8-dcd35bc4a13f"}, { new NonTerminator(NonTerminatorType.Variable), "0cb8880d-7fd6-442b-97b7-5939869faac7"}, { new Terminator(DelimiterType.LeftParenthesis), "571d117e-bca5-4cec-a8aa-99b81b821045"}, { Terminator.IdentifierTerminator, "6f5f06f1-5c16-4c28-aa0b-a254c725d359"}, { new Terminator(KeywordType.Not), "c90dca67-28e6-41ad-b475-ab75f98bff0f"}, { new Terminator(OperatorType.Minus), "48762e8c-76c8-477d-8b60-3bb834e285b3"},}, new Dictionary<Terminator, ReduceInformation>{ }, "48762e8c-76c8-477d-8b60-3bb834e285b3") },
		{ "9466bb5e-b5e9-435e-a93a-c6f73e1303da", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SubprogramBody))}, }, "9466bb5e-b5e9-435e-a93a-c6f73e1303da") },
		{ "1a0ae9ce-0fa3-4795-b20b-26ac2b51461c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.StatementList), "34039465-af61-4338-b7c2-0eb1815f1daf"}, { new NonTerminator(NonTerminatorType.Statement), "c4bea712-ff83-427d-bf36-f8d83c3ed530"}, { new NonTerminator(NonTerminatorType.Variable), "be92d52d-f5cd-445d-b485-bd33a70d96d0"}, { Terminator.IdentifierTerminator, "b2246d94-54c6-4d0a-b568-08ce2b05aed2"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "54a3f285-f7dd-4bc1-bea4-a895b234ba48"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "430eb033-8171-4e0b-a572-c7fc61d11c56"}, { new Terminator(KeywordType.If), "5a82796b-2d92-4130-9660-38c01054dba4"}, { new Terminator(KeywordType.For), "6f8cff58-95d7-47f8-8456-787a36fc1c87"}, { new Terminator(KeywordType.Begin), "e9129679-d275-4d11-a66e-30fda7679656"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "1a0ae9ce-0fa3-4795-b20b-26ac2b51461c") },
		{ "5ca40820-c830-4dd8-8c3b-faa9f33be727", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Semicolon), "2a7df14b-08ec-4525-bb66-f3c237a49eb8"},}, new Dictionary<Terminator, ReduceInformation>{ }, "5ca40820-c830-4dd8-8c3b-faa9f33be727") },
		{ "b8c76ca4-ca81-46e6-a697-db2be07a0306", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "b550be58-4517-415d-b55d-5cb7151be29a"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ConstDeclarations))}, { new Terminator(KeywordType.Var), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ConstDeclarations))}, }, "b8c76ca4-ca81-46e6-a697-db2be07a0306") },
		{ "d2f0477a-42ad-4435-81e4-4acd7ae12d76", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.FormalParameter))}, }, "d2f0477a-42ad-4435-81e4-4acd7ae12d76") },
		{ "ac72cf4d-3f76-4b5b-a7b5-b7883396427c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Parameter), "c06b6c77-de1e-43a1-8f04-3abcf96c7df6"}, { new NonTerminator(NonTerminatorType.VarParameter), "3b2120f9-1746-4d68-9805-d552c9073b1a"}, { new NonTerminator(NonTerminatorType.ValueParameter), "88ddcbe8-4835-4b0b-939a-7fe235ac52d9"}, { new Terminator(KeywordType.Var), "05d3e85d-786a-4f42-85e2-dd90338f0594"}, { Terminator.IdentifierTerminator, "8266ea0c-3873-4a6f-b7d3-1cf665e02669"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ac72cf4d-3f76-4b5b-a7b5-b7883396427c") },
		{ "a7a4f3f8-03d0-4b27-bec4-2b6a42f2da67", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.VarParameter))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.VarParameter))}, }, "a7a4f3f8-03d0-4b27-bec4-2b6a42f2da67") },
		{ "57d2c113-bfeb-43f5-9e37-00ef947387c5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.ValueParameter))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.ValueParameter))}, }, "57d2c113-bfeb-43f5-9e37-00ef947387c5") },
		{ "59ea175e-51e1-4f0e-a58a-dba5258c48a3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "ef86660d-a03b-4bcc-8f6a-69fc3f53d31a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "59ea175e-51e1-4f0e-a58a-dba5258c48a3") },
		{ "63f9e1a5-607e-4602-90bc-9c1d20cbc841", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Type), "8111ccc8-74bd-4724-ab82-acae096c5edb"}, { new NonTerminator(NonTerminatorType.BasicType), "65cd38a0-f803-4f6d-b49e-87fb80c73b51"}, { new Terminator(KeywordType.Array), "901ff9f6-fa85-4471-a491-2a980d2e789d"}, { new Terminator(KeywordType.Integer), "2352d3f9-6384-4719-a336-d52576da05cd"}, { new Terminator(KeywordType.Real), "f0f06df7-693c-4589-a52c-1fc0b527a0c4"}, { new Terminator(KeywordType.Boolean), "b946cd1b-56cc-44f0-a1aa-9a4204988ac3"}, { new Terminator(KeywordType.Character), "c1517388-d76a-4ff8-81ce-ee6053dae200"},}, new Dictionary<Terminator, ReduceInformation>{ }, "63f9e1a5-607e-4602-90bc-9c1d20cbc841") },
		{ "52e16e08-cdf4-412f-9656-f1f051afdfbe", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(5, new NonTerminator(NonTerminatorType.SubprogramHead))}, }, "52e16e08-cdf4-412f-9656-f1f051afdfbe") },
		{ "a53abbcc-dead-4d62-9ee6-7dfa0ce96e2f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Colon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.FormalParameter))}, }, "a53abbcc-dead-4d62-9ee6-7dfa0ce96e2f") },
		{ "aed33747-2903-44e4-b7dc-d6c3da4637af", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.Of), "30a2b1a4-cd6b-4b1d-9243-99ac4c30938e"},}, new Dictionary<Terminator, ReduceInformation>{ }, "aed33747-2903-44e4-b7dc-d6c3da4637af") },
		{ "05594cd9-2dec-4ee8-bddd-e67bb8ce693f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(6, new NonTerminator(NonTerminatorType.Type))}, }, "05594cd9-2dec-4ee8-bddd-e67bb8ce693f") },
		{ "11ab4735-af5c-4c5f-a303-84f519fdb27f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.NumberTerminator, "2a91e2ef-3704-40d1-9dca-bd96294397d3"},}, new Dictionary<Terminator, ReduceInformation>{ }, "11ab4735-af5c-4c5f-a303-84f519fdb27f") },
		{ "210f20bb-d846-4551-9755-3cb2da04ddac", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.AddOperator), "e89dd997-5dee-441e-94be-a210e6209a4e"}, { new Terminator(OperatorType.Plus), "2cf28f7e-6888-4e54-9825-8a80b00c7a56"}, { new Terminator(OperatorType.Minus), "1804ef7c-da92-431e-b725-ea7823b69327"}, { new Terminator(KeywordType.Or), "e777b812-5ca1-4317-9fd4-cc37fbcb2af0"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, }, "210f20bb-d846-4551-9755-3cb2da04ddac") },
		{ "62602618-d5b3-48b8-98fd-c7800d4b080c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "88cf57b9-1ed9-494d-836a-62c9992ed253"}, { new Terminator(OperatorType.Multiply), "4d79eeb4-6b1a-4ca0-97f6-1f6675aed09f"}, { new Terminator(OperatorType.Divide), "fecb6832-79a2-4f95-95c4-fc698109ba3e"}, { new Terminator(KeywordType.Divide), "784cd050-c0ae-4844-8706-7f532d156c70"}, { new Terminator(KeywordType.Mod), "f3fdd777-da4f-4c74-9196-b8c4d7b7bace"}, { new Terminator(KeywordType.And), "376ab083-df94-433e-bcac-f300687907b1"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "62602618-d5b3-48b8-98fd-c7800d4b080c") },
		{ "c3d60e4d-5c66-4705-a482-2058ad1fedc1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "c3d60e4d-5c66-4705-a482-2058ad1fedc1") },
		{ "69de6cb0-b810-436f-8aa1-7f9cd7ffdc47", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "69de6cb0-b810-436f-8aa1-7f9cd7ffdc47") },
		{ "f8d34924-6053-411a-bcf7-fbd2f2c5a5d0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "f8d34924-6053-411a-bcf7-fbd2f2c5a5d0") },
		{ "14c29ffd-630e-465b-953a-59d607e06a6a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "f2e8832a-44a1-4857-bfb3-e99bbd8a64a6"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "550fbb43-0150-468f-9eb4-7bf5912daa0a"}, { new NonTerminator(NonTerminatorType.Term), "f0cad299-28f5-4e84-b9e7-2bc048f33840"}, { new NonTerminator(NonTerminatorType.Factor), "43e04c0c-8568-4c6b-ab2b-bceedcc9fa32"}, { Terminator.NumberTerminator, "c04070de-7d08-4aad-b5d7-41002b95f79d"}, { new NonTerminator(NonTerminatorType.Variable), "448fb14e-b117-467e-94e2-0ffad798acf2"}, { new Terminator(DelimiterType.LeftParenthesis), "61941794-1fbf-4cdd-a20f-059c95559fcc"}, { Terminator.IdentifierTerminator, "e932245d-2b23-4f7c-b6a4-c6fbd2b3dd9c"}, { new Terminator(KeywordType.Not), "8fcb5991-4a57-4bc9-b6ed-c1c6b22af95e"}, { new Terminator(OperatorType.Minus), "7312a539-9334-411e-b2f6-dc420da87c55"},}, new Dictionary<Terminator, ReduceInformation>{ }, "14c29ffd-630e-465b-953a-59d607e06a6a") },
		{ "e11b9910-5816-47fc-9455-7015c3ffd940", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "ac97e816-8114-4188-89f5-7b607e0c31ef"}, { new NonTerminator(NonTerminatorType.IdVarPart), "3bb0a54a-8410-4eed-9e88-4a1811f69f01"}, { new Terminator(DelimiterType.LeftSquareBracket), "576b6c29-5517-4fca-9771-e7631bee026d"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "e11b9910-5816-47fc-9455-7015c3ffd940") },
		{ "33da2634-9ccd-439e-9316-87806b898b4c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "daad863e-bcd3-4cae-8020-518f4d7d96d0"}, { Terminator.NumberTerminator, "69de6cb0-b810-436f-8aa1-7f9cd7ffdc47"}, { new NonTerminator(NonTerminatorType.Variable), "f8d34924-6053-411a-bcf7-fbd2f2c5a5d0"}, { new Terminator(DelimiterType.LeftParenthesis), "14c29ffd-630e-465b-953a-59d607e06a6a"}, { Terminator.IdentifierTerminator, "e11b9910-5816-47fc-9455-7015c3ffd940"}, { new Terminator(KeywordType.Not), "33da2634-9ccd-439e-9316-87806b898b4c"}, { new Terminator(OperatorType.Minus), "d4dd0bca-2671-4abf-8dab-793593072da6"},}, new Dictionary<Terminator, ReduceInformation>{ }, "33da2634-9ccd-439e-9316-87806b898b4c") },
		{ "d4dd0bca-2671-4abf-8dab-793593072da6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "6f3ea32e-ec21-430a-8d5b-8b28f6f36572"}, { Terminator.NumberTerminator, "69de6cb0-b810-436f-8aa1-7f9cd7ffdc47"}, { new NonTerminator(NonTerminatorType.Variable), "f8d34924-6053-411a-bcf7-fbd2f2c5a5d0"}, { new Terminator(DelimiterType.LeftParenthesis), "14c29ffd-630e-465b-953a-59d607e06a6a"}, { Terminator.IdentifierTerminator, "e11b9910-5816-47fc-9455-7015c3ffd940"}, { new Terminator(KeywordType.Not), "33da2634-9ccd-439e-9316-87806b898b4c"}, { new Terminator(OperatorType.Minus), "d4dd0bca-2671-4abf-8dab-793593072da6"},}, new Dictionary<Terminator, ReduceInformation>{ }, "d4dd0bca-2671-4abf-8dab-793593072da6") },
		{ "6cf85861-3b32-41ad-982d-ccbb00c8a40c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "045ec7dd-2546-4063-ac79-3c172c197c7d"}, { new Terminator(OperatorType.Multiply), "4d79eeb4-6b1a-4ca0-97f6-1f6675aed09f"}, { new Terminator(OperatorType.Divide), "fecb6832-79a2-4f95-95c4-fc698109ba3e"}, { new Terminator(KeywordType.Divide), "784cd050-c0ae-4844-8706-7f532d156c70"}, { new Terminator(KeywordType.Mod), "f3fdd777-da4f-4c74-9196-b8c4d7b7bace"}, { new Terminator(KeywordType.And), "376ab083-df94-433e-bcac-f300687907b1"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "6cf85861-3b32-41ad-982d-ccbb00c8a40c") },
		{ "e5b2b50c-7474-4328-b827-d0c25cffd3ba", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "e5b2b50c-7474-4328-b827-d0c25cffd3ba") },
		{ "48ff5b3e-da68-4a4f-a5ed-472350ac8b56", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "48ff5b3e-da68-4a4f-a5ed-472350ac8b56") },
		{ "2d8a36df-a7df-4c0f-9ac4-701ecbcb325b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "891d0e39-5ba0-45da-ab22-7b27cc04350f"}, { new Terminator(DelimiterType.Comma), "f8c6158b-c7b5-4520-8a37-49ad141ff150"},}, new Dictionary<Terminator, ReduceInformation>{ }, "2d8a36df-a7df-4c0f-9ac4-701ecbcb325b") },
		{ "ca1fbe72-6394-4195-9520-b7e6063d190e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "a7d27889-a27b-4e71-8b0c-4a6a830b7695"}, { new Terminator(DelimiterType.Comma), "61999b62-48c1-4d86-8277-f2ec86cd3141"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ca1fbe72-6394-4195-9520-b7e6063d190e") },
		{ "557bddf8-f28b-4b8e-ab05-3eb1d60d62f4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ExpressionList))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ExpressionList))}, }, "557bddf8-f28b-4b8e-ab05-3eb1d60d62f4") },
		{ "8144becc-3e46-43b9-9333-9a20fe517e94", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.AddOperator), "dbab63d2-df35-428a-b5cc-2dd2e2524f08"}, { new Terminator(OperatorType.Plus), "2cf28f7e-6888-4e54-9825-8a80b00c7a56"}, { new Terminator(OperatorType.Minus), "1804ef7c-da92-431e-b725-ea7823b69327"}, { new Terminator(KeywordType.Or), "e777b812-5ca1-4317-9fd4-cc37fbcb2af0"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, }, "8144becc-3e46-43b9-9333-9a20fe517e94") },
		{ "06691bf4-bc4e-43f7-93b8-c56af8b73219", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "72099dbe-7244-4758-92c1-c936b2f150d3"}, { new Terminator(OperatorType.Multiply), "4d79eeb4-6b1a-4ca0-97f6-1f6675aed09f"}, { new Terminator(OperatorType.Divide), "fecb6832-79a2-4f95-95c4-fc698109ba3e"}, { new Terminator(KeywordType.Divide), "784cd050-c0ae-4844-8706-7f532d156c70"}, { new Terminator(KeywordType.Mod), "f3fdd777-da4f-4c74-9196-b8c4d7b7bace"}, { new Terminator(KeywordType.And), "376ab083-df94-433e-bcac-f300687907b1"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "06691bf4-bc4e-43f7-93b8-c56af8b73219") },
		{ "e6af18a0-1b24-4fe5-b501-7bf4ba96defd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "e6af18a0-1b24-4fe5-b501-7bf4ba96defd") },
		{ "ba493efc-5d7d-425a-ab17-3057add7633e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "ba493efc-5d7d-425a-ab17-3057add7633e") },
		{ "e937950c-aa65-4d42-97a8-6d55502689bf", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "e937950c-aa65-4d42-97a8-6d55502689bf") },
		{ "e3448ecf-4866-44ba-a34b-af6d031ad38d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "9f5e2fb3-7703-40e8-a983-7ad7f3424016"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "550fbb43-0150-468f-9eb4-7bf5912daa0a"}, { new NonTerminator(NonTerminatorType.Term), "f0cad299-28f5-4e84-b9e7-2bc048f33840"}, { new NonTerminator(NonTerminatorType.Factor), "43e04c0c-8568-4c6b-ab2b-bceedcc9fa32"}, { Terminator.NumberTerminator, "c04070de-7d08-4aad-b5d7-41002b95f79d"}, { new NonTerminator(NonTerminatorType.Variable), "448fb14e-b117-467e-94e2-0ffad798acf2"}, { new Terminator(DelimiterType.LeftParenthesis), "61941794-1fbf-4cdd-a20f-059c95559fcc"}, { Terminator.IdentifierTerminator, "e932245d-2b23-4f7c-b6a4-c6fbd2b3dd9c"}, { new Terminator(KeywordType.Not), "8fcb5991-4a57-4bc9-b6ed-c1c6b22af95e"}, { new Terminator(OperatorType.Minus), "7312a539-9334-411e-b2f6-dc420da87c55"},}, new Dictionary<Terminator, ReduceInformation>{ }, "e3448ecf-4866-44ba-a34b-af6d031ad38d") },
		{ "685e6fcf-000d-47c8-aee8-cc1d8fee08c3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "be34b8ce-e4cf-4326-a6c0-e7d413a828c9"}, { new NonTerminator(NonTerminatorType.IdVarPart), "269fa658-7c13-48b0-a885-b40f67446a14"}, { new Terminator(DelimiterType.LeftSquareBracket), "79745d95-9048-43ff-9f8b-40f7d392845f"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "685e6fcf-000d-47c8-aee8-cc1d8fee08c3") },
		{ "814b64e0-3336-426d-8775-15f9eddce24e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "8a847411-8db9-414d-8fd9-3188aea86f56"}, { Terminator.NumberTerminator, "ba493efc-5d7d-425a-ab17-3057add7633e"}, { new NonTerminator(NonTerminatorType.Variable), "e937950c-aa65-4d42-97a8-6d55502689bf"}, { new Terminator(DelimiterType.LeftParenthesis), "e3448ecf-4866-44ba-a34b-af6d031ad38d"}, { Terminator.IdentifierTerminator, "685e6fcf-000d-47c8-aee8-cc1d8fee08c3"}, { new Terminator(KeywordType.Not), "814b64e0-3336-426d-8775-15f9eddce24e"}, { new Terminator(OperatorType.Minus), "910b40b6-c2d2-4c5a-8567-9da1ff7eee9f"},}, new Dictionary<Terminator, ReduceInformation>{ }, "814b64e0-3336-426d-8775-15f9eddce24e") },
		{ "910b40b6-c2d2-4c5a-8567-9da1ff7eee9f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "af4ff363-b5b6-4610-b43a-051015e34257"}, { Terminator.NumberTerminator, "ba493efc-5d7d-425a-ab17-3057add7633e"}, { new NonTerminator(NonTerminatorType.Variable), "e937950c-aa65-4d42-97a8-6d55502689bf"}, { new Terminator(DelimiterType.LeftParenthesis), "e3448ecf-4866-44ba-a34b-af6d031ad38d"}, { Terminator.IdentifierTerminator, "685e6fcf-000d-47c8-aee8-cc1d8fee08c3"}, { new Terminator(KeywordType.Not), "814b64e0-3336-426d-8775-15f9eddce24e"}, { new Terminator(OperatorType.Minus), "910b40b6-c2d2-4c5a-8567-9da1ff7eee9f"},}, new Dictionary<Terminator, ReduceInformation>{ }, "910b40b6-c2d2-4c5a-8567-9da1ff7eee9f") },
		{ "644e4ee9-99a3-4975-a4cc-5bca42aa4242", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "87a430b5-3e68-44a2-a5ce-31d848e8f156"}, { new Terminator(OperatorType.Multiply), "4d79eeb4-6b1a-4ca0-97f6-1f6675aed09f"}, { new Terminator(OperatorType.Divide), "fecb6832-79a2-4f95-95c4-fc698109ba3e"}, { new Terminator(KeywordType.Divide), "784cd050-c0ae-4844-8706-7f532d156c70"}, { new Terminator(KeywordType.Mod), "f3fdd777-da4f-4c74-9196-b8c4d7b7bace"}, { new Terminator(KeywordType.And), "376ab083-df94-433e-bcac-f300687907b1"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "644e4ee9-99a3-4975-a4cc-5bca42aa4242") },
		{ "2d852909-7739-4f52-ad12-81dd511cfbad", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "2d852909-7739-4f52-ad12-81dd511cfbad") },
		{ "3dd126ec-924d-41fa-8e46-323a74482868", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "3dd126ec-924d-41fa-8e46-323a74482868") },
		{ "e1737cb5-9154-46f2-917c-feac4a8278f1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "acb92439-8833-498e-b71b-bcca5ab90adf"}, { new Terminator(DelimiterType.Comma), "f8c6158b-c7b5-4520-8a37-49ad141ff150"},}, new Dictionary<Terminator, ReduceInformation>{ }, "e1737cb5-9154-46f2-917c-feac4a8278f1") },
		{ "b2f144b3-1f43-491c-b2de-6e366eebc2e6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "3fee0f49-bfd3-41cd-8e1d-5c622b5503b1"}, { new Terminator(DelimiterType.Comma), "61999b62-48c1-4d86-8277-f2ec86cd3141"},}, new Dictionary<Terminator, ReduceInformation>{ }, "b2f144b3-1f43-491c-b2de-6e366eebc2e6") },
		{ "ff510cc9-d631-41d9-9295-5ff93f00529b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ExpressionList))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ExpressionList))}, }, "ff510cc9-d631-41d9-9295-5ff93f00529b") },
		{ "d23120c8-422d-484d-b8a5-49bab4fa8b01", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.AddOperator), "76688848-6c3e-483f-b362-f7c0d3904500"}, { new Terminator(OperatorType.Plus), "2cf28f7e-6888-4e54-9825-8a80b00c7a56"}, { new Terminator(OperatorType.Minus), "1804ef7c-da92-431e-b725-ea7823b69327"}, { new Terminator(KeywordType.Or), "e777b812-5ca1-4317-9fd4-cc37fbcb2af0"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, }, "d23120c8-422d-484d-b8a5-49bab4fa8b01") },
		{ "51b7b5be-9201-40c3-9d6e-fc8bf9bf7dbc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "9b50ef94-e2d1-4693-aaac-d1a529701978"}, { new Terminator(OperatorType.Multiply), "4d79eeb4-6b1a-4ca0-97f6-1f6675aed09f"}, { new Terminator(OperatorType.Divide), "fecb6832-79a2-4f95-95c4-fc698109ba3e"}, { new Terminator(KeywordType.Divide), "784cd050-c0ae-4844-8706-7f532d156c70"}, { new Terminator(KeywordType.Mod), "f3fdd777-da4f-4c74-9196-b8c4d7b7bace"}, { new Terminator(KeywordType.And), "376ab083-df94-433e-bcac-f300687907b1"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "51b7b5be-9201-40c3-9d6e-fc8bf9bf7dbc") },
		{ "08be8a9e-6d3e-471f-96fd-089d3fd390aa", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "08be8a9e-6d3e-471f-96fd-089d3fd390aa") },
		{ "cd6de66d-c8ca-4eb3-8314-72445a11c5d4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "cd6de66d-c8ca-4eb3-8314-72445a11c5d4") },
		{ "61f16867-4d7b-413e-b862-5468cafac64b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "61f16867-4d7b-413e-b862-5468cafac64b") },
		{ "d08f77fa-a766-40ac-9825-0f7480e8c974", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "eea72f37-bfd6-4309-a5c9-13ebd76e9c42"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "550fbb43-0150-468f-9eb4-7bf5912daa0a"}, { new NonTerminator(NonTerminatorType.Term), "f0cad299-28f5-4e84-b9e7-2bc048f33840"}, { new NonTerminator(NonTerminatorType.Factor), "43e04c0c-8568-4c6b-ab2b-bceedcc9fa32"}, { Terminator.NumberTerminator, "c04070de-7d08-4aad-b5d7-41002b95f79d"}, { new NonTerminator(NonTerminatorType.Variable), "448fb14e-b117-467e-94e2-0ffad798acf2"}, { new Terminator(DelimiterType.LeftParenthesis), "61941794-1fbf-4cdd-a20f-059c95559fcc"}, { Terminator.IdentifierTerminator, "e932245d-2b23-4f7c-b6a4-c6fbd2b3dd9c"}, { new Terminator(KeywordType.Not), "8fcb5991-4a57-4bc9-b6ed-c1c6b22af95e"}, { new Terminator(OperatorType.Minus), "7312a539-9334-411e-b2f6-dc420da87c55"},}, new Dictionary<Terminator, ReduceInformation>{ }, "d08f77fa-a766-40ac-9825-0f7480e8c974") },
		{ "1483a6f8-0714-4bb8-93f7-d0445f241849", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "378b88c2-0901-48ae-b1ab-24bf24f0ea8b"}, { new NonTerminator(NonTerminatorType.IdVarPart), "f8430992-f55c-4ba9-943f-c7183f2f8bf9"}, { new Terminator(DelimiterType.LeftSquareBracket), "ffb7bd7a-7e9f-4b96-bf49-c595094a3b0e"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "1483a6f8-0714-4bb8-93f7-d0445f241849") },
		{ "65db3e18-ac36-4517-98e5-6d7090d47fe9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "f88ad60c-073f-4b63-be18-135959523e95"}, { Terminator.NumberTerminator, "cd6de66d-c8ca-4eb3-8314-72445a11c5d4"}, { new NonTerminator(NonTerminatorType.Variable), "61f16867-4d7b-413e-b862-5468cafac64b"}, { new Terminator(DelimiterType.LeftParenthesis), "d08f77fa-a766-40ac-9825-0f7480e8c974"}, { Terminator.IdentifierTerminator, "1483a6f8-0714-4bb8-93f7-d0445f241849"}, { new Terminator(KeywordType.Not), "65db3e18-ac36-4517-98e5-6d7090d47fe9"}, { new Terminator(OperatorType.Minus), "8880f166-e99b-47c7-8de6-d2c669de9dc4"},}, new Dictionary<Terminator, ReduceInformation>{ }, "65db3e18-ac36-4517-98e5-6d7090d47fe9") },
		{ "8880f166-e99b-47c7-8de6-d2c669de9dc4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "2f2f05cc-bdcf-4457-a43f-1f40592ec5d4"}, { Terminator.NumberTerminator, "cd6de66d-c8ca-4eb3-8314-72445a11c5d4"}, { new NonTerminator(NonTerminatorType.Variable), "61f16867-4d7b-413e-b862-5468cafac64b"}, { new Terminator(DelimiterType.LeftParenthesis), "d08f77fa-a766-40ac-9825-0f7480e8c974"}, { Terminator.IdentifierTerminator, "1483a6f8-0714-4bb8-93f7-d0445f241849"}, { new Terminator(KeywordType.Not), "65db3e18-ac36-4517-98e5-6d7090d47fe9"}, { new Terminator(OperatorType.Minus), "8880f166-e99b-47c7-8de6-d2c669de9dc4"},}, new Dictionary<Terminator, ReduceInformation>{ }, "8880f166-e99b-47c7-8de6-d2c669de9dc4") },
		{ "ca08da60-76aa-4421-9e70-ca15e204ffb7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "c85c13e8-5099-4466-a6ad-914cf3ae1717"}, { new Terminator(OperatorType.Multiply), "4d79eeb4-6b1a-4ca0-97f6-1f6675aed09f"}, { new Terminator(OperatorType.Divide), "fecb6832-79a2-4f95-95c4-fc698109ba3e"}, { new Terminator(KeywordType.Divide), "784cd050-c0ae-4844-8706-7f532d156c70"}, { new Terminator(KeywordType.Mod), "f3fdd777-da4f-4c74-9196-b8c4d7b7bace"}, { new Terminator(KeywordType.And), "376ab083-df94-433e-bcac-f300687907b1"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "ca08da60-76aa-4421-9e70-ca15e204ffb7") },
		{ "ef7b5804-394e-4175-9407-f0631f20b5dc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "ef7b5804-394e-4175-9407-f0631f20b5dc") },
		{ "11159801-194d-4524-9dfe-0e0dea3c4c43", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "11159801-194d-4524-9dfe-0e0dea3c4c43") },
		{ "b5a0c726-5e62-4f5e-8cb3-d44bf5dd10a7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "369d713c-aaa9-4c82-a597-77c4aef39d50"}, { new Terminator(DelimiterType.Comma), "f8c6158b-c7b5-4520-8a37-49ad141ff150"},}, new Dictionary<Terminator, ReduceInformation>{ }, "b5a0c726-5e62-4f5e-8cb3-d44bf5dd10a7") },
		{ "e2351150-31b1-4938-95e8-64434d03df78", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "d536904a-9fa6-434b-9cb5-f7eb5ca3af1c"}, { new Terminator(DelimiterType.Comma), "61999b62-48c1-4d86-8277-f2ec86cd3141"},}, new Dictionary<Terminator, ReduceInformation>{ }, "e2351150-31b1-4938-95e8-64434d03df78") },
		{ "1a9db394-1c7b-4c1f-84c7-35381f1fc444", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(5, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(5, new NonTerminator(NonTerminatorType.Statement))}, }, "1a9db394-1c7b-4c1f-84c7-35381f1fc444") },
		{ "71b8c7ca-8cfc-4672-b67d-1539564b08ee", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Statement), "25d6bc40-dc69-4f2b-9976-ee660d147555"}, { new NonTerminator(NonTerminatorType.Variable), "be92d52d-f5cd-445d-b485-bd33a70d96d0"}, { Terminator.IdentifierTerminator, "b2246d94-54c6-4d0a-b568-08ce2b05aed2"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "54a3f285-f7dd-4bc1-bea4-a895b234ba48"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "430eb033-8171-4e0b-a572-c7fc61d11c56"}, { new Terminator(KeywordType.If), "5a82796b-2d92-4130-9660-38c01054dba4"}, { new Terminator(KeywordType.For), "6f8cff58-95d7-47f8-8456-787a36fc1c87"}, { new Terminator(KeywordType.Begin), "e9129679-d275-4d11-a66e-30fda7679656"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "71b8c7ca-8cfc-4672-b67d-1539564b08ee") },
		{ "aa62e00d-51e2-4441-8eef-92b42c06300b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "764f6824-9035-43d0-932b-067e544093c8"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "5d725b4b-caf4-4222-a977-00ef92cec269"}, { new NonTerminator(NonTerminatorType.Term), "a783da79-bb67-4798-969d-9ba08a5268e8"}, { new NonTerminator(NonTerminatorType.Factor), "84eb5d09-6a9c-4152-8ee1-08cf1977b4b4"}, { Terminator.NumberTerminator, "8695b530-26f3-4534-a7ec-6e01b3b8b90c"}, { new NonTerminator(NonTerminatorType.Variable), "dd2136a1-9348-4c93-a3d6-cd3454e51c22"}, { new Terminator(DelimiterType.LeftParenthesis), "d1159160-d982-4e79-9cb1-37896891fa6b"}, { Terminator.IdentifierTerminator, "74fa8c5c-9ee2-4481-b6b9-1cf08e5fd510"}, { new Terminator(KeywordType.Not), "d2ee92ce-60e1-42c5-93bb-b132c2796e16"}, { new Terminator(OperatorType.Minus), "410a8465-e4b5-4ba9-b1b4-fc3e3f81080e"},}, new Dictionary<Terminator, ReduceInformation>{ }, "aa62e00d-51e2-4441-8eef-92b42c06300b") },
		{ "7bdd3fd9-2f7a-4a2e-bd74-971d2ae26907", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "e1c9e4a0-9cd0-4cf8-bbf4-0838f0657394"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "5d725b4b-caf4-4222-a977-00ef92cec269"}, { new NonTerminator(NonTerminatorType.Term), "a783da79-bb67-4798-969d-9ba08a5268e8"}, { new NonTerminator(NonTerminatorType.Factor), "84eb5d09-6a9c-4152-8ee1-08cf1977b4b4"}, { Terminator.NumberTerminator, "8695b530-26f3-4534-a7ec-6e01b3b8b90c"}, { new NonTerminator(NonTerminatorType.Variable), "dd2136a1-9348-4c93-a3d6-cd3454e51c22"}, { new Terminator(DelimiterType.LeftParenthesis), "d1159160-d982-4e79-9cb1-37896891fa6b"}, { Terminator.IdentifierTerminator, "74fa8c5c-9ee2-4481-b6b9-1cf08e5fd510"}, { new Terminator(KeywordType.Not), "d2ee92ce-60e1-42c5-93bb-b132c2796e16"}, { new Terminator(OperatorType.Minus), "410a8465-e4b5-4ba9-b1b4-fc3e3f81080e"},}, new Dictionary<Terminator, ReduceInformation>{ }, "7bdd3fd9-2f7a-4a2e-bd74-971d2ae26907") },
		{ "ad1510b1-a594-412b-9993-a59e4a06effa", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "6dd2e068-a1b5-445d-858d-af446583ed22"}, { new NonTerminator(NonTerminatorType.Expression), "2e7bde52-48a5-420d-a6e3-9f664510da68"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "17428d1e-ed4e-461b-ad85-a5f1d8f006eb"}, { new NonTerminator(NonTerminatorType.Term), "bab97185-47af-4ddd-8275-1863e6c37e23"}, { new NonTerminator(NonTerminatorType.Factor), "f5d56db2-25f1-4001-abce-4572f92e534a"}, { Terminator.NumberTerminator, "35cb5529-4f39-4b7c-a937-233f6efe6cdb"}, { new NonTerminator(NonTerminatorType.Variable), "ca1b9fbe-0a27-4b7d-ad6a-ee1ea8f20f5f"}, { new Terminator(DelimiterType.LeftParenthesis), "5f4ba076-b3cf-4472-848c-74d2a618eacb"}, { Terminator.IdentifierTerminator, "99071775-1827-45c4-b161-826ababf0384"}, { new Terminator(KeywordType.Not), "ed5ffdaa-b277-4c22-ad93-e4b1ed293ae4"}, { new Terminator(OperatorType.Minus), "15dc2c62-c938-4d0a-b0aa-1f0a0b73661f"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ad1510b1-a594-412b-9993-a59e4a06effa") },
		{ "fbf0622e-4f3e-43e6-82e9-71d3cbf7a770", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.Then), "1cb0b7ad-0584-48f2-abe2-b5e18f1e1baa"},}, new Dictionary<Terminator, ReduceInformation>{ }, "fbf0622e-4f3e-43e6-82e9-71d3cbf7a770") },
		{ "ccbd478c-b8c3-4f71-8769-3ce5bceec8d5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(OperatorType.Assign), "982b4453-5b96-4fcb-9512-e3090089f0c2"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ccbd478c-b8c3-4f71-8769-3ce5bceec8d5") },
		{ "08e2a0ee-82ef-41c3-901f-dd395e3e5e9c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.End), "272221c9-80f3-4475-84a7-c85130ddca87"}, { new Terminator(DelimiterType.Semicolon), "655817d0-6fda-4a1b-a6fb-1075ad52b093"},}, new Dictionary<Terminator, ReduceInformation>{ }, "08e2a0ee-82ef-41c3-901f-dd395e3e5e9c") },
		{ "9c1bf7cf-cf3e-4585-abcf-6e031d9a82a3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "cfc69cbe-ab80-4316-b80c-76dfaa865934"}, { new NonTerminator(NonTerminatorType.Factor), "aee43856-25e2-4f03-9e53-df91ae81e154"}, { Terminator.NumberTerminator, "cefd8fd4-7bfb-4296-b351-cf55cdf6df21"}, { new NonTerminator(NonTerminatorType.Variable), "84d3de93-f5f0-4bc3-80c5-34a876c227e9"}, { new Terminator(DelimiterType.LeftParenthesis), "0354f240-08fd-4673-96f5-75f3b0f9a1b8"}, { Terminator.IdentifierTerminator, "432a891c-9cfc-4720-aa49-66c77dc4da69"}, { new Terminator(KeywordType.Not), "79165cf0-6db5-4b11-b390-2e376bdf6296"}, { new Terminator(OperatorType.Minus), "3c0d12a5-5808-4427-ac54-415f251911e7"},}, new Dictionary<Terminator, ReduceInformation>{ }, "9c1bf7cf-cf3e-4585-abcf-6e031d9a82a3") },
		{ "02ea126a-ea08-4518-984c-8f376afc1b38", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "aef41990-e07f-40e0-8bef-1a2bb868b834"}, { Terminator.NumberTerminator, "cefd8fd4-7bfb-4296-b351-cf55cdf6df21"}, { new NonTerminator(NonTerminatorType.Variable), "84d3de93-f5f0-4bc3-80c5-34a876c227e9"}, { new Terminator(DelimiterType.LeftParenthesis), "0354f240-08fd-4673-96f5-75f3b0f9a1b8"}, { Terminator.IdentifierTerminator, "432a891c-9cfc-4720-aa49-66c77dc4da69"}, { new Terminator(KeywordType.Not), "79165cf0-6db5-4b11-b390-2e376bdf6296"}, { new Terminator(OperatorType.Minus), "3c0d12a5-5808-4427-ac54-415f251911e7"},}, new Dictionary<Terminator, ReduceInformation>{ }, "02ea126a-ea08-4518-984c-8f376afc1b38") },
		{ "c58fa8cf-09b3-4c8a-8280-b55f8bb2d523", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "a614a956-b58d-4554-8e34-772753c618be"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c58fa8cf-09b3-4c8a-8280-b55f8bb2d523") },
		{ "502cdade-c22e-4968-8b28-4d7c703c2a0a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "73f75d0e-0fa3-43ca-a75d-47bc4224c8eb"}, { new NonTerminator(NonTerminatorType.Expression), "2e7bde52-48a5-420d-a6e3-9f664510da68"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "17428d1e-ed4e-461b-ad85-a5f1d8f006eb"}, { new NonTerminator(NonTerminatorType.Term), "bab97185-47af-4ddd-8275-1863e6c37e23"}, { new NonTerminator(NonTerminatorType.Factor), "f5d56db2-25f1-4001-abce-4572f92e534a"}, { Terminator.NumberTerminator, "35cb5529-4f39-4b7c-a937-233f6efe6cdb"}, { new NonTerminator(NonTerminatorType.Variable), "ca1b9fbe-0a27-4b7d-ad6a-ee1ea8f20f5f"}, { new Terminator(DelimiterType.LeftParenthesis), "5f4ba076-b3cf-4472-848c-74d2a618eacb"}, { Terminator.IdentifierTerminator, "99071775-1827-45c4-b161-826ababf0384"}, { new Terminator(KeywordType.Not), "ed5ffdaa-b277-4c22-ad93-e4b1ed293ae4"}, { new Terminator(OperatorType.Minus), "15dc2c62-c938-4d0a-b0aa-1f0a0b73661f"},}, new Dictionary<Terminator, ReduceInformation>{ }, "502cdade-c22e-4968-8b28-4d7c703c2a0a") },
		{ "6af26c0c-8215-4877-87a6-68a21569fb78", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "6af26c0c-8215-4877-87a6-68a21569fb78") },
		{ "20c1ad35-feaa-4a52-af58-6cb3a15484dd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "f3892946-11cd-44eb-a569-7cf3cf854b6c"}, { new NonTerminator(NonTerminatorType.Expression), "2e88b57e-7c35-4aee-9387-ff1c061cb7ab"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "089240b9-2be9-4199-97a8-41a6a9ac2f54"}, { new NonTerminator(NonTerminatorType.Term), "f46a091e-b991-43bd-877a-3c2eef645ab7"}, { new NonTerminator(NonTerminatorType.Factor), "5cb8d05c-bce3-47cc-8728-d7c037f2e9b1"}, { Terminator.NumberTerminator, "3928c790-d1f1-4d6a-8e98-6a0d0a3182e7"}, { new NonTerminator(NonTerminatorType.Variable), "08f29b35-75b5-4fe9-b677-ed18c2caa51b"}, { new Terminator(DelimiterType.LeftParenthesis), "7bb9f1fe-edd2-4812-9091-30ae6a61e59d"}, { Terminator.IdentifierTerminator, "ad126546-60a0-4982-8f68-c9a727b1c223"}, { new Terminator(KeywordType.Not), "d17a2e13-31b3-4e22-a0a6-5cf5fa910500"}, { new Terminator(OperatorType.Minus), "e8e4e781-e847-4c03-b1c8-fc377a1a86f7"},}, new Dictionary<Terminator, ReduceInformation>{ }, "20c1ad35-feaa-4a52-af58-6cb3a15484dd") },
		{ "f2d1210b-c6d4-4cdc-9a61-6fcbefb9e594", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "f2d1210b-c6d4-4cdc-9a61-6fcbefb9e594") },
		{ "2bcf4acc-c4bd-4690-8c64-bb0f16687d2d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "2bcf4acc-c4bd-4690-8c64-bb0f16687d2d") },
		{ "31e5f84b-03ee-4fe8-a706-410f74703e01", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.AddOperator), "6720fbfb-d07f-4e2d-9d03-8f2af5594db0"}, { new Terminator(OperatorType.Plus), "2cf28f7e-6888-4e54-9825-8a80b00c7a56"}, { new Terminator(OperatorType.Minus), "1804ef7c-da92-431e-b725-ea7823b69327"}, { new Terminator(KeywordType.Or), "e777b812-5ca1-4317-9fd4-cc37fbcb2af0"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, }, "31e5f84b-03ee-4fe8-a706-410f74703e01") },
		{ "2c38c7f3-8fa6-40e2-97cc-476e91825784", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "4d0b17ae-8185-4259-8a8d-16432387f97f"}, { new Terminator(OperatorType.Multiply), "4d79eeb4-6b1a-4ca0-97f6-1f6675aed09f"}, { new Terminator(OperatorType.Divide), "fecb6832-79a2-4f95-95c4-fc698109ba3e"}, { new Terminator(KeywordType.Divide), "784cd050-c0ae-4844-8706-7f532d156c70"}, { new Terminator(KeywordType.Mod), "f3fdd777-da4f-4c74-9196-b8c4d7b7bace"}, { new Terminator(KeywordType.And), "376ab083-df94-433e-bcac-f300687907b1"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "2c38c7f3-8fa6-40e2-97cc-476e91825784") },
		{ "a10796f7-837e-4ca0-9010-6fb4ffbac4bb", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "a10796f7-837e-4ca0-9010-6fb4ffbac4bb") },
		{ "907327ce-1671-4244-ba93-3de261d8481f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "907327ce-1671-4244-ba93-3de261d8481f") },
		{ "45c1cf08-1190-49ce-8ef4-f9e3b94b8c01", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "45c1cf08-1190-49ce-8ef4-f9e3b94b8c01") },
		{ "51c9eb00-0d9b-46c5-9551-93b200bcda53", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "519745f5-6613-4db1-bfc2-019209bf796b"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "550fbb43-0150-468f-9eb4-7bf5912daa0a"}, { new NonTerminator(NonTerminatorType.Term), "f0cad299-28f5-4e84-b9e7-2bc048f33840"}, { new NonTerminator(NonTerminatorType.Factor), "43e04c0c-8568-4c6b-ab2b-bceedcc9fa32"}, { Terminator.NumberTerminator, "c04070de-7d08-4aad-b5d7-41002b95f79d"}, { new NonTerminator(NonTerminatorType.Variable), "448fb14e-b117-467e-94e2-0ffad798acf2"}, { new Terminator(DelimiterType.LeftParenthesis), "61941794-1fbf-4cdd-a20f-059c95559fcc"}, { Terminator.IdentifierTerminator, "e932245d-2b23-4f7c-b6a4-c6fbd2b3dd9c"}, { new Terminator(KeywordType.Not), "8fcb5991-4a57-4bc9-b6ed-c1c6b22af95e"}, { new Terminator(OperatorType.Minus), "7312a539-9334-411e-b2f6-dc420da87c55"},}, new Dictionary<Terminator, ReduceInformation>{ }, "51c9eb00-0d9b-46c5-9551-93b200bcda53") },
		{ "52164dca-2b28-47de-ab17-a0588c9bb84a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "a2b9e513-db1a-4804-a121-38f1876d137c"}, { new NonTerminator(NonTerminatorType.IdVarPart), "e9a81368-5389-482a-8972-f02d89229728"}, { new Terminator(DelimiterType.LeftSquareBracket), "a6abda55-a744-4aff-b76c-631bb4fc4d3d"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "52164dca-2b28-47de-ab17-a0588c9bb84a") },
		{ "9bce9b84-8a03-41d5-a1ab-ac98d336a010", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "9b98e7e0-086c-4fec-bf8e-ca82f5c9fabe"}, { Terminator.NumberTerminator, "907327ce-1671-4244-ba93-3de261d8481f"}, { new NonTerminator(NonTerminatorType.Variable), "45c1cf08-1190-49ce-8ef4-f9e3b94b8c01"}, { new Terminator(DelimiterType.LeftParenthesis), "51c9eb00-0d9b-46c5-9551-93b200bcda53"}, { Terminator.IdentifierTerminator, "52164dca-2b28-47de-ab17-a0588c9bb84a"}, { new Terminator(KeywordType.Not), "9bce9b84-8a03-41d5-a1ab-ac98d336a010"}, { new Terminator(OperatorType.Minus), "dc93c5ce-0987-492a-ac7e-928dd81cfb43"},}, new Dictionary<Terminator, ReduceInformation>{ }, "9bce9b84-8a03-41d5-a1ab-ac98d336a010") },
		{ "dc93c5ce-0987-492a-ac7e-928dd81cfb43", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "ac9b22f6-41e1-4f6e-95d6-ce70d76fe888"}, { Terminator.NumberTerminator, "907327ce-1671-4244-ba93-3de261d8481f"}, { new NonTerminator(NonTerminatorType.Variable), "45c1cf08-1190-49ce-8ef4-f9e3b94b8c01"}, { new Terminator(DelimiterType.LeftParenthesis), "51c9eb00-0d9b-46c5-9551-93b200bcda53"}, { Terminator.IdentifierTerminator, "52164dca-2b28-47de-ab17-a0588c9bb84a"}, { new Terminator(KeywordType.Not), "9bce9b84-8a03-41d5-a1ab-ac98d336a010"}, { new Terminator(OperatorType.Minus), "dc93c5ce-0987-492a-ac7e-928dd81cfb43"},}, new Dictionary<Terminator, ReduceInformation>{ }, "dc93c5ce-0987-492a-ac7e-928dd81cfb43") },
		{ "fe5d1683-713f-4eb0-a24e-10242ebfa865", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "2bdea77e-b2e7-4243-a30d-f025bdb7cc68"}, { new Terminator(OperatorType.Multiply), "4d79eeb4-6b1a-4ca0-97f6-1f6675aed09f"}, { new Terminator(OperatorType.Divide), "fecb6832-79a2-4f95-95c4-fc698109ba3e"}, { new Terminator(KeywordType.Divide), "784cd050-c0ae-4844-8706-7f532d156c70"}, { new Terminator(KeywordType.Mod), "f3fdd777-da4f-4c74-9196-b8c4d7b7bace"}, { new Terminator(KeywordType.And), "376ab083-df94-433e-bcac-f300687907b1"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "fe5d1683-713f-4eb0-a24e-10242ebfa865") },
		{ "3318c501-976b-4993-8466-da251afd6ba3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "3318c501-976b-4993-8466-da251afd6ba3") },
		{ "0d41ea0c-f6dc-4a84-b4cd-9b9e840301fe", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "0d41ea0c-f6dc-4a84-b4cd-9b9e840301fe") },
		{ "c5d9c802-7e5b-4308-9c78-d178dfede699", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "94db1226-8145-43bc-8c98-c4b1589b5665"}, { new Terminator(DelimiterType.Comma), "f8c6158b-c7b5-4520-8a37-49ad141ff150"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c5d9c802-7e5b-4308-9c78-d178dfede699") },
		{ "dbb4007c-b0d3-43ae-8f6f-29c5b3953f57", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "b79e2959-9ba5-42ce-a941-ad533e21d0a8"}, { new Terminator(DelimiterType.Comma), "61999b62-48c1-4d86-8277-f2ec86cd3141"},}, new Dictionary<Terminator, ReduceInformation>{ }, "dbb4007c-b0d3-43ae-8f6f-29c5b3953f57") },
		{ "c7f53818-ab2a-453e-a733-168baec40f8b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "c7f53818-ab2a-453e-a733-168baec40f8b") },
		{ "d54210ee-86f2-4f15-bc6f-cf10e39f24e2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "d54210ee-86f2-4f15-bc6f-cf10e39f24e2") },
		{ "85851ee5-eae3-437d-a8d9-1dbf794b236e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "4fe9222b-5e17-471b-a4f2-d7a29c2d05ec"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "caa8525c-e146-4f2e-9489-00c244cfad57"}, { new NonTerminator(NonTerminatorType.Term), "54afe343-cda5-4f84-aa6d-5a21b405f421"}, { new NonTerminator(NonTerminatorType.Factor), "e0658f9a-921e-4b74-8fb5-e6b77d907947"}, { Terminator.NumberTerminator, "799b1e98-e544-4cd9-b48d-386861187fe6"}, { new NonTerminator(NonTerminatorType.Variable), "71ecda72-cc64-4040-8496-5a1c0ec09b77"}, { new Terminator(DelimiterType.LeftParenthesis), "c2439980-49fc-48d9-9626-5ba6e380cde7"}, { Terminator.IdentifierTerminator, "820af974-dbec-4e03-8931-dfc4ef51c15a"}, { new Terminator(KeywordType.Not), "c7800d99-09f1-4c4b-a492-985868834e26"}, { new Terminator(OperatorType.Minus), "e1f14f62-b8e2-44d0-9ca8-130cecab42e5"},}, new Dictionary<Terminator, ReduceInformation>{ }, "85851ee5-eae3-437d-a8d9-1dbf794b236e") },
		{ "d4d50c3d-c8a6-4857-a212-582d7eda7163", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SimpleExpression), "c30bcfe6-462c-4961-b7d2-f2f81fc8b911"}, { new NonTerminator(NonTerminatorType.Term), "431166b7-a50a-4ccb-8ca2-a6a3b6376190"}, { new NonTerminator(NonTerminatorType.Factor), "ebaa5ae2-0b7d-4e21-8868-fa6d971f9af4"}, { Terminator.NumberTerminator, "62ff026c-b825-42a4-b94f-5a7f0c5eadc3"}, { new NonTerminator(NonTerminatorType.Variable), "0eb974e5-1b89-4348-ae84-86606df4b3e2"}, { new Terminator(DelimiterType.LeftParenthesis), "70c7bd8e-7d2d-457f-8e4f-aa520d90eceb"}, { Terminator.IdentifierTerminator, "7f4abdfb-ae44-46c5-baf8-95faf8050421"}, { new Terminator(KeywordType.Not), "39ace80c-b849-4b52-b2dd-2b29b089d55e"}, { new Terminator(OperatorType.Minus), "d2dd7e4e-1dd6-472f-847a-452c97248860"},}, new Dictionary<Terminator, ReduceInformation>{ }, "d4d50c3d-c8a6-4857-a212-582d7eda7163") },
		{ "c3c9dba3-d13a-4cf0-9d67-40b23a01a4d8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "49fe4340-9c7e-4d3f-a315-3b907851410b"}, { new NonTerminator(NonTerminatorType.Factor), "c901372b-e50e-432c-ac38-9f6def8240bb"}, { Terminator.NumberTerminator, "d2a4c365-b1fc-48c2-aba8-dcd35bc4a13f"}, { new NonTerminator(NonTerminatorType.Variable), "0cb8880d-7fd6-442b-97b7-5939869faac7"}, { new Terminator(DelimiterType.LeftParenthesis), "571d117e-bca5-4cec-a8aa-99b81b821045"}, { Terminator.IdentifierTerminator, "6f5f06f1-5c16-4c28-aa0b-a254c725d359"}, { new Terminator(KeywordType.Not), "c90dca67-28e6-41ad-b475-ab75f98bff0f"}, { new Terminator(OperatorType.Minus), "48762e8c-76c8-477d-8b60-3bb834e285b3"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c3c9dba3-d13a-4cf0-9d67-40b23a01a4d8") },
		{ "71324a45-2a90-476d-814b-b47ef0e7d15f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "1d05d9be-273f-4d0e-8d7f-7905f70aeada"}, { Terminator.NumberTerminator, "d2a4c365-b1fc-48c2-aba8-dcd35bc4a13f"}, { new NonTerminator(NonTerminatorType.Variable), "0cb8880d-7fd6-442b-97b7-5939869faac7"}, { new Terminator(DelimiterType.LeftParenthesis), "571d117e-bca5-4cec-a8aa-99b81b821045"}, { Terminator.IdentifierTerminator, "6f5f06f1-5c16-4c28-aa0b-a254c725d359"}, { new Terminator(KeywordType.Not), "c90dca67-28e6-41ad-b475-ab75f98bff0f"}, { new Terminator(OperatorType.Minus), "48762e8c-76c8-477d-8b60-3bb834e285b3"},}, new Dictionary<Terminator, ReduceInformation>{ }, "71324a45-2a90-476d-814b-b47ef0e7d15f") },
		{ "4fe911e7-081d-46a5-932c-7168521bb2f5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "5487a61e-d1be-478c-8fea-776b8cce7c72"},}, new Dictionary<Terminator, ReduceInformation>{ }, "4fe911e7-081d-46a5-932c-7168521bb2f5") },
		{ "12f4f138-4fc2-4133-a4b1-ffbed9b7df21", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "ad33a481-047b-4172-bb2c-574e5be261be"}, { new NonTerminator(NonTerminatorType.Expression), "2e7bde52-48a5-420d-a6e3-9f664510da68"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "17428d1e-ed4e-461b-ad85-a5f1d8f006eb"}, { new NonTerminator(NonTerminatorType.Term), "bab97185-47af-4ddd-8275-1863e6c37e23"}, { new NonTerminator(NonTerminatorType.Factor), "f5d56db2-25f1-4001-abce-4572f92e534a"}, { Terminator.NumberTerminator, "35cb5529-4f39-4b7c-a937-233f6efe6cdb"}, { new NonTerminator(NonTerminatorType.Variable), "ca1b9fbe-0a27-4b7d-ad6a-ee1ea8f20f5f"}, { new Terminator(DelimiterType.LeftParenthesis), "5f4ba076-b3cf-4472-848c-74d2a618eacb"}, { Terminator.IdentifierTerminator, "99071775-1827-45c4-b161-826ababf0384"}, { new Terminator(KeywordType.Not), "ed5ffdaa-b277-4c22-ad93-e4b1ed293ae4"}, { new Terminator(OperatorType.Minus), "15dc2c62-c938-4d0a-b0aa-1f0a0b73661f"},}, new Dictionary<Terminator, ReduceInformation>{ }, "12f4f138-4fc2-4133-a4b1-ffbed9b7df21") },
		{ "f1af3f1f-cacc-436b-a491-5409056a0450", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "f1af3f1f-cacc-436b-a491-5409056a0450") },
		{ "4fc267dd-06b2-4bc2-976c-0ff630da0054", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "9d21fd54-8c38-4b05-b4b8-4b6f529b8d43"}, { new NonTerminator(NonTerminatorType.Expression), "2e88b57e-7c35-4aee-9387-ff1c061cb7ab"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "089240b9-2be9-4199-97a8-41a6a9ac2f54"}, { new NonTerminator(NonTerminatorType.Term), "f46a091e-b991-43bd-877a-3c2eef645ab7"}, { new NonTerminator(NonTerminatorType.Factor), "5cb8d05c-bce3-47cc-8728-d7c037f2e9b1"}, { Terminator.NumberTerminator, "3928c790-d1f1-4d6a-8e98-6a0d0a3182e7"}, { new NonTerminator(NonTerminatorType.Variable), "08f29b35-75b5-4fe9-b677-ed18c2caa51b"}, { new Terminator(DelimiterType.LeftParenthesis), "7bb9f1fe-edd2-4812-9091-30ae6a61e59d"}, { Terminator.IdentifierTerminator, "ad126546-60a0-4982-8f68-c9a727b1c223"}, { new Terminator(KeywordType.Not), "d17a2e13-31b3-4e22-a0a6-5cf5fa910500"}, { new Terminator(OperatorType.Minus), "e8e4e781-e847-4c03-b1c8-fc377a1a86f7"},}, new Dictionary<Terminator, ReduceInformation>{ }, "4fc267dd-06b2-4bc2-976c-0ff630da0054") },
		{ "f067b908-10b5-450a-ac38-85d5c0592067", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "f067b908-10b5-450a-ac38-85d5c0592067") },
		{ "b121a507-28c6-40b7-af6c-6944c0309c70", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "b121a507-28c6-40b7-af6c-6944c0309c70") },
		{ "34039465-af61-4338-b7c2-0eb1815f1daf", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.End), "75c1506e-487e-4018-a995-51419f376ef7"}, { new Terminator(DelimiterType.Semicolon), "655817d0-6fda-4a1b-a6fb-1075ad52b093"},}, new Dictionary<Terminator, ReduceInformation>{ }, "34039465-af61-4338-b7c2-0eb1815f1daf") },
		{ "2a7df14b-08ec-4525-bb66-f3c237a49eb8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "935d6c9f-d4d4-445b-9154-26664a79ca9a"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(3, new NonTerminator(NonTerminatorType.VarDeclarations))}, }, "2a7df14b-08ec-4525-bb66-f3c237a49eb8") },
		{ "c06b6c77-de1e-43a1-8f04-3abcf96c7df6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ParameterList))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ParameterList))}, }, "c06b6c77-de1e-43a1-8f04-3abcf96c7df6") },
		{ "ef86660d-a03b-4bcc-8f6a-69fc3f53d31a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.IdentifierList), "7422a0bb-5b5d-4e39-b5e1-38e88f81634d"}, { new Terminator(DelimiterType.Comma), "59ea175e-51e1-4f0e-a58a-dba5258c48a3"}, { new Terminator(DelimiterType.Colon), "63f9e1a5-607e-4602-90bc-9c1d20cbc841"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ef86660d-a03b-4bcc-8f6a-69fc3f53d31a") },
		{ "8111ccc8-74bd-4724-ab82-acae096c5edb", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.IdentifierList))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.IdentifierList))}, }, "8111ccc8-74bd-4724-ab82-acae096c5edb") },
		{ "65cd38a0-f803-4f6d-b49e-87fb80c73b51", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Type))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Type))}, }, "65cd38a0-f803-4f6d-b49e-87fb80c73b51") },
		{ "901ff9f6-fa85-4471-a491-2a980d2e789d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftSquareBracket), "bc0bbb00-97d8-487c-8149-40725edf8672"},}, new Dictionary<Terminator, ReduceInformation>{ }, "901ff9f6-fa85-4471-a491-2a980d2e789d") },
		{ "2352d3f9-6384-4719-a336-d52576da05cd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "2352d3f9-6384-4719-a336-d52576da05cd") },
		{ "f0f06df7-693c-4589-a52c-1fc0b527a0c4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "f0f06df7-693c-4589-a52c-1fc0b527a0c4") },
		{ "b946cd1b-56cc-44f0-a1aa-9a4204988ac3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "b946cd1b-56cc-44f0-a1aa-9a4204988ac3") },
		{ "c1517388-d76a-4ff8-81ce-ee6053dae200", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "c1517388-d76a-4ff8-81ce-ee6053dae200") },
		{ "30a2b1a4-cd6b-4b1d-9243-99ac4c30938e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.BasicType), "cc29535d-b050-47c8-a988-cbda96148685"}, { new Terminator(KeywordType.Integer), "ebe37bec-c209-4195-a140-f57d3f2763ea"}, { new Terminator(KeywordType.Real), "f3352fc3-8224-4718-b0cc-5afcdbcea304"}, { new Terminator(KeywordType.Boolean), "7942b503-91a1-408d-bdf4-f3f76c9a33a0"}, { new Terminator(KeywordType.Character), "ad9d0031-e9ad-485c-bbee-619e17a3a472"},}, new Dictionary<Terminator, ReduceInformation>{ }, "30a2b1a4-cd6b-4b1d-9243-99ac4c30938e") },
		{ "2a91e2ef-3704-40d1-9dca-bd96294397d3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(5, new NonTerminator(NonTerminatorType.Period))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(5, new NonTerminator(NonTerminatorType.Period))}, }, "2a91e2ef-3704-40d1-9dca-bd96294397d3") },
		{ "e89dd997-5dee-441e-94be-a210e6209a4e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "524789dd-5fbb-4950-8ddb-fa9b8c0c446c"}, { new NonTerminator(NonTerminatorType.Factor), "c3d60e4d-5c66-4705-a482-2058ad1fedc1"}, { Terminator.NumberTerminator, "69de6cb0-b810-436f-8aa1-7f9cd7ffdc47"}, { new NonTerminator(NonTerminatorType.Variable), "f8d34924-6053-411a-bcf7-fbd2f2c5a5d0"}, { new Terminator(DelimiterType.LeftParenthesis), "14c29ffd-630e-465b-953a-59d607e06a6a"}, { Terminator.IdentifierTerminator, "e11b9910-5816-47fc-9455-7015c3ffd940"}, { new Terminator(KeywordType.Not), "33da2634-9ccd-439e-9316-87806b898b4c"}, { new Terminator(OperatorType.Minus), "d4dd0bca-2671-4abf-8dab-793593072da6"},}, new Dictionary<Terminator, ReduceInformation>{ }, "e89dd997-5dee-441e-94be-a210e6209a4e") },
		{ "88cf57b9-1ed9-494d-836a-62c9992ed253", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "a0be572d-6a2f-4faa-ae73-a22c5664c6cc"}, { Terminator.NumberTerminator, "69de6cb0-b810-436f-8aa1-7f9cd7ffdc47"}, { new NonTerminator(NonTerminatorType.Variable), "f8d34924-6053-411a-bcf7-fbd2f2c5a5d0"}, { new Terminator(DelimiterType.LeftParenthesis), "14c29ffd-630e-465b-953a-59d607e06a6a"}, { Terminator.IdentifierTerminator, "e11b9910-5816-47fc-9455-7015c3ffd940"}, { new Terminator(KeywordType.Not), "33da2634-9ccd-439e-9316-87806b898b4c"}, { new Terminator(OperatorType.Minus), "d4dd0bca-2671-4abf-8dab-793593072da6"},}, new Dictionary<Terminator, ReduceInformation>{ }, "88cf57b9-1ed9-494d-836a-62c9992ed253") },
		{ "f2e8832a-44a1-4857-bfb3-e99bbd8a64a6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "1980bc4e-3495-46c3-ba86-84e97c1dd237"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f2e8832a-44a1-4857-bfb3-e99bbd8a64a6") },
		{ "ac97e816-8114-4188-89f5-7b607e0c31ef", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "d9d40659-3731-484f-935a-09a99e7dc108"}, { new NonTerminator(NonTerminatorType.Expression), "2e7bde52-48a5-420d-a6e3-9f664510da68"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "17428d1e-ed4e-461b-ad85-a5f1d8f006eb"}, { new NonTerminator(NonTerminatorType.Term), "bab97185-47af-4ddd-8275-1863e6c37e23"}, { new NonTerminator(NonTerminatorType.Factor), "f5d56db2-25f1-4001-abce-4572f92e534a"}, { Terminator.NumberTerminator, "35cb5529-4f39-4b7c-a937-233f6efe6cdb"}, { new NonTerminator(NonTerminatorType.Variable), "ca1b9fbe-0a27-4b7d-ad6a-ee1ea8f20f5f"}, { new Terminator(DelimiterType.LeftParenthesis), "5f4ba076-b3cf-4472-848c-74d2a618eacb"}, { Terminator.IdentifierTerminator, "99071775-1827-45c4-b161-826ababf0384"}, { new Terminator(KeywordType.Not), "ed5ffdaa-b277-4c22-ad93-e4b1ed293ae4"}, { new Terminator(OperatorType.Minus), "15dc2c62-c938-4d0a-b0aa-1f0a0b73661f"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ac97e816-8114-4188-89f5-7b607e0c31ef") },
		{ "3bb0a54a-8410-4eed-9e88-4a1811f69f01", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "3bb0a54a-8410-4eed-9e88-4a1811f69f01") },
		{ "576b6c29-5517-4fca-9771-e7631bee026d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "f3816498-e846-4718-87b1-fa06bfe1a33c"}, { new NonTerminator(NonTerminatorType.Expression), "2e88b57e-7c35-4aee-9387-ff1c061cb7ab"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "089240b9-2be9-4199-97a8-41a6a9ac2f54"}, { new NonTerminator(NonTerminatorType.Term), "f46a091e-b991-43bd-877a-3c2eef645ab7"}, { new NonTerminator(NonTerminatorType.Factor), "5cb8d05c-bce3-47cc-8728-d7c037f2e9b1"}, { Terminator.NumberTerminator, "3928c790-d1f1-4d6a-8e98-6a0d0a3182e7"}, { new NonTerminator(NonTerminatorType.Variable), "08f29b35-75b5-4fe9-b677-ed18c2caa51b"}, { new Terminator(DelimiterType.LeftParenthesis), "7bb9f1fe-edd2-4812-9091-30ae6a61e59d"}, { Terminator.IdentifierTerminator, "ad126546-60a0-4982-8f68-c9a727b1c223"}, { new Terminator(KeywordType.Not), "d17a2e13-31b3-4e22-a0a6-5cf5fa910500"}, { new Terminator(OperatorType.Minus), "e8e4e781-e847-4c03-b1c8-fc377a1a86f7"},}, new Dictionary<Terminator, ReduceInformation>{ }, "576b6c29-5517-4fca-9771-e7631bee026d") },
		{ "daad863e-bcd3-4cae-8020-518f4d7d96d0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "daad863e-bcd3-4cae-8020-518f4d7d96d0") },
		{ "6f3ea32e-ec21-430a-8d5b-8b28f6f36572", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "6f3ea32e-ec21-430a-8d5b-8b28f6f36572") },
		{ "891d0e39-5ba0-45da-ab22-7b27cc04350f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "891d0e39-5ba0-45da-ab22-7b27cc04350f") },
		{ "a7d27889-a27b-4e71-8b0c-4a6a830b7695", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "a7d27889-a27b-4e71-8b0c-4a6a830b7695") },
		{ "dbab63d2-df35-428a-b5cc-2dd2e2524f08", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "da7433d4-48c3-4fa4-9cf8-ae06585d0f46"}, { new NonTerminator(NonTerminatorType.Factor), "e6af18a0-1b24-4fe5-b501-7bf4ba96defd"}, { Terminator.NumberTerminator, "ba493efc-5d7d-425a-ab17-3057add7633e"}, { new NonTerminator(NonTerminatorType.Variable), "e937950c-aa65-4d42-97a8-6d55502689bf"}, { new Terminator(DelimiterType.LeftParenthesis), "e3448ecf-4866-44ba-a34b-af6d031ad38d"}, { Terminator.IdentifierTerminator, "685e6fcf-000d-47c8-aee8-cc1d8fee08c3"}, { new Terminator(KeywordType.Not), "814b64e0-3336-426d-8775-15f9eddce24e"}, { new Terminator(OperatorType.Minus), "910b40b6-c2d2-4c5a-8567-9da1ff7eee9f"},}, new Dictionary<Terminator, ReduceInformation>{ }, "dbab63d2-df35-428a-b5cc-2dd2e2524f08") },
		{ "72099dbe-7244-4758-92c1-c936b2f150d3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "7ec91a7c-0eaa-4a38-bf8e-523eb640d538"}, { Terminator.NumberTerminator, "ba493efc-5d7d-425a-ab17-3057add7633e"}, { new NonTerminator(NonTerminatorType.Variable), "e937950c-aa65-4d42-97a8-6d55502689bf"}, { new Terminator(DelimiterType.LeftParenthesis), "e3448ecf-4866-44ba-a34b-af6d031ad38d"}, { Terminator.IdentifierTerminator, "685e6fcf-000d-47c8-aee8-cc1d8fee08c3"}, { new Terminator(KeywordType.Not), "814b64e0-3336-426d-8775-15f9eddce24e"}, { new Terminator(OperatorType.Minus), "910b40b6-c2d2-4c5a-8567-9da1ff7eee9f"},}, new Dictionary<Terminator, ReduceInformation>{ }, "72099dbe-7244-4758-92c1-c936b2f150d3") },
		{ "9f5e2fb3-7703-40e8-a983-7ad7f3424016", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "b4af54eb-e516-4dd0-85e0-48113f9465ce"},}, new Dictionary<Terminator, ReduceInformation>{ }, "9f5e2fb3-7703-40e8-a983-7ad7f3424016") },
		{ "be34b8ce-e4cf-4326-a6c0-e7d413a828c9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "893254fe-3b97-43eb-9313-55cdd8bad4cc"}, { new NonTerminator(NonTerminatorType.Expression), "2e7bde52-48a5-420d-a6e3-9f664510da68"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "17428d1e-ed4e-461b-ad85-a5f1d8f006eb"}, { new NonTerminator(NonTerminatorType.Term), "bab97185-47af-4ddd-8275-1863e6c37e23"}, { new NonTerminator(NonTerminatorType.Factor), "f5d56db2-25f1-4001-abce-4572f92e534a"}, { Terminator.NumberTerminator, "35cb5529-4f39-4b7c-a937-233f6efe6cdb"}, { new NonTerminator(NonTerminatorType.Variable), "ca1b9fbe-0a27-4b7d-ad6a-ee1ea8f20f5f"}, { new Terminator(DelimiterType.LeftParenthesis), "5f4ba076-b3cf-4472-848c-74d2a618eacb"}, { Terminator.IdentifierTerminator, "99071775-1827-45c4-b161-826ababf0384"}, { new Terminator(KeywordType.Not), "ed5ffdaa-b277-4c22-ad93-e4b1ed293ae4"}, { new Terminator(OperatorType.Minus), "15dc2c62-c938-4d0a-b0aa-1f0a0b73661f"},}, new Dictionary<Terminator, ReduceInformation>{ }, "be34b8ce-e4cf-4326-a6c0-e7d413a828c9") },
		{ "269fa658-7c13-48b0-a885-b40f67446a14", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "269fa658-7c13-48b0-a885-b40f67446a14") },
		{ "79745d95-9048-43ff-9f8b-40f7d392845f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "8eed3d68-4201-4d80-a06b-4f1f949ceba5"}, { new NonTerminator(NonTerminatorType.Expression), "2e88b57e-7c35-4aee-9387-ff1c061cb7ab"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "089240b9-2be9-4199-97a8-41a6a9ac2f54"}, { new NonTerminator(NonTerminatorType.Term), "f46a091e-b991-43bd-877a-3c2eef645ab7"}, { new NonTerminator(NonTerminatorType.Factor), "5cb8d05c-bce3-47cc-8728-d7c037f2e9b1"}, { Terminator.NumberTerminator, "3928c790-d1f1-4d6a-8e98-6a0d0a3182e7"}, { new NonTerminator(NonTerminatorType.Variable), "08f29b35-75b5-4fe9-b677-ed18c2caa51b"}, { new Terminator(DelimiterType.LeftParenthesis), "7bb9f1fe-edd2-4812-9091-30ae6a61e59d"}, { Terminator.IdentifierTerminator, "ad126546-60a0-4982-8f68-c9a727b1c223"}, { new Terminator(KeywordType.Not), "d17a2e13-31b3-4e22-a0a6-5cf5fa910500"}, { new Terminator(OperatorType.Minus), "e8e4e781-e847-4c03-b1c8-fc377a1a86f7"},}, new Dictionary<Terminator, ReduceInformation>{ }, "79745d95-9048-43ff-9f8b-40f7d392845f") },
		{ "8a847411-8db9-414d-8fd9-3188aea86f56", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "8a847411-8db9-414d-8fd9-3188aea86f56") },
		{ "af4ff363-b5b6-4610-b43a-051015e34257", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "af4ff363-b5b6-4610-b43a-051015e34257") },
		{ "acb92439-8833-498e-b71b-bcca5ab90adf", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "acb92439-8833-498e-b71b-bcca5ab90adf") },
		{ "3fee0f49-bfd3-41cd-8e1d-5c622b5503b1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "3fee0f49-bfd3-41cd-8e1d-5c622b5503b1") },
		{ "76688848-6c3e-483f-b362-f7c0d3904500", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "47b44660-5b1c-458a-80bc-a4be9775dcfe"}, { new NonTerminator(NonTerminatorType.Factor), "08be8a9e-6d3e-471f-96fd-089d3fd390aa"}, { Terminator.NumberTerminator, "cd6de66d-c8ca-4eb3-8314-72445a11c5d4"}, { new NonTerminator(NonTerminatorType.Variable), "61f16867-4d7b-413e-b862-5468cafac64b"}, { new Terminator(DelimiterType.LeftParenthesis), "d08f77fa-a766-40ac-9825-0f7480e8c974"}, { Terminator.IdentifierTerminator, "1483a6f8-0714-4bb8-93f7-d0445f241849"}, { new Terminator(KeywordType.Not), "65db3e18-ac36-4517-98e5-6d7090d47fe9"}, { new Terminator(OperatorType.Minus), "8880f166-e99b-47c7-8de6-d2c669de9dc4"},}, new Dictionary<Terminator, ReduceInformation>{ }, "76688848-6c3e-483f-b362-f7c0d3904500") },
		{ "9b50ef94-e2d1-4693-aaac-d1a529701978", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "12ce3d0c-2b67-468f-b6e8-52f5e717ccbe"}, { Terminator.NumberTerminator, "cd6de66d-c8ca-4eb3-8314-72445a11c5d4"}, { new NonTerminator(NonTerminatorType.Variable), "61f16867-4d7b-413e-b862-5468cafac64b"}, { new Terminator(DelimiterType.LeftParenthesis), "d08f77fa-a766-40ac-9825-0f7480e8c974"}, { Terminator.IdentifierTerminator, "1483a6f8-0714-4bb8-93f7-d0445f241849"}, { new Terminator(KeywordType.Not), "65db3e18-ac36-4517-98e5-6d7090d47fe9"}, { new Terminator(OperatorType.Minus), "8880f166-e99b-47c7-8de6-d2c669de9dc4"},}, new Dictionary<Terminator, ReduceInformation>{ }, "9b50ef94-e2d1-4693-aaac-d1a529701978") },
		{ "eea72f37-bfd6-4309-a5c9-13ebd76e9c42", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "3fedf865-d1f4-48af-bfeb-bbe85aada635"},}, new Dictionary<Terminator, ReduceInformation>{ }, "eea72f37-bfd6-4309-a5c9-13ebd76e9c42") },
		{ "378b88c2-0901-48ae-b1ab-24bf24f0ea8b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "48d0c436-828e-452f-a4a8-50f43be27808"}, { new NonTerminator(NonTerminatorType.Expression), "2e7bde52-48a5-420d-a6e3-9f664510da68"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "17428d1e-ed4e-461b-ad85-a5f1d8f006eb"}, { new NonTerminator(NonTerminatorType.Term), "bab97185-47af-4ddd-8275-1863e6c37e23"}, { new NonTerminator(NonTerminatorType.Factor), "f5d56db2-25f1-4001-abce-4572f92e534a"}, { Terminator.NumberTerminator, "35cb5529-4f39-4b7c-a937-233f6efe6cdb"}, { new NonTerminator(NonTerminatorType.Variable), "ca1b9fbe-0a27-4b7d-ad6a-ee1ea8f20f5f"}, { new Terminator(DelimiterType.LeftParenthesis), "5f4ba076-b3cf-4472-848c-74d2a618eacb"}, { Terminator.IdentifierTerminator, "99071775-1827-45c4-b161-826ababf0384"}, { new Terminator(KeywordType.Not), "ed5ffdaa-b277-4c22-ad93-e4b1ed293ae4"}, { new Terminator(OperatorType.Minus), "15dc2c62-c938-4d0a-b0aa-1f0a0b73661f"},}, new Dictionary<Terminator, ReduceInformation>{ }, "378b88c2-0901-48ae-b1ab-24bf24f0ea8b") },
		{ "f8430992-f55c-4ba9-943f-c7183f2f8bf9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "f8430992-f55c-4ba9-943f-c7183f2f8bf9") },
		{ "ffb7bd7a-7e9f-4b96-bf49-c595094a3b0e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "282d579c-461d-45a2-9531-5ab79224cb85"}, { new NonTerminator(NonTerminatorType.Expression), "2e88b57e-7c35-4aee-9387-ff1c061cb7ab"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "089240b9-2be9-4199-97a8-41a6a9ac2f54"}, { new NonTerminator(NonTerminatorType.Term), "f46a091e-b991-43bd-877a-3c2eef645ab7"}, { new NonTerminator(NonTerminatorType.Factor), "5cb8d05c-bce3-47cc-8728-d7c037f2e9b1"}, { Terminator.NumberTerminator, "3928c790-d1f1-4d6a-8e98-6a0d0a3182e7"}, { new NonTerminator(NonTerminatorType.Variable), "08f29b35-75b5-4fe9-b677-ed18c2caa51b"}, { new Terminator(DelimiterType.LeftParenthesis), "7bb9f1fe-edd2-4812-9091-30ae6a61e59d"}, { Terminator.IdentifierTerminator, "ad126546-60a0-4982-8f68-c9a727b1c223"}, { new Terminator(KeywordType.Not), "d17a2e13-31b3-4e22-a0a6-5cf5fa910500"}, { new Terminator(OperatorType.Minus), "e8e4e781-e847-4c03-b1c8-fc377a1a86f7"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ffb7bd7a-7e9f-4b96-bf49-c595094a3b0e") },
		{ "f88ad60c-073f-4b63-be18-135959523e95", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "f88ad60c-073f-4b63-be18-135959523e95") },
		{ "2f2f05cc-bdcf-4457-a43f-1f40592ec5d4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "2f2f05cc-bdcf-4457-a43f-1f40592ec5d4") },
		{ "369d713c-aaa9-4c82-a597-77c4aef39d50", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "369d713c-aaa9-4c82-a597-77c4aef39d50") },
		{ "d536904a-9fa6-434b-9cb5-f7eb5ca3af1c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "d536904a-9fa6-434b-9cb5-f7eb5ca3af1c") },
		{ "25d6bc40-dc69-4f2b-9976-ee660d147555", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.ElsePart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.ElsePart))}, }, "25d6bc40-dc69-4f2b-9976-ee660d147555") },
		{ "764f6824-9035-43d0-932b-067e544093c8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Statement))}, }, "764f6824-9035-43d0-932b-067e544093c8") },
		{ "5d725b4b-caf4-4222-a977-00ef92cec269", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.RelationOperator), "73da82a8-6ec8-4cfa-a4c7-52162174ccac"}, { new Terminator(OperatorType.Equal), "5cba3bbd-2f4e-4072-ab57-6933c47cf686"}, { new Terminator(OperatorType.NotEqual), "f19c1f01-646c-448a-a706-47091a7f6df7"}, { new Terminator(OperatorType.Less), "94b6c574-1915-47b0-b2de-d2c73eb5c1c2"}, { new Terminator(OperatorType.LessEqual), "0f2fec62-57bd-4061-9dff-9bcc1114b3a4"}, { new Terminator(OperatorType.Greater), "14be83c5-2fdf-4057-b5ed-b5bb521a20a2"}, { new Terminator(OperatorType.GreaterEqual), "83ce7741-272c-4749-8008-fcc200294ed8"}, { new NonTerminator(NonTerminatorType.AddOperator), "59e6221e-b909-4f1d-b0d2-2d53ddfb8baf"}, { new Terminator(OperatorType.Plus), "2cf28f7e-6888-4e54-9825-8a80b00c7a56"}, { new Terminator(OperatorType.Minus), "1804ef7c-da92-431e-b725-ea7823b69327"}, { new Terminator(KeywordType.Or), "e777b812-5ca1-4317-9fd4-cc37fbcb2af0"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, }, "5d725b4b-caf4-4222-a977-00ef92cec269") },
		{ "a783da79-bb67-4798-969d-9ba08a5268e8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "6b7d3e6e-5966-4384-a88d-8c063e753c57"}, { new Terminator(OperatorType.Multiply), "4d79eeb4-6b1a-4ca0-97f6-1f6675aed09f"}, { new Terminator(OperatorType.Divide), "fecb6832-79a2-4f95-95c4-fc698109ba3e"}, { new Terminator(KeywordType.Divide), "784cd050-c0ae-4844-8706-7f532d156c70"}, { new Terminator(KeywordType.Mod), "f3fdd777-da4f-4c74-9196-b8c4d7b7bace"}, { new Terminator(KeywordType.And), "376ab083-df94-433e-bcac-f300687907b1"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "a783da79-bb67-4798-969d-9ba08a5268e8") },
		{ "84eb5d09-6a9c-4152-8ee1-08cf1977b4b4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "84eb5d09-6a9c-4152-8ee1-08cf1977b4b4") },
		{ "8695b530-26f3-4534-a7ec-6e01b3b8b90c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "8695b530-26f3-4534-a7ec-6e01b3b8b90c") },
		{ "dd2136a1-9348-4c93-a3d6-cd3454e51c22", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "dd2136a1-9348-4c93-a3d6-cd3454e51c22") },
		{ "d1159160-d982-4e79-9cb1-37896891fa6b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "6c171c98-ed71-468e-a69a-3ca45f08c88d"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "550fbb43-0150-468f-9eb4-7bf5912daa0a"}, { new NonTerminator(NonTerminatorType.Term), "f0cad299-28f5-4e84-b9e7-2bc048f33840"}, { new NonTerminator(NonTerminatorType.Factor), "43e04c0c-8568-4c6b-ab2b-bceedcc9fa32"}, { Terminator.NumberTerminator, "c04070de-7d08-4aad-b5d7-41002b95f79d"}, { new NonTerminator(NonTerminatorType.Variable), "448fb14e-b117-467e-94e2-0ffad798acf2"}, { new Terminator(DelimiterType.LeftParenthesis), "61941794-1fbf-4cdd-a20f-059c95559fcc"}, { Terminator.IdentifierTerminator, "e932245d-2b23-4f7c-b6a4-c6fbd2b3dd9c"}, { new Terminator(KeywordType.Not), "8fcb5991-4a57-4bc9-b6ed-c1c6b22af95e"}, { new Terminator(OperatorType.Minus), "7312a539-9334-411e-b2f6-dc420da87c55"},}, new Dictionary<Terminator, ReduceInformation>{ }, "d1159160-d982-4e79-9cb1-37896891fa6b") },
		{ "74fa8c5c-9ee2-4481-b6b9-1cf08e5fd510", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "3aef74b6-a29a-43fc-854a-653867e15a25"}, { new NonTerminator(NonTerminatorType.IdVarPart), "6ca30117-d0b1-4275-9308-1c3c50834c65"}, { new Terminator(DelimiterType.LeftSquareBracket), "80ebbe93-93fc-4242-8cb0-18b7b1273475"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Else), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "74fa8c5c-9ee2-4481-b6b9-1cf08e5fd510") },
		{ "d2ee92ce-60e1-42c5-93bb-b132c2796e16", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "1e78bd7f-21e6-43a0-b91b-da2faec0dc95"}, { Terminator.NumberTerminator, "8695b530-26f3-4534-a7ec-6e01b3b8b90c"}, { new NonTerminator(NonTerminatorType.Variable), "dd2136a1-9348-4c93-a3d6-cd3454e51c22"}, { new Terminator(DelimiterType.LeftParenthesis), "d1159160-d982-4e79-9cb1-37896891fa6b"}, { Terminator.IdentifierTerminator, "74fa8c5c-9ee2-4481-b6b9-1cf08e5fd510"}, { new Terminator(KeywordType.Not), "d2ee92ce-60e1-42c5-93bb-b132c2796e16"}, { new Terminator(OperatorType.Minus), "410a8465-e4b5-4ba9-b1b4-fc3e3f81080e"},}, new Dictionary<Terminator, ReduceInformation>{ }, "d2ee92ce-60e1-42c5-93bb-b132c2796e16") },
		{ "410a8465-e4b5-4ba9-b1b4-fc3e3f81080e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "881bcf0b-7877-4393-be62-590162979146"}, { Terminator.NumberTerminator, "8695b530-26f3-4534-a7ec-6e01b3b8b90c"}, { new NonTerminator(NonTerminatorType.Variable), "dd2136a1-9348-4c93-a3d6-cd3454e51c22"}, { new Terminator(DelimiterType.LeftParenthesis), "d1159160-d982-4e79-9cb1-37896891fa6b"}, { Terminator.IdentifierTerminator, "74fa8c5c-9ee2-4481-b6b9-1cf08e5fd510"}, { new Terminator(KeywordType.Not), "d2ee92ce-60e1-42c5-93bb-b132c2796e16"}, { new Terminator(OperatorType.Minus), "410a8465-e4b5-4ba9-b1b4-fc3e3f81080e"},}, new Dictionary<Terminator, ReduceInformation>{ }, "410a8465-e4b5-4ba9-b1b4-fc3e3f81080e") },
		{ "e1c9e4a0-9cd0-4cf8-bbf4-0838f0657394", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Statement))}, }, "e1c9e4a0-9cd0-4cf8-bbf4-0838f0657394") },
		{ "6dd2e068-a1b5-445d-858d-af446583ed22", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "9181974f-69f2-4398-9909-9adaf29f8ece"}, { new Terminator(DelimiterType.Comma), "f8c6158b-c7b5-4520-8a37-49ad141ff150"},}, new Dictionary<Terminator, ReduceInformation>{ }, "6dd2e068-a1b5-445d-858d-af446583ed22") },
		{ "1cb0b7ad-0584-48f2-abe2-b5e18f1e1baa", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Statement), "95ca7b97-7e0d-400d-9762-4826cf7f4f49"}, { new NonTerminator(NonTerminatorType.Variable), "ea208c56-799b-4542-aab9-97041337f8cd"}, { Terminator.IdentifierTerminator, "9a4e95ea-b154-4175-98e3-a2691d4eb20f"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "14f3970e-268e-4251-aa12-eaa8effa762a"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "b1815343-33db-47b4-8679-35c240abe9b0"}, { new Terminator(KeywordType.If), "e3fc592c-2252-423a-8c29-eb6efd3e239e"}, { new Terminator(KeywordType.For), "35582002-f9e8-40c5-b5fe-930d8a63d94a"}, { new Terminator(KeywordType.Begin), "09269d2d-ccde-4852-8c11-5c03f8f1f9f2"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "1cb0b7ad-0584-48f2-abe2-b5e18f1e1baa") },
		{ "982b4453-5b96-4fcb-9512-e3090089f0c2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "52a8b6bf-9f1a-47f1-82e8-da3979aa3d77"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "3e55bf24-5789-49ed-a5ae-5eb7ec7347cd"}, { new NonTerminator(NonTerminatorType.Term), "08443152-f969-4da6-aa5b-97e3b3345fc9"}, { new NonTerminator(NonTerminatorType.Factor), "c901372b-e50e-432c-ac38-9f6def8240bb"}, { Terminator.NumberTerminator, "d2a4c365-b1fc-48c2-aba8-dcd35bc4a13f"}, { new NonTerminator(NonTerminatorType.Variable), "0cb8880d-7fd6-442b-97b7-5939869faac7"}, { new Terminator(DelimiterType.LeftParenthesis), "571d117e-bca5-4cec-a8aa-99b81b821045"}, { Terminator.IdentifierTerminator, "6f5f06f1-5c16-4c28-aa0b-a254c725d359"}, { new Terminator(KeywordType.Not), "c90dca67-28e6-41ad-b475-ab75f98bff0f"}, { new Terminator(OperatorType.Minus), "48762e8c-76c8-477d-8b60-3bb834e285b3"},}, new Dictionary<Terminator, ReduceInformation>{ }, "982b4453-5b96-4fcb-9512-e3090089f0c2") },
		{ "272221c9-80f3-4475-84a7-c85130ddca87", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.CompoundStatement))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.CompoundStatement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.CompoundStatement))}, }, "272221c9-80f3-4475-84a7-c85130ddca87") },
		{ "cfc69cbe-ab80-4316-b80c-76dfaa865934", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "02ea126a-ea08-4518-984c-8f376afc1b38"}, { new Terminator(OperatorType.Multiply), "4d79eeb4-6b1a-4ca0-97f6-1f6675aed09f"}, { new Terminator(OperatorType.Divide), "fecb6832-79a2-4f95-95c4-fc698109ba3e"}, { new Terminator(KeywordType.Divide), "784cd050-c0ae-4844-8706-7f532d156c70"}, { new Terminator(KeywordType.Mod), "f3fdd777-da4f-4c74-9196-b8c4d7b7bace"}, { new Terminator(KeywordType.And), "376ab083-df94-433e-bcac-f300687907b1"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "cfc69cbe-ab80-4316-b80c-76dfaa865934") },
		{ "aef41990-e07f-40e0-8bef-1a2bb868b834", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "aef41990-e07f-40e0-8bef-1a2bb868b834") },
		{ "a614a956-b58d-4554-8e34-772753c618be", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "a614a956-b58d-4554-8e34-772753c618be") },
		{ "73f75d0e-0fa3-43ca-a75d-47bc4224c8eb", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "fabb50fe-9b49-4d81-84a8-cf9fa2ee4df4"}, { new Terminator(DelimiterType.Comma), "f8c6158b-c7b5-4520-8a37-49ad141ff150"},}, new Dictionary<Terminator, ReduceInformation>{ }, "73f75d0e-0fa3-43ca-a75d-47bc4224c8eb") },
		{ "f3892946-11cd-44eb-a569-7cf3cf854b6c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "5ce1e180-7ce4-49c8-8179-3a766e92d767"}, { new Terminator(DelimiterType.Comma), "61999b62-48c1-4d86-8277-f2ec86cd3141"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f3892946-11cd-44eb-a569-7cf3cf854b6c") },
		{ "6720fbfb-d07f-4e2d-9d03-8f2af5594db0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "b1c2ba9f-5e9d-4c3b-932a-9d5f8e776a41"}, { new NonTerminator(NonTerminatorType.Factor), "a10796f7-837e-4ca0-9010-6fb4ffbac4bb"}, { Terminator.NumberTerminator, "907327ce-1671-4244-ba93-3de261d8481f"}, { new NonTerminator(NonTerminatorType.Variable), "45c1cf08-1190-49ce-8ef4-f9e3b94b8c01"}, { new Terminator(DelimiterType.LeftParenthesis), "51c9eb00-0d9b-46c5-9551-93b200bcda53"}, { Terminator.IdentifierTerminator, "52164dca-2b28-47de-ab17-a0588c9bb84a"}, { new Terminator(KeywordType.Not), "9bce9b84-8a03-41d5-a1ab-ac98d336a010"}, { new Terminator(OperatorType.Minus), "dc93c5ce-0987-492a-ac7e-928dd81cfb43"},}, new Dictionary<Terminator, ReduceInformation>{ }, "6720fbfb-d07f-4e2d-9d03-8f2af5594db0") },
		{ "4d0b17ae-8185-4259-8a8d-16432387f97f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "ac830f96-6bde-49f0-b548-93cb338306f0"}, { Terminator.NumberTerminator, "907327ce-1671-4244-ba93-3de261d8481f"}, { new NonTerminator(NonTerminatorType.Variable), "45c1cf08-1190-49ce-8ef4-f9e3b94b8c01"}, { new Terminator(DelimiterType.LeftParenthesis), "51c9eb00-0d9b-46c5-9551-93b200bcda53"}, { Terminator.IdentifierTerminator, "52164dca-2b28-47de-ab17-a0588c9bb84a"}, { new Terminator(KeywordType.Not), "9bce9b84-8a03-41d5-a1ab-ac98d336a010"}, { new Terminator(OperatorType.Minus), "dc93c5ce-0987-492a-ac7e-928dd81cfb43"},}, new Dictionary<Terminator, ReduceInformation>{ }, "4d0b17ae-8185-4259-8a8d-16432387f97f") },
		{ "519745f5-6613-4db1-bfc2-019209bf796b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "567a4123-b5e2-494b-9d1a-6753d7e88083"},}, new Dictionary<Terminator, ReduceInformation>{ }, "519745f5-6613-4db1-bfc2-019209bf796b") },
		{ "a2b9e513-db1a-4804-a121-38f1876d137c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "ffb7db92-10ca-4be1-8be1-e0a960c3b0af"}, { new NonTerminator(NonTerminatorType.Expression), "2e7bde52-48a5-420d-a6e3-9f664510da68"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "17428d1e-ed4e-461b-ad85-a5f1d8f006eb"}, { new NonTerminator(NonTerminatorType.Term), "bab97185-47af-4ddd-8275-1863e6c37e23"}, { new NonTerminator(NonTerminatorType.Factor), "f5d56db2-25f1-4001-abce-4572f92e534a"}, { Terminator.NumberTerminator, "35cb5529-4f39-4b7c-a937-233f6efe6cdb"}, { new NonTerminator(NonTerminatorType.Variable), "ca1b9fbe-0a27-4b7d-ad6a-ee1ea8f20f5f"}, { new Terminator(DelimiterType.LeftParenthesis), "5f4ba076-b3cf-4472-848c-74d2a618eacb"}, { Terminator.IdentifierTerminator, "99071775-1827-45c4-b161-826ababf0384"}, { new Terminator(KeywordType.Not), "ed5ffdaa-b277-4c22-ad93-e4b1ed293ae4"}, { new Terminator(OperatorType.Minus), "15dc2c62-c938-4d0a-b0aa-1f0a0b73661f"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a2b9e513-db1a-4804-a121-38f1876d137c") },
		{ "e9a81368-5389-482a-8972-f02d89229728", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "e9a81368-5389-482a-8972-f02d89229728") },
		{ "a6abda55-a744-4aff-b76c-631bb4fc4d3d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "af467ff1-300f-4027-98bc-61d2da0ae1ed"}, { new NonTerminator(NonTerminatorType.Expression), "2e88b57e-7c35-4aee-9387-ff1c061cb7ab"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "089240b9-2be9-4199-97a8-41a6a9ac2f54"}, { new NonTerminator(NonTerminatorType.Term), "f46a091e-b991-43bd-877a-3c2eef645ab7"}, { new NonTerminator(NonTerminatorType.Factor), "5cb8d05c-bce3-47cc-8728-d7c037f2e9b1"}, { Terminator.NumberTerminator, "3928c790-d1f1-4d6a-8e98-6a0d0a3182e7"}, { new NonTerminator(NonTerminatorType.Variable), "08f29b35-75b5-4fe9-b677-ed18c2caa51b"}, { new Terminator(DelimiterType.LeftParenthesis), "7bb9f1fe-edd2-4812-9091-30ae6a61e59d"}, { Terminator.IdentifierTerminator, "ad126546-60a0-4982-8f68-c9a727b1c223"}, { new Terminator(KeywordType.Not), "d17a2e13-31b3-4e22-a0a6-5cf5fa910500"}, { new Terminator(OperatorType.Minus), "e8e4e781-e847-4c03-b1c8-fc377a1a86f7"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a6abda55-a744-4aff-b76c-631bb4fc4d3d") },
		{ "9b98e7e0-086c-4fec-bf8e-ca82f5c9fabe", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "9b98e7e0-086c-4fec-bf8e-ca82f5c9fabe") },
		{ "ac9b22f6-41e1-4f6e-95d6-ce70d76fe888", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "ac9b22f6-41e1-4f6e-95d6-ce70d76fe888") },
		{ "94db1226-8145-43bc-8c98-c4b1589b5665", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "94db1226-8145-43bc-8c98-c4b1589b5665") },
		{ "b79e2959-9ba5-42ce-a941-ad533e21d0a8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "b79e2959-9ba5-42ce-a941-ad533e21d0a8") },
		{ "4fe9222b-5e17-471b-a4f2-d7a29c2d05ec", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.Do), "e321f889-9cdd-4db4-9986-78df04737558"},}, new Dictionary<Terminator, ReduceInformation>{ }, "4fe9222b-5e17-471b-a4f2-d7a29c2d05ec") },
		{ "caa8525c-e146-4f2e-9489-00c244cfad57", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.RelationOperator), "221d48f5-6183-4f2f-b33b-4ebac32a134b"}, { new Terminator(OperatorType.Equal), "5cba3bbd-2f4e-4072-ab57-6933c47cf686"}, { new Terminator(OperatorType.NotEqual), "f19c1f01-646c-448a-a706-47091a7f6df7"}, { new Terminator(OperatorType.Less), "94b6c574-1915-47b0-b2de-d2c73eb5c1c2"}, { new Terminator(OperatorType.LessEqual), "0f2fec62-57bd-4061-9dff-9bcc1114b3a4"}, { new Terminator(OperatorType.Greater), "14be83c5-2fdf-4057-b5ed-b5bb521a20a2"}, { new Terminator(OperatorType.GreaterEqual), "83ce7741-272c-4749-8008-fcc200294ed8"}, { new NonTerminator(NonTerminatorType.AddOperator), "7854af7d-22e7-4d14-bd30-b994c816eb01"}, { new Terminator(OperatorType.Plus), "2cf28f7e-6888-4e54-9825-8a80b00c7a56"}, { new Terminator(OperatorType.Minus), "1804ef7c-da92-431e-b725-ea7823b69327"}, { new Terminator(KeywordType.Or), "e777b812-5ca1-4317-9fd4-cc37fbcb2af0"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, }, "caa8525c-e146-4f2e-9489-00c244cfad57") },
		{ "54afe343-cda5-4f84-aa6d-5a21b405f421", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "fa1484e0-5a30-4654-a01f-8e372ea2fa7e"}, { new Terminator(OperatorType.Multiply), "4d79eeb4-6b1a-4ca0-97f6-1f6675aed09f"}, { new Terminator(OperatorType.Divide), "fecb6832-79a2-4f95-95c4-fc698109ba3e"}, { new Terminator(KeywordType.Divide), "784cd050-c0ae-4844-8706-7f532d156c70"}, { new Terminator(KeywordType.Mod), "f3fdd777-da4f-4c74-9196-b8c4d7b7bace"}, { new Terminator(KeywordType.And), "376ab083-df94-433e-bcac-f300687907b1"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "54afe343-cda5-4f84-aa6d-5a21b405f421") },
		{ "e0658f9a-921e-4b74-8fb5-e6b77d907947", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "e0658f9a-921e-4b74-8fb5-e6b77d907947") },
		{ "799b1e98-e544-4cd9-b48d-386861187fe6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "799b1e98-e544-4cd9-b48d-386861187fe6") },
		{ "71ecda72-cc64-4040-8496-5a1c0ec09b77", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "71ecda72-cc64-4040-8496-5a1c0ec09b77") },
		{ "c2439980-49fc-48d9-9626-5ba6e380cde7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "50bfb7d9-979d-4726-9a67-74a8202fdca3"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "550fbb43-0150-468f-9eb4-7bf5912daa0a"}, { new NonTerminator(NonTerminatorType.Term), "f0cad299-28f5-4e84-b9e7-2bc048f33840"}, { new NonTerminator(NonTerminatorType.Factor), "43e04c0c-8568-4c6b-ab2b-bceedcc9fa32"}, { Terminator.NumberTerminator, "c04070de-7d08-4aad-b5d7-41002b95f79d"}, { new NonTerminator(NonTerminatorType.Variable), "448fb14e-b117-467e-94e2-0ffad798acf2"}, { new Terminator(DelimiterType.LeftParenthesis), "61941794-1fbf-4cdd-a20f-059c95559fcc"}, { Terminator.IdentifierTerminator, "e932245d-2b23-4f7c-b6a4-c6fbd2b3dd9c"}, { new Terminator(KeywordType.Not), "8fcb5991-4a57-4bc9-b6ed-c1c6b22af95e"}, { new Terminator(OperatorType.Minus), "7312a539-9334-411e-b2f6-dc420da87c55"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c2439980-49fc-48d9-9626-5ba6e380cde7") },
		{ "820af974-dbec-4e03-8931-dfc4ef51c15a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "a6590b22-d882-4595-bf8e-44c11f64da01"}, { new NonTerminator(NonTerminatorType.IdVarPart), "d6d654a3-e442-4123-b40a-45fb135e8016"}, { new Terminator(DelimiterType.LeftSquareBracket), "33b650ee-e803-4027-afae-79827cbeb38a"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "820af974-dbec-4e03-8931-dfc4ef51c15a") },
		{ "c7800d99-09f1-4c4b-a492-985868834e26", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "e4c05e25-1793-4efa-b58c-19f0283a7b8a"}, { Terminator.NumberTerminator, "799b1e98-e544-4cd9-b48d-386861187fe6"}, { new NonTerminator(NonTerminatorType.Variable), "71ecda72-cc64-4040-8496-5a1c0ec09b77"}, { new Terminator(DelimiterType.LeftParenthesis), "c2439980-49fc-48d9-9626-5ba6e380cde7"}, { Terminator.IdentifierTerminator, "820af974-dbec-4e03-8931-dfc4ef51c15a"}, { new Terminator(KeywordType.Not), "c7800d99-09f1-4c4b-a492-985868834e26"}, { new Terminator(OperatorType.Minus), "e1f14f62-b8e2-44d0-9ca8-130cecab42e5"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c7800d99-09f1-4c4b-a492-985868834e26") },
		{ "e1f14f62-b8e2-44d0-9ca8-130cecab42e5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "2cc3b96f-2e3c-4e25-9394-12a8c2db113f"}, { Terminator.NumberTerminator, "799b1e98-e544-4cd9-b48d-386861187fe6"}, { new NonTerminator(NonTerminatorType.Variable), "71ecda72-cc64-4040-8496-5a1c0ec09b77"}, { new Terminator(DelimiterType.LeftParenthesis), "c2439980-49fc-48d9-9626-5ba6e380cde7"}, { Terminator.IdentifierTerminator, "820af974-dbec-4e03-8931-dfc4ef51c15a"}, { new Terminator(KeywordType.Not), "c7800d99-09f1-4c4b-a492-985868834e26"}, { new Terminator(OperatorType.Minus), "e1f14f62-b8e2-44d0-9ca8-130cecab42e5"},}, new Dictionary<Terminator, ReduceInformation>{ }, "e1f14f62-b8e2-44d0-9ca8-130cecab42e5") },
		{ "c30bcfe6-462c-4961-b7d2-f2f81fc8b911", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.AddOperator), "c395cf12-2ca1-471d-b748-264d25b9258c"}, { new Terminator(OperatorType.Plus), "2cf28f7e-6888-4e54-9825-8a80b00c7a56"}, { new Terminator(OperatorType.Minus), "1804ef7c-da92-431e-b725-ea7823b69327"}, { new Terminator(KeywordType.Or), "e777b812-5ca1-4317-9fd4-cc37fbcb2af0"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, }, "c30bcfe6-462c-4961-b7d2-f2f81fc8b911") },
		{ "431166b7-a50a-4ccb-8ca2-a6a3b6376190", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "7ce1dff2-875d-411b-be06-9a57981cbdc7"}, { new Terminator(OperatorType.Multiply), "4d79eeb4-6b1a-4ca0-97f6-1f6675aed09f"}, { new Terminator(OperatorType.Divide), "fecb6832-79a2-4f95-95c4-fc698109ba3e"}, { new Terminator(KeywordType.Divide), "784cd050-c0ae-4844-8706-7f532d156c70"}, { new Terminator(KeywordType.Mod), "f3fdd777-da4f-4c74-9196-b8c4d7b7bace"}, { new Terminator(KeywordType.And), "376ab083-df94-433e-bcac-f300687907b1"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "431166b7-a50a-4ccb-8ca2-a6a3b6376190") },
		{ "ebaa5ae2-0b7d-4e21-8868-fa6d971f9af4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "ebaa5ae2-0b7d-4e21-8868-fa6d971f9af4") },
		{ "62ff026c-b825-42a4-b94f-5a7f0c5eadc3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "62ff026c-b825-42a4-b94f-5a7f0c5eadc3") },
		{ "0eb974e5-1b89-4348-ae84-86606df4b3e2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "0eb974e5-1b89-4348-ae84-86606df4b3e2") },
		{ "70c7bd8e-7d2d-457f-8e4f-aa520d90eceb", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "3b6919e6-5658-4ad9-90b2-9a0bb4f12b16"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "550fbb43-0150-468f-9eb4-7bf5912daa0a"}, { new NonTerminator(NonTerminatorType.Term), "f0cad299-28f5-4e84-b9e7-2bc048f33840"}, { new NonTerminator(NonTerminatorType.Factor), "43e04c0c-8568-4c6b-ab2b-bceedcc9fa32"}, { Terminator.NumberTerminator, "c04070de-7d08-4aad-b5d7-41002b95f79d"}, { new NonTerminator(NonTerminatorType.Variable), "448fb14e-b117-467e-94e2-0ffad798acf2"}, { new Terminator(DelimiterType.LeftParenthesis), "61941794-1fbf-4cdd-a20f-059c95559fcc"}, { Terminator.IdentifierTerminator, "e932245d-2b23-4f7c-b6a4-c6fbd2b3dd9c"}, { new Terminator(KeywordType.Not), "8fcb5991-4a57-4bc9-b6ed-c1c6b22af95e"}, { new Terminator(OperatorType.Minus), "7312a539-9334-411e-b2f6-dc420da87c55"},}, new Dictionary<Terminator, ReduceInformation>{ }, "70c7bd8e-7d2d-457f-8e4f-aa520d90eceb") },
		{ "7f4abdfb-ae44-46c5-baf8-95faf8050421", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "90319653-e56e-41e7-be39-783854eff4ac"}, { new NonTerminator(NonTerminatorType.IdVarPart), "e94b7f87-0f74-4763-bf2f-d7a0efbd24cf"}, { new Terminator(DelimiterType.LeftSquareBracket), "18b94f07-9d3a-4007-a5f8-31e8534fc4bf"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "7f4abdfb-ae44-46c5-baf8-95faf8050421") },
		{ "39ace80c-b849-4b52-b2dd-2b29b089d55e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "1cbf2ca5-d581-460a-b994-70e6f18e762c"}, { Terminator.NumberTerminator, "62ff026c-b825-42a4-b94f-5a7f0c5eadc3"}, { new NonTerminator(NonTerminatorType.Variable), "0eb974e5-1b89-4348-ae84-86606df4b3e2"}, { new Terminator(DelimiterType.LeftParenthesis), "70c7bd8e-7d2d-457f-8e4f-aa520d90eceb"}, { Terminator.IdentifierTerminator, "7f4abdfb-ae44-46c5-baf8-95faf8050421"}, { new Terminator(KeywordType.Not), "39ace80c-b849-4b52-b2dd-2b29b089d55e"}, { new Terminator(OperatorType.Minus), "d2dd7e4e-1dd6-472f-847a-452c97248860"},}, new Dictionary<Terminator, ReduceInformation>{ }, "39ace80c-b849-4b52-b2dd-2b29b089d55e") },
		{ "d2dd7e4e-1dd6-472f-847a-452c97248860", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "0cdebb20-6f56-457a-8ded-55622b8856e3"}, { Terminator.NumberTerminator, "62ff026c-b825-42a4-b94f-5a7f0c5eadc3"}, { new NonTerminator(NonTerminatorType.Variable), "0eb974e5-1b89-4348-ae84-86606df4b3e2"}, { new Terminator(DelimiterType.LeftParenthesis), "70c7bd8e-7d2d-457f-8e4f-aa520d90eceb"}, { Terminator.IdentifierTerminator, "7f4abdfb-ae44-46c5-baf8-95faf8050421"}, { new Terminator(KeywordType.Not), "39ace80c-b849-4b52-b2dd-2b29b089d55e"}, { new Terminator(OperatorType.Minus), "d2dd7e4e-1dd6-472f-847a-452c97248860"},}, new Dictionary<Terminator, ReduceInformation>{ }, "d2dd7e4e-1dd6-472f-847a-452c97248860") },
		{ "49fe4340-9c7e-4d3f-a315-3b907851410b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "71324a45-2a90-476d-814b-b47ef0e7d15f"}, { new Terminator(OperatorType.Multiply), "4d79eeb4-6b1a-4ca0-97f6-1f6675aed09f"}, { new Terminator(OperatorType.Divide), "fecb6832-79a2-4f95-95c4-fc698109ba3e"}, { new Terminator(KeywordType.Divide), "784cd050-c0ae-4844-8706-7f532d156c70"}, { new Terminator(KeywordType.Mod), "f3fdd777-da4f-4c74-9196-b8c4d7b7bace"}, { new Terminator(KeywordType.And), "376ab083-df94-433e-bcac-f300687907b1"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "49fe4340-9c7e-4d3f-a315-3b907851410b") },
		{ "1d05d9be-273f-4d0e-8d7f-7905f70aeada", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "1d05d9be-273f-4d0e-8d7f-7905f70aeada") },
		{ "5487a61e-d1be-478c-8fea-776b8cce7c72", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "5487a61e-d1be-478c-8fea-776b8cce7c72") },
		{ "ad33a481-047b-4172-bb2c-574e5be261be", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "8d706990-1d64-4190-8e16-854d2433cbe7"}, { new Terminator(DelimiterType.Comma), "f8c6158b-c7b5-4520-8a37-49ad141ff150"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ad33a481-047b-4172-bb2c-574e5be261be") },
		{ "9d21fd54-8c38-4b05-b4b8-4b6f529b8d43", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "3acd814b-6f7d-4e13-b7ae-479fd3099075"}, { new Terminator(DelimiterType.Comma), "61999b62-48c1-4d86-8277-f2ec86cd3141"},}, new Dictionary<Terminator, ReduceInformation>{ }, "9d21fd54-8c38-4b05-b4b8-4b6f529b8d43") },
		{ "75c1506e-487e-4018-a995-51419f376ef7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.CompoundStatement))}, }, "75c1506e-487e-4018-a995-51419f376ef7") },
		{ "7422a0bb-5b5d-4e39-b5e1-38e88f81634d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdentifierList))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdentifierList))}, }, "7422a0bb-5b5d-4e39-b5e1-38e88f81634d") },
		{ "bc0bbb00-97d8-487c-8149-40725edf8672", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Period), "a1fa8af5-e306-4d6f-aaa9-713ffe33912d"}, { Terminator.NumberTerminator, "9c9213aa-db0c-400f-a84f-f3bf743ec7d9"},}, new Dictionary<Terminator, ReduceInformation>{ }, "bc0bbb00-97d8-487c-8149-40725edf8672") },
		{ "cc29535d-b050-47c8-a988-cbda96148685", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(6, new NonTerminator(NonTerminatorType.Type))}, }, "cc29535d-b050-47c8-a988-cbda96148685") },
		{ "524789dd-5fbb-4950-8ddb-fa9b8c0c446c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "88cf57b9-1ed9-494d-836a-62c9992ed253"}, { new Terminator(OperatorType.Multiply), "4d79eeb4-6b1a-4ca0-97f6-1f6675aed09f"}, { new Terminator(OperatorType.Divide), "fecb6832-79a2-4f95-95c4-fc698109ba3e"}, { new Terminator(KeywordType.Divide), "784cd050-c0ae-4844-8706-7f532d156c70"}, { new Terminator(KeywordType.Mod), "f3fdd777-da4f-4c74-9196-b8c4d7b7bace"}, { new Terminator(KeywordType.And), "376ab083-df94-433e-bcac-f300687907b1"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "524789dd-5fbb-4950-8ddb-fa9b8c0c446c") },
		{ "a0be572d-6a2f-4faa-ae73-a22c5664c6cc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "a0be572d-6a2f-4faa-ae73-a22c5664c6cc") },
		{ "1980bc4e-3495-46c3-ba86-84e97c1dd237", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "1980bc4e-3495-46c3-ba86-84e97c1dd237") },
		{ "d9d40659-3731-484f-935a-09a99e7dc108", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "5e9813a9-404f-4e71-9078-2d41585b4a37"}, { new Terminator(DelimiterType.Comma), "f8c6158b-c7b5-4520-8a37-49ad141ff150"},}, new Dictionary<Terminator, ReduceInformation>{ }, "d9d40659-3731-484f-935a-09a99e7dc108") },
		{ "f3816498-e846-4718-87b1-fa06bfe1a33c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "ce5cc742-13b1-4f1e-8edb-73e91d87f58a"}, { new Terminator(DelimiterType.Comma), "61999b62-48c1-4d86-8277-f2ec86cd3141"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f3816498-e846-4718-87b1-fa06bfe1a33c") },
		{ "da7433d4-48c3-4fa4-9cf8-ae06585d0f46", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "72099dbe-7244-4758-92c1-c936b2f150d3"}, { new Terminator(OperatorType.Multiply), "4d79eeb4-6b1a-4ca0-97f6-1f6675aed09f"}, { new Terminator(OperatorType.Divide), "fecb6832-79a2-4f95-95c4-fc698109ba3e"}, { new Terminator(KeywordType.Divide), "784cd050-c0ae-4844-8706-7f532d156c70"}, { new Terminator(KeywordType.Mod), "f3fdd777-da4f-4c74-9196-b8c4d7b7bace"}, { new Terminator(KeywordType.And), "376ab083-df94-433e-bcac-f300687907b1"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "da7433d4-48c3-4fa4-9cf8-ae06585d0f46") },
		{ "7ec91a7c-0eaa-4a38-bf8e-523eb640d538", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "7ec91a7c-0eaa-4a38-bf8e-523eb640d538") },
		{ "b4af54eb-e516-4dd0-85e0-48113f9465ce", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "b4af54eb-e516-4dd0-85e0-48113f9465ce") },
		{ "893254fe-3b97-43eb-9313-55cdd8bad4cc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "9b0a2104-fda6-435a-8598-b0b50696f70d"}, { new Terminator(DelimiterType.Comma), "f8c6158b-c7b5-4520-8a37-49ad141ff150"},}, new Dictionary<Terminator, ReduceInformation>{ }, "893254fe-3b97-43eb-9313-55cdd8bad4cc") },
		{ "8eed3d68-4201-4d80-a06b-4f1f949ceba5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "fda9c802-7c73-45ef-985b-e8f37de7635a"}, { new Terminator(DelimiterType.Comma), "61999b62-48c1-4d86-8277-f2ec86cd3141"},}, new Dictionary<Terminator, ReduceInformation>{ }, "8eed3d68-4201-4d80-a06b-4f1f949ceba5") },
		{ "47b44660-5b1c-458a-80bc-a4be9775dcfe", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "9b50ef94-e2d1-4693-aaac-d1a529701978"}, { new Terminator(OperatorType.Multiply), "4d79eeb4-6b1a-4ca0-97f6-1f6675aed09f"}, { new Terminator(OperatorType.Divide), "fecb6832-79a2-4f95-95c4-fc698109ba3e"}, { new Terminator(KeywordType.Divide), "784cd050-c0ae-4844-8706-7f532d156c70"}, { new Terminator(KeywordType.Mod), "f3fdd777-da4f-4c74-9196-b8c4d7b7bace"}, { new Terminator(KeywordType.And), "376ab083-df94-433e-bcac-f300687907b1"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "47b44660-5b1c-458a-80bc-a4be9775dcfe") },
		{ "12ce3d0c-2b67-468f-b6e8-52f5e717ccbe", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "12ce3d0c-2b67-468f-b6e8-52f5e717ccbe") },
		{ "3fedf865-d1f4-48af-bfeb-bbe85aada635", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "3fedf865-d1f4-48af-bfeb-bbe85aada635") },
		{ "48d0c436-828e-452f-a4a8-50f43be27808", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "d6f3e5f8-e189-4d8e-94fd-f06d27b77684"}, { new Terminator(DelimiterType.Comma), "f8c6158b-c7b5-4520-8a37-49ad141ff150"},}, new Dictionary<Terminator, ReduceInformation>{ }, "48d0c436-828e-452f-a4a8-50f43be27808") },
		{ "282d579c-461d-45a2-9531-5ab79224cb85", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "bcdba1b5-1250-4fbe-8821-a283fad50e32"}, { new Terminator(DelimiterType.Comma), "61999b62-48c1-4d86-8277-f2ec86cd3141"},}, new Dictionary<Terminator, ReduceInformation>{ }, "282d579c-461d-45a2-9531-5ab79224cb85") },
		{ "73da82a8-6ec8-4cfa-a4c7-52162174ccac", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SimpleExpression), "32c5f88f-099d-4abb-b0a0-413cf7a36bf4"}, { new NonTerminator(NonTerminatorType.Term), "bacf40a9-9be3-45c7-be5d-2c9e5fa15ffd"}, { new NonTerminator(NonTerminatorType.Factor), "9fd368d0-a001-4b84-aa46-8f0704f4d4b8"}, { Terminator.NumberTerminator, "be9e0c83-08b7-49d7-8b82-da11ae725966"}, { new NonTerminator(NonTerminatorType.Variable), "b87e4e27-3965-48ef-9dfe-6176fac8c44d"}, { new Terminator(DelimiterType.LeftParenthesis), "a8c2102f-0ff3-42cd-bc52-6e1fc69cda7a"}, { Terminator.IdentifierTerminator, "0a4b8e6a-3cff-40c5-bfd1-e0b1db376926"}, { new Terminator(KeywordType.Not), "b3ba0d75-1e89-4d58-aebe-ed5e196ba21e"}, { new Terminator(OperatorType.Minus), "fb5850b4-f92c-4dbd-8f57-e1443165325e"},}, new Dictionary<Terminator, ReduceInformation>{ }, "73da82a8-6ec8-4cfa-a4c7-52162174ccac") },
		{ "59e6221e-b909-4f1d-b0d2-2d53ddfb8baf", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "8f8e13ef-e482-418d-b881-a9277e9bcaab"}, { new NonTerminator(NonTerminatorType.Factor), "84eb5d09-6a9c-4152-8ee1-08cf1977b4b4"}, { Terminator.NumberTerminator, "8695b530-26f3-4534-a7ec-6e01b3b8b90c"}, { new NonTerminator(NonTerminatorType.Variable), "dd2136a1-9348-4c93-a3d6-cd3454e51c22"}, { new Terminator(DelimiterType.LeftParenthesis), "d1159160-d982-4e79-9cb1-37896891fa6b"}, { Terminator.IdentifierTerminator, "74fa8c5c-9ee2-4481-b6b9-1cf08e5fd510"}, { new Terminator(KeywordType.Not), "d2ee92ce-60e1-42c5-93bb-b132c2796e16"}, { new Terminator(OperatorType.Minus), "410a8465-e4b5-4ba9-b1b4-fc3e3f81080e"},}, new Dictionary<Terminator, ReduceInformation>{ }, "59e6221e-b909-4f1d-b0d2-2d53ddfb8baf") },
		{ "6b7d3e6e-5966-4384-a88d-8c063e753c57", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "759e0257-2d80-4fa7-a005-43afbb4e8482"}, { Terminator.NumberTerminator, "8695b530-26f3-4534-a7ec-6e01b3b8b90c"}, { new NonTerminator(NonTerminatorType.Variable), "dd2136a1-9348-4c93-a3d6-cd3454e51c22"}, { new Terminator(DelimiterType.LeftParenthesis), "d1159160-d982-4e79-9cb1-37896891fa6b"}, { Terminator.IdentifierTerminator, "74fa8c5c-9ee2-4481-b6b9-1cf08e5fd510"}, { new Terminator(KeywordType.Not), "d2ee92ce-60e1-42c5-93bb-b132c2796e16"}, { new Terminator(OperatorType.Minus), "410a8465-e4b5-4ba9-b1b4-fc3e3f81080e"},}, new Dictionary<Terminator, ReduceInformation>{ }, "6b7d3e6e-5966-4384-a88d-8c063e753c57") },
		{ "6c171c98-ed71-468e-a69a-3ca45f08c88d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "c2f9c3c1-1e7b-4869-ba2d-27d73c950603"},}, new Dictionary<Terminator, ReduceInformation>{ }, "6c171c98-ed71-468e-a69a-3ca45f08c88d") },
		{ "3aef74b6-a29a-43fc-854a-653867e15a25", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "d8ab9463-73ee-4252-a5a6-55cc72bfed14"}, { new NonTerminator(NonTerminatorType.Expression), "2e7bde52-48a5-420d-a6e3-9f664510da68"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "17428d1e-ed4e-461b-ad85-a5f1d8f006eb"}, { new NonTerminator(NonTerminatorType.Term), "bab97185-47af-4ddd-8275-1863e6c37e23"}, { new NonTerminator(NonTerminatorType.Factor), "f5d56db2-25f1-4001-abce-4572f92e534a"}, { Terminator.NumberTerminator, "35cb5529-4f39-4b7c-a937-233f6efe6cdb"}, { new NonTerminator(NonTerminatorType.Variable), "ca1b9fbe-0a27-4b7d-ad6a-ee1ea8f20f5f"}, { new Terminator(DelimiterType.LeftParenthesis), "5f4ba076-b3cf-4472-848c-74d2a618eacb"}, { Terminator.IdentifierTerminator, "99071775-1827-45c4-b161-826ababf0384"}, { new Terminator(KeywordType.Not), "ed5ffdaa-b277-4c22-ad93-e4b1ed293ae4"}, { new Terminator(OperatorType.Minus), "15dc2c62-c938-4d0a-b0aa-1f0a0b73661f"},}, new Dictionary<Terminator, ReduceInformation>{ }, "3aef74b6-a29a-43fc-854a-653867e15a25") },
		{ "6ca30117-d0b1-4275-9308-1c3c50834c65", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Else), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "6ca30117-d0b1-4275-9308-1c3c50834c65") },
		{ "80ebbe93-93fc-4242-8cb0-18b7b1273475", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "198e02f6-3f24-4aae-8f3c-01cae33ca184"}, { new NonTerminator(NonTerminatorType.Expression), "2e88b57e-7c35-4aee-9387-ff1c061cb7ab"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "089240b9-2be9-4199-97a8-41a6a9ac2f54"}, { new NonTerminator(NonTerminatorType.Term), "f46a091e-b991-43bd-877a-3c2eef645ab7"}, { new NonTerminator(NonTerminatorType.Factor), "5cb8d05c-bce3-47cc-8728-d7c037f2e9b1"}, { Terminator.NumberTerminator, "3928c790-d1f1-4d6a-8e98-6a0d0a3182e7"}, { new NonTerminator(NonTerminatorType.Variable), "08f29b35-75b5-4fe9-b677-ed18c2caa51b"}, { new Terminator(DelimiterType.LeftParenthesis), "7bb9f1fe-edd2-4812-9091-30ae6a61e59d"}, { Terminator.IdentifierTerminator, "ad126546-60a0-4982-8f68-c9a727b1c223"}, { new Terminator(KeywordType.Not), "d17a2e13-31b3-4e22-a0a6-5cf5fa910500"}, { new Terminator(OperatorType.Minus), "e8e4e781-e847-4c03-b1c8-fc377a1a86f7"},}, new Dictionary<Terminator, ReduceInformation>{ }, "80ebbe93-93fc-4242-8cb0-18b7b1273475") },
		{ "1e78bd7f-21e6-43a0-b91b-da2faec0dc95", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "1e78bd7f-21e6-43a0-b91b-da2faec0dc95") },
		{ "881bcf0b-7877-4393-be62-590162979146", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "881bcf0b-7877-4393-be62-590162979146") },
		{ "9181974f-69f2-4398-9909-9adaf29f8ece", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(4, new NonTerminator(NonTerminatorType.ProcedureCall))}, { new Terminator(KeywordType.Else), new ReduceInformation(4, new NonTerminator(NonTerminatorType.ProcedureCall))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(4, new NonTerminator(NonTerminatorType.ProcedureCall))}, }, "9181974f-69f2-4398-9909-9adaf29f8ece") },
		{ "95ca7b97-7e0d-400d-9762-4826cf7f4f49", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ElsePart), "249527b3-486f-4857-8b1e-f4c74d8d7bc0"}, { new Terminator(KeywordType.Else), "56f92e89-3412-4011-b345-70993dfa794d"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ElsePart))}, { new Terminator(KeywordType.Else), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ElsePart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ElsePart))}, }, "95ca7b97-7e0d-400d-9762-4826cf7f4f49") },
		{ "52a8b6bf-9f1a-47f1-82e8-da3979aa3d77", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.To), "4ec69f94-9f17-43dd-a84b-63f2a76fbdb1"},}, new Dictionary<Terminator, ReduceInformation>{ }, "52a8b6bf-9f1a-47f1-82e8-da3979aa3d77") },
		{ "fabb50fe-9b49-4d81-84a8-cf9fa2ee4df4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "fabb50fe-9b49-4d81-84a8-cf9fa2ee4df4") },
		{ "5ce1e180-7ce4-49c8-8179-3a766e92d767", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "5ce1e180-7ce4-49c8-8179-3a766e92d767") },
		{ "b1c2ba9f-5e9d-4c3b-932a-9d5f8e776a41", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "4d0b17ae-8185-4259-8a8d-16432387f97f"}, { new Terminator(OperatorType.Multiply), "4d79eeb4-6b1a-4ca0-97f6-1f6675aed09f"}, { new Terminator(OperatorType.Divide), "fecb6832-79a2-4f95-95c4-fc698109ba3e"}, { new Terminator(KeywordType.Divide), "784cd050-c0ae-4844-8706-7f532d156c70"}, { new Terminator(KeywordType.Mod), "f3fdd777-da4f-4c74-9196-b8c4d7b7bace"}, { new Terminator(KeywordType.And), "376ab083-df94-433e-bcac-f300687907b1"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "b1c2ba9f-5e9d-4c3b-932a-9d5f8e776a41") },
		{ "ac830f96-6bde-49f0-b548-93cb338306f0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "ac830f96-6bde-49f0-b548-93cb338306f0") },
		{ "567a4123-b5e2-494b-9d1a-6753d7e88083", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "567a4123-b5e2-494b-9d1a-6753d7e88083") },
		{ "ffb7db92-10ca-4be1-8be1-e0a960c3b0af", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "43ad4380-070f-48e0-be8f-bf74edc4a14a"}, { new Terminator(DelimiterType.Comma), "f8c6158b-c7b5-4520-8a37-49ad141ff150"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ffb7db92-10ca-4be1-8be1-e0a960c3b0af") },
		{ "af467ff1-300f-4027-98bc-61d2da0ae1ed", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "92b1cf0a-f049-4fcd-9c1e-2b0133b0abea"}, { new Terminator(DelimiterType.Comma), "61999b62-48c1-4d86-8277-f2ec86cd3141"},}, new Dictionary<Terminator, ReduceInformation>{ }, "af467ff1-300f-4027-98bc-61d2da0ae1ed") },
		{ "e321f889-9cdd-4db4-9986-78df04737558", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Statement), "7107a391-7ff3-4cd8-b26a-e898ebd67453"}, { new NonTerminator(NonTerminatorType.Variable), "be92d52d-f5cd-445d-b485-bd33a70d96d0"}, { Terminator.IdentifierTerminator, "b2246d94-54c6-4d0a-b568-08ce2b05aed2"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "54a3f285-f7dd-4bc1-bea4-a895b234ba48"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "430eb033-8171-4e0b-a572-c7fc61d11c56"}, { new Terminator(KeywordType.If), "5a82796b-2d92-4130-9660-38c01054dba4"}, { new Terminator(KeywordType.For), "6f8cff58-95d7-47f8-8456-787a36fc1c87"}, { new Terminator(KeywordType.Begin), "e9129679-d275-4d11-a66e-30fda7679656"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "e321f889-9cdd-4db4-9986-78df04737558") },
		{ "221d48f5-6183-4f2f-b33b-4ebac32a134b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SimpleExpression), "f2cd366a-443f-44f7-97d6-c24ff0404163"}, { new NonTerminator(NonTerminatorType.Term), "71f042c2-e2eb-42cf-833f-9619ff161c25"}, { new NonTerminator(NonTerminatorType.Factor), "9f7de901-90af-4779-b4b9-e0606669d2a1"}, { Terminator.NumberTerminator, "63768ff0-cb59-46b5-b913-b63b40ba4e2b"}, { new NonTerminator(NonTerminatorType.Variable), "7999084a-55b1-40ae-a7b2-b51f36816c27"}, { new Terminator(DelimiterType.LeftParenthesis), "b92de3f4-1e09-44e3-8269-0843ed413914"}, { Terminator.IdentifierTerminator, "2e9e5fb3-80c4-4b84-b46b-283f085e3b3d"}, { new Terminator(KeywordType.Not), "126dac58-a406-4ff3-90ae-c9370b8bcc8c"}, { new Terminator(OperatorType.Minus), "a3403d06-c578-45d3-9e33-30cd0b356b36"},}, new Dictionary<Terminator, ReduceInformation>{ }, "221d48f5-6183-4f2f-b33b-4ebac32a134b") },
		{ "7854af7d-22e7-4d14-bd30-b994c816eb01", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "9222ec95-f220-4c3d-b965-12e88be97404"}, { new NonTerminator(NonTerminatorType.Factor), "e0658f9a-921e-4b74-8fb5-e6b77d907947"}, { Terminator.NumberTerminator, "799b1e98-e544-4cd9-b48d-386861187fe6"}, { new NonTerminator(NonTerminatorType.Variable), "71ecda72-cc64-4040-8496-5a1c0ec09b77"}, { new Terminator(DelimiterType.LeftParenthesis), "c2439980-49fc-48d9-9626-5ba6e380cde7"}, { Terminator.IdentifierTerminator, "820af974-dbec-4e03-8931-dfc4ef51c15a"}, { new Terminator(KeywordType.Not), "c7800d99-09f1-4c4b-a492-985868834e26"}, { new Terminator(OperatorType.Minus), "e1f14f62-b8e2-44d0-9ca8-130cecab42e5"},}, new Dictionary<Terminator, ReduceInformation>{ }, "7854af7d-22e7-4d14-bd30-b994c816eb01") },
		{ "fa1484e0-5a30-4654-a01f-8e372ea2fa7e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "6a941053-1074-4f8f-9acc-1aa8de8125e3"}, { Terminator.NumberTerminator, "799b1e98-e544-4cd9-b48d-386861187fe6"}, { new NonTerminator(NonTerminatorType.Variable), "71ecda72-cc64-4040-8496-5a1c0ec09b77"}, { new Terminator(DelimiterType.LeftParenthesis), "c2439980-49fc-48d9-9626-5ba6e380cde7"}, { Terminator.IdentifierTerminator, "820af974-dbec-4e03-8931-dfc4ef51c15a"}, { new Terminator(KeywordType.Not), "c7800d99-09f1-4c4b-a492-985868834e26"}, { new Terminator(OperatorType.Minus), "e1f14f62-b8e2-44d0-9ca8-130cecab42e5"},}, new Dictionary<Terminator, ReduceInformation>{ }, "fa1484e0-5a30-4654-a01f-8e372ea2fa7e") },
		{ "50bfb7d9-979d-4726-9a67-74a8202fdca3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "6c740b2f-c230-4f14-8216-0ca5ea33b7f6"},}, new Dictionary<Terminator, ReduceInformation>{ }, "50bfb7d9-979d-4726-9a67-74a8202fdca3") },
		{ "a6590b22-d882-4595-bf8e-44c11f64da01", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "7450ae6c-4df7-49f4-918f-1d8c4cbaf24c"}, { new NonTerminator(NonTerminatorType.Expression), "2e7bde52-48a5-420d-a6e3-9f664510da68"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "17428d1e-ed4e-461b-ad85-a5f1d8f006eb"}, { new NonTerminator(NonTerminatorType.Term), "bab97185-47af-4ddd-8275-1863e6c37e23"}, { new NonTerminator(NonTerminatorType.Factor), "f5d56db2-25f1-4001-abce-4572f92e534a"}, { Terminator.NumberTerminator, "35cb5529-4f39-4b7c-a937-233f6efe6cdb"}, { new NonTerminator(NonTerminatorType.Variable), "ca1b9fbe-0a27-4b7d-ad6a-ee1ea8f20f5f"}, { new Terminator(DelimiterType.LeftParenthesis), "5f4ba076-b3cf-4472-848c-74d2a618eacb"}, { Terminator.IdentifierTerminator, "99071775-1827-45c4-b161-826ababf0384"}, { new Terminator(KeywordType.Not), "ed5ffdaa-b277-4c22-ad93-e4b1ed293ae4"}, { new Terminator(OperatorType.Minus), "15dc2c62-c938-4d0a-b0aa-1f0a0b73661f"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a6590b22-d882-4595-bf8e-44c11f64da01") },
		{ "d6d654a3-e442-4123-b40a-45fb135e8016", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "d6d654a3-e442-4123-b40a-45fb135e8016") },
		{ "33b650ee-e803-4027-afae-79827cbeb38a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "fd501094-a95c-46df-962a-ff5d3f400cf0"}, { new NonTerminator(NonTerminatorType.Expression), "2e88b57e-7c35-4aee-9387-ff1c061cb7ab"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "089240b9-2be9-4199-97a8-41a6a9ac2f54"}, { new NonTerminator(NonTerminatorType.Term), "f46a091e-b991-43bd-877a-3c2eef645ab7"}, { new NonTerminator(NonTerminatorType.Factor), "5cb8d05c-bce3-47cc-8728-d7c037f2e9b1"}, { Terminator.NumberTerminator, "3928c790-d1f1-4d6a-8e98-6a0d0a3182e7"}, { new NonTerminator(NonTerminatorType.Variable), "08f29b35-75b5-4fe9-b677-ed18c2caa51b"}, { new Terminator(DelimiterType.LeftParenthesis), "7bb9f1fe-edd2-4812-9091-30ae6a61e59d"}, { Terminator.IdentifierTerminator, "ad126546-60a0-4982-8f68-c9a727b1c223"}, { new Terminator(KeywordType.Not), "d17a2e13-31b3-4e22-a0a6-5cf5fa910500"}, { new Terminator(OperatorType.Minus), "e8e4e781-e847-4c03-b1c8-fc377a1a86f7"},}, new Dictionary<Terminator, ReduceInformation>{ }, "33b650ee-e803-4027-afae-79827cbeb38a") },
		{ "e4c05e25-1793-4efa-b58c-19f0283a7b8a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "e4c05e25-1793-4efa-b58c-19f0283a7b8a") },
		{ "2cc3b96f-2e3c-4e25-9394-12a8c2db113f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "2cc3b96f-2e3c-4e25-9394-12a8c2db113f") },
		{ "c395cf12-2ca1-471d-b748-264d25b9258c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "0d65f563-2d7d-4e13-b302-f07d946e5997"}, { new NonTerminator(NonTerminatorType.Factor), "ebaa5ae2-0b7d-4e21-8868-fa6d971f9af4"}, { Terminator.NumberTerminator, "62ff026c-b825-42a4-b94f-5a7f0c5eadc3"}, { new NonTerminator(NonTerminatorType.Variable), "0eb974e5-1b89-4348-ae84-86606df4b3e2"}, { new Terminator(DelimiterType.LeftParenthesis), "70c7bd8e-7d2d-457f-8e4f-aa520d90eceb"}, { Terminator.IdentifierTerminator, "7f4abdfb-ae44-46c5-baf8-95faf8050421"}, { new Terminator(KeywordType.Not), "39ace80c-b849-4b52-b2dd-2b29b089d55e"}, { new Terminator(OperatorType.Minus), "d2dd7e4e-1dd6-472f-847a-452c97248860"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c395cf12-2ca1-471d-b748-264d25b9258c") },
		{ "7ce1dff2-875d-411b-be06-9a57981cbdc7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "4d6ddd46-0a92-4f16-bb5e-21d745b4033a"}, { Terminator.NumberTerminator, "62ff026c-b825-42a4-b94f-5a7f0c5eadc3"}, { new NonTerminator(NonTerminatorType.Variable), "0eb974e5-1b89-4348-ae84-86606df4b3e2"}, { new Terminator(DelimiterType.LeftParenthesis), "70c7bd8e-7d2d-457f-8e4f-aa520d90eceb"}, { Terminator.IdentifierTerminator, "7f4abdfb-ae44-46c5-baf8-95faf8050421"}, { new Terminator(KeywordType.Not), "39ace80c-b849-4b52-b2dd-2b29b089d55e"}, { new Terminator(OperatorType.Minus), "d2dd7e4e-1dd6-472f-847a-452c97248860"},}, new Dictionary<Terminator, ReduceInformation>{ }, "7ce1dff2-875d-411b-be06-9a57981cbdc7") },
		{ "3b6919e6-5658-4ad9-90b2-9a0bb4f12b16", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "a974e5c4-e289-4571-8875-a1d13bdd0868"},}, new Dictionary<Terminator, ReduceInformation>{ }, "3b6919e6-5658-4ad9-90b2-9a0bb4f12b16") },
		{ "90319653-e56e-41e7-be39-783854eff4ac", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "9c2f583e-7b4c-44af-b1be-1690351180ee"}, { new NonTerminator(NonTerminatorType.Expression), "2e7bde52-48a5-420d-a6e3-9f664510da68"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "17428d1e-ed4e-461b-ad85-a5f1d8f006eb"}, { new NonTerminator(NonTerminatorType.Term), "bab97185-47af-4ddd-8275-1863e6c37e23"}, { new NonTerminator(NonTerminatorType.Factor), "f5d56db2-25f1-4001-abce-4572f92e534a"}, { Terminator.NumberTerminator, "35cb5529-4f39-4b7c-a937-233f6efe6cdb"}, { new NonTerminator(NonTerminatorType.Variable), "ca1b9fbe-0a27-4b7d-ad6a-ee1ea8f20f5f"}, { new Terminator(DelimiterType.LeftParenthesis), "5f4ba076-b3cf-4472-848c-74d2a618eacb"}, { Terminator.IdentifierTerminator, "99071775-1827-45c4-b161-826ababf0384"}, { new Terminator(KeywordType.Not), "ed5ffdaa-b277-4c22-ad93-e4b1ed293ae4"}, { new Terminator(OperatorType.Minus), "15dc2c62-c938-4d0a-b0aa-1f0a0b73661f"},}, new Dictionary<Terminator, ReduceInformation>{ }, "90319653-e56e-41e7-be39-783854eff4ac") },
		{ "e94b7f87-0f74-4763-bf2f-d7a0efbd24cf", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "e94b7f87-0f74-4763-bf2f-d7a0efbd24cf") },
		{ "18b94f07-9d3a-4007-a5f8-31e8534fc4bf", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "4af346d7-3a64-465f-b754-925b9c293852"}, { new NonTerminator(NonTerminatorType.Expression), "2e88b57e-7c35-4aee-9387-ff1c061cb7ab"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "089240b9-2be9-4199-97a8-41a6a9ac2f54"}, { new NonTerminator(NonTerminatorType.Term), "f46a091e-b991-43bd-877a-3c2eef645ab7"}, { new NonTerminator(NonTerminatorType.Factor), "5cb8d05c-bce3-47cc-8728-d7c037f2e9b1"}, { Terminator.NumberTerminator, "3928c790-d1f1-4d6a-8e98-6a0d0a3182e7"}, { new NonTerminator(NonTerminatorType.Variable), "08f29b35-75b5-4fe9-b677-ed18c2caa51b"}, { new Terminator(DelimiterType.LeftParenthesis), "7bb9f1fe-edd2-4812-9091-30ae6a61e59d"}, { Terminator.IdentifierTerminator, "ad126546-60a0-4982-8f68-c9a727b1c223"}, { new Terminator(KeywordType.Not), "d17a2e13-31b3-4e22-a0a6-5cf5fa910500"}, { new Terminator(OperatorType.Minus), "e8e4e781-e847-4c03-b1c8-fc377a1a86f7"},}, new Dictionary<Terminator, ReduceInformation>{ }, "18b94f07-9d3a-4007-a5f8-31e8534fc4bf") },
		{ "1cbf2ca5-d581-460a-b994-70e6f18e762c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "1cbf2ca5-d581-460a-b994-70e6f18e762c") },
		{ "0cdebb20-6f56-457a-8ded-55622b8856e3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "0cdebb20-6f56-457a-8ded-55622b8856e3") },
		{ "8d706990-1d64-4190-8e16-854d2433cbe7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "8d706990-1d64-4190-8e16-854d2433cbe7") },
		{ "3acd814b-6f7d-4e13-b7ae-479fd3099075", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "3acd814b-6f7d-4e13-b7ae-479fd3099075") },
		{ "a1fa8af5-e306-4d6f-aaa9-713ffe33912d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "2ea4476d-3abf-4762-b269-83ff04634928"}, { new Terminator(DelimiterType.Comma), "16576908-4ae2-4aea-a555-907676e5b539"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a1fa8af5-e306-4d6f-aaa9-713ffe33912d") },
		{ "5e9813a9-404f-4e71-9078-2d41585b4a37", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "5e9813a9-404f-4e71-9078-2d41585b4a37") },
		{ "ce5cc742-13b1-4f1e-8edb-73e91d87f58a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "ce5cc742-13b1-4f1e-8edb-73e91d87f58a") },
		{ "9b0a2104-fda6-435a-8598-b0b50696f70d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "9b0a2104-fda6-435a-8598-b0b50696f70d") },
		{ "fda9c802-7c73-45ef-985b-e8f37de7635a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "fda9c802-7c73-45ef-985b-e8f37de7635a") },
		{ "d6f3e5f8-e189-4d8e-94fd-f06d27b77684", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "d6f3e5f8-e189-4d8e-94fd-f06d27b77684") },
		{ "bcdba1b5-1250-4fbe-8821-a283fad50e32", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "bcdba1b5-1250-4fbe-8821-a283fad50e32") },
		{ "32c5f88f-099d-4abb-b0a0-413cf7a36bf4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.AddOperator), "d54112f1-c9d9-4628-a9ba-3063f250de16"}, { new Terminator(OperatorType.Plus), "2cf28f7e-6888-4e54-9825-8a80b00c7a56"}, { new Terminator(OperatorType.Minus), "1804ef7c-da92-431e-b725-ea7823b69327"}, { new Terminator(KeywordType.Or), "e777b812-5ca1-4317-9fd4-cc37fbcb2af0"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, }, "32c5f88f-099d-4abb-b0a0-413cf7a36bf4") },
		{ "bacf40a9-9be3-45c7-be5d-2c9e5fa15ffd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "dcee0aef-7e0a-4ce1-bee4-34e8f8fcd066"}, { new Terminator(OperatorType.Multiply), "4d79eeb4-6b1a-4ca0-97f6-1f6675aed09f"}, { new Terminator(OperatorType.Divide), "fecb6832-79a2-4f95-95c4-fc698109ba3e"}, { new Terminator(KeywordType.Divide), "784cd050-c0ae-4844-8706-7f532d156c70"}, { new Terminator(KeywordType.Mod), "f3fdd777-da4f-4c74-9196-b8c4d7b7bace"}, { new Terminator(KeywordType.And), "376ab083-df94-433e-bcac-f300687907b1"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "bacf40a9-9be3-45c7-be5d-2c9e5fa15ffd") },
		{ "9fd368d0-a001-4b84-aa46-8f0704f4d4b8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "9fd368d0-a001-4b84-aa46-8f0704f4d4b8") },
		{ "be9e0c83-08b7-49d7-8b82-da11ae725966", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "be9e0c83-08b7-49d7-8b82-da11ae725966") },
		{ "b87e4e27-3965-48ef-9dfe-6176fac8c44d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "b87e4e27-3965-48ef-9dfe-6176fac8c44d") },
		{ "a8c2102f-0ff3-42cd-bc52-6e1fc69cda7a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "afbd32b4-0244-4fd5-9861-f20fb9a07435"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "550fbb43-0150-468f-9eb4-7bf5912daa0a"}, { new NonTerminator(NonTerminatorType.Term), "f0cad299-28f5-4e84-b9e7-2bc048f33840"}, { new NonTerminator(NonTerminatorType.Factor), "43e04c0c-8568-4c6b-ab2b-bceedcc9fa32"}, { Terminator.NumberTerminator, "c04070de-7d08-4aad-b5d7-41002b95f79d"}, { new NonTerminator(NonTerminatorType.Variable), "448fb14e-b117-467e-94e2-0ffad798acf2"}, { new Terminator(DelimiterType.LeftParenthesis), "61941794-1fbf-4cdd-a20f-059c95559fcc"}, { Terminator.IdentifierTerminator, "e932245d-2b23-4f7c-b6a4-c6fbd2b3dd9c"}, { new Terminator(KeywordType.Not), "8fcb5991-4a57-4bc9-b6ed-c1c6b22af95e"}, { new Terminator(OperatorType.Minus), "7312a539-9334-411e-b2f6-dc420da87c55"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a8c2102f-0ff3-42cd-bc52-6e1fc69cda7a") },
		{ "0a4b8e6a-3cff-40c5-bfd1-e0b1db376926", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "7992617e-e6c2-4aae-84e1-6ea8d61af027"}, { new NonTerminator(NonTerminatorType.IdVarPart), "de281630-3dd6-4481-a75d-ab630d81854d"}, { new Terminator(DelimiterType.LeftSquareBracket), "f576bd7c-0d9c-42d2-a4a1-7915264de14b"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Else), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "0a4b8e6a-3cff-40c5-bfd1-e0b1db376926") },
		{ "b3ba0d75-1e89-4d58-aebe-ed5e196ba21e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "e8fe72ef-8bc6-4a08-a72e-4825b2645720"}, { Terminator.NumberTerminator, "be9e0c83-08b7-49d7-8b82-da11ae725966"}, { new NonTerminator(NonTerminatorType.Variable), "b87e4e27-3965-48ef-9dfe-6176fac8c44d"}, { new Terminator(DelimiterType.LeftParenthesis), "a8c2102f-0ff3-42cd-bc52-6e1fc69cda7a"}, { Terminator.IdentifierTerminator, "0a4b8e6a-3cff-40c5-bfd1-e0b1db376926"}, { new Terminator(KeywordType.Not), "b3ba0d75-1e89-4d58-aebe-ed5e196ba21e"}, { new Terminator(OperatorType.Minus), "fb5850b4-f92c-4dbd-8f57-e1443165325e"},}, new Dictionary<Terminator, ReduceInformation>{ }, "b3ba0d75-1e89-4d58-aebe-ed5e196ba21e") },
		{ "fb5850b4-f92c-4dbd-8f57-e1443165325e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "3c78c5e2-584c-4851-a378-0fe21fd94baf"}, { Terminator.NumberTerminator, "be9e0c83-08b7-49d7-8b82-da11ae725966"}, { new NonTerminator(NonTerminatorType.Variable), "b87e4e27-3965-48ef-9dfe-6176fac8c44d"}, { new Terminator(DelimiterType.LeftParenthesis), "a8c2102f-0ff3-42cd-bc52-6e1fc69cda7a"}, { Terminator.IdentifierTerminator, "0a4b8e6a-3cff-40c5-bfd1-e0b1db376926"}, { new Terminator(KeywordType.Not), "b3ba0d75-1e89-4d58-aebe-ed5e196ba21e"}, { new Terminator(OperatorType.Minus), "fb5850b4-f92c-4dbd-8f57-e1443165325e"},}, new Dictionary<Terminator, ReduceInformation>{ }, "fb5850b4-f92c-4dbd-8f57-e1443165325e") },
		{ "8f8e13ef-e482-418d-b881-a9277e9bcaab", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "6b7d3e6e-5966-4384-a88d-8c063e753c57"}, { new Terminator(OperatorType.Multiply), "4d79eeb4-6b1a-4ca0-97f6-1f6675aed09f"}, { new Terminator(OperatorType.Divide), "fecb6832-79a2-4f95-95c4-fc698109ba3e"}, { new Terminator(KeywordType.Divide), "784cd050-c0ae-4844-8706-7f532d156c70"}, { new Terminator(KeywordType.Mod), "f3fdd777-da4f-4c74-9196-b8c4d7b7bace"}, { new Terminator(KeywordType.And), "376ab083-df94-433e-bcac-f300687907b1"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "8f8e13ef-e482-418d-b881-a9277e9bcaab") },
		{ "759e0257-2d80-4fa7-a005-43afbb4e8482", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "759e0257-2d80-4fa7-a005-43afbb4e8482") },
		{ "c2f9c3c1-1e7b-4869-ba2d-27d73c950603", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "c2f9c3c1-1e7b-4869-ba2d-27d73c950603") },
		{ "d8ab9463-73ee-4252-a5a6-55cc72bfed14", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "797298f3-691a-4c09-8fde-7f00b8a20224"}, { new Terminator(DelimiterType.Comma), "f8c6158b-c7b5-4520-8a37-49ad141ff150"},}, new Dictionary<Terminator, ReduceInformation>{ }, "d8ab9463-73ee-4252-a5a6-55cc72bfed14") },
		{ "198e02f6-3f24-4aae-8f3c-01cae33ca184", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "52403dde-9f3e-43d9-bba1-b8d2f00c3c67"}, { new Terminator(DelimiterType.Comma), "61999b62-48c1-4d86-8277-f2ec86cd3141"},}, new Dictionary<Terminator, ReduceInformation>{ }, "198e02f6-3f24-4aae-8f3c-01cae33ca184") },
		{ "249527b3-486f-4857-8b1e-f4c74d8d7bc0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(5, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(5, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(5, new NonTerminator(NonTerminatorType.Statement))}, }, "249527b3-486f-4857-8b1e-f4c74d8d7bc0") },
		{ "56f92e89-3412-4011-b345-70993dfa794d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Statement), "51ea81ae-034c-42a9-af9c-c1492628cf59"}, { new NonTerminator(NonTerminatorType.Variable), "ea208c56-799b-4542-aab9-97041337f8cd"}, { Terminator.IdentifierTerminator, "9a4e95ea-b154-4175-98e3-a2691d4eb20f"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "14f3970e-268e-4251-aa12-eaa8effa762a"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "b1815343-33db-47b4-8679-35c240abe9b0"}, { new Terminator(KeywordType.If), "e3fc592c-2252-423a-8c29-eb6efd3e239e"}, { new Terminator(KeywordType.For), "35582002-f9e8-40c5-b5fe-930d8a63d94a"}, { new Terminator(KeywordType.Begin), "09269d2d-ccde-4852-8c11-5c03f8f1f9f2"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "56f92e89-3412-4011-b345-70993dfa794d") },
		{ "4ec69f94-9f17-43dd-a84b-63f2a76fbdb1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "06410c8b-ce8a-4caf-96a2-973d1d02b6c1"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "caa8525c-e146-4f2e-9489-00c244cfad57"}, { new NonTerminator(NonTerminatorType.Term), "54afe343-cda5-4f84-aa6d-5a21b405f421"}, { new NonTerminator(NonTerminatorType.Factor), "e0658f9a-921e-4b74-8fb5-e6b77d907947"}, { Terminator.NumberTerminator, "799b1e98-e544-4cd9-b48d-386861187fe6"}, { new NonTerminator(NonTerminatorType.Variable), "71ecda72-cc64-4040-8496-5a1c0ec09b77"}, { new Terminator(DelimiterType.LeftParenthesis), "c2439980-49fc-48d9-9626-5ba6e380cde7"}, { Terminator.IdentifierTerminator, "820af974-dbec-4e03-8931-dfc4ef51c15a"}, { new Terminator(KeywordType.Not), "c7800d99-09f1-4c4b-a492-985868834e26"}, { new Terminator(OperatorType.Minus), "e1f14f62-b8e2-44d0-9ca8-130cecab42e5"},}, new Dictionary<Terminator, ReduceInformation>{ }, "4ec69f94-9f17-43dd-a84b-63f2a76fbdb1") },
		{ "43ad4380-070f-48e0-be8f-bf74edc4a14a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "43ad4380-070f-48e0-be8f-bf74edc4a14a") },
		{ "92b1cf0a-f049-4fcd-9c1e-2b0133b0abea", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "92b1cf0a-f049-4fcd-9c1e-2b0133b0abea") },
		{ "7107a391-7ff3-4cd8-b26a-e898ebd67453", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(8, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(8, new NonTerminator(NonTerminatorType.Statement))}, }, "7107a391-7ff3-4cd8-b26a-e898ebd67453") },
		{ "f2cd366a-443f-44f7-97d6-c24ff0404163", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.AddOperator), "f60b2069-d973-4564-a4a7-5df43e010dce"}, { new Terminator(OperatorType.Plus), "2cf28f7e-6888-4e54-9825-8a80b00c7a56"}, { new Terminator(OperatorType.Minus), "1804ef7c-da92-431e-b725-ea7823b69327"}, { new Terminator(KeywordType.Or), "e777b812-5ca1-4317-9fd4-cc37fbcb2af0"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, }, "f2cd366a-443f-44f7-97d6-c24ff0404163") },
		{ "71f042c2-e2eb-42cf-833f-9619ff161c25", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "93a69f85-c213-4ee3-bb14-d44235ddaffb"}, { new Terminator(OperatorType.Multiply), "4d79eeb4-6b1a-4ca0-97f6-1f6675aed09f"}, { new Terminator(OperatorType.Divide), "fecb6832-79a2-4f95-95c4-fc698109ba3e"}, { new Terminator(KeywordType.Divide), "784cd050-c0ae-4844-8706-7f532d156c70"}, { new Terminator(KeywordType.Mod), "f3fdd777-da4f-4c74-9196-b8c4d7b7bace"}, { new Terminator(KeywordType.And), "376ab083-df94-433e-bcac-f300687907b1"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "71f042c2-e2eb-42cf-833f-9619ff161c25") },
		{ "9f7de901-90af-4779-b4b9-e0606669d2a1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "9f7de901-90af-4779-b4b9-e0606669d2a1") },
		{ "63768ff0-cb59-46b5-b913-b63b40ba4e2b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "63768ff0-cb59-46b5-b913-b63b40ba4e2b") },
		{ "7999084a-55b1-40ae-a7b2-b51f36816c27", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "7999084a-55b1-40ae-a7b2-b51f36816c27") },
		{ "b92de3f4-1e09-44e3-8269-0843ed413914", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "d1d80ce4-6c73-42af-97af-71be663d9ca1"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "550fbb43-0150-468f-9eb4-7bf5912daa0a"}, { new NonTerminator(NonTerminatorType.Term), "f0cad299-28f5-4e84-b9e7-2bc048f33840"}, { new NonTerminator(NonTerminatorType.Factor), "43e04c0c-8568-4c6b-ab2b-bceedcc9fa32"}, { Terminator.NumberTerminator, "c04070de-7d08-4aad-b5d7-41002b95f79d"}, { new NonTerminator(NonTerminatorType.Variable), "448fb14e-b117-467e-94e2-0ffad798acf2"}, { new Terminator(DelimiterType.LeftParenthesis), "61941794-1fbf-4cdd-a20f-059c95559fcc"}, { Terminator.IdentifierTerminator, "e932245d-2b23-4f7c-b6a4-c6fbd2b3dd9c"}, { new Terminator(KeywordType.Not), "8fcb5991-4a57-4bc9-b6ed-c1c6b22af95e"}, { new Terminator(OperatorType.Minus), "7312a539-9334-411e-b2f6-dc420da87c55"},}, new Dictionary<Terminator, ReduceInformation>{ }, "b92de3f4-1e09-44e3-8269-0843ed413914") },
		{ "2e9e5fb3-80c4-4b84-b46b-283f085e3b3d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "909a2e1a-6021-485c-957a-7461c8354655"}, { new NonTerminator(NonTerminatorType.IdVarPart), "4dccbe6f-6fc1-4eef-87b5-bdaa6136ea29"}, { new Terminator(DelimiterType.LeftSquareBracket), "8f0085c3-f3ad-444b-a57d-185fb4d1d3df"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "2e9e5fb3-80c4-4b84-b46b-283f085e3b3d") },
		{ "126dac58-a406-4ff3-90ae-c9370b8bcc8c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "a1c50061-d325-4b45-b30c-745956b63f7d"}, { Terminator.NumberTerminator, "63768ff0-cb59-46b5-b913-b63b40ba4e2b"}, { new NonTerminator(NonTerminatorType.Variable), "7999084a-55b1-40ae-a7b2-b51f36816c27"}, { new Terminator(DelimiterType.LeftParenthesis), "b92de3f4-1e09-44e3-8269-0843ed413914"}, { Terminator.IdentifierTerminator, "2e9e5fb3-80c4-4b84-b46b-283f085e3b3d"}, { new Terminator(KeywordType.Not), "126dac58-a406-4ff3-90ae-c9370b8bcc8c"}, { new Terminator(OperatorType.Minus), "a3403d06-c578-45d3-9e33-30cd0b356b36"},}, new Dictionary<Terminator, ReduceInformation>{ }, "126dac58-a406-4ff3-90ae-c9370b8bcc8c") },
		{ "a3403d06-c578-45d3-9e33-30cd0b356b36", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "59be0810-6d6e-46ef-94dd-0d72553aaf53"}, { Terminator.NumberTerminator, "63768ff0-cb59-46b5-b913-b63b40ba4e2b"}, { new NonTerminator(NonTerminatorType.Variable), "7999084a-55b1-40ae-a7b2-b51f36816c27"}, { new Terminator(DelimiterType.LeftParenthesis), "b92de3f4-1e09-44e3-8269-0843ed413914"}, { Terminator.IdentifierTerminator, "2e9e5fb3-80c4-4b84-b46b-283f085e3b3d"}, { new Terminator(KeywordType.Not), "126dac58-a406-4ff3-90ae-c9370b8bcc8c"}, { new Terminator(OperatorType.Minus), "a3403d06-c578-45d3-9e33-30cd0b356b36"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a3403d06-c578-45d3-9e33-30cd0b356b36") },
		{ "9222ec95-f220-4c3d-b965-12e88be97404", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "fa1484e0-5a30-4654-a01f-8e372ea2fa7e"}, { new Terminator(OperatorType.Multiply), "4d79eeb4-6b1a-4ca0-97f6-1f6675aed09f"}, { new Terminator(OperatorType.Divide), "fecb6832-79a2-4f95-95c4-fc698109ba3e"}, { new Terminator(KeywordType.Divide), "784cd050-c0ae-4844-8706-7f532d156c70"}, { new Terminator(KeywordType.Mod), "f3fdd777-da4f-4c74-9196-b8c4d7b7bace"}, { new Terminator(KeywordType.And), "376ab083-df94-433e-bcac-f300687907b1"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "9222ec95-f220-4c3d-b965-12e88be97404") },
		{ "6a941053-1074-4f8f-9acc-1aa8de8125e3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "6a941053-1074-4f8f-9acc-1aa8de8125e3") },
		{ "6c740b2f-c230-4f14-8216-0ca5ea33b7f6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "6c740b2f-c230-4f14-8216-0ca5ea33b7f6") },
		{ "7450ae6c-4df7-49f4-918f-1d8c4cbaf24c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "ca752cb2-e90c-4865-aba4-95cae299ce5a"}, { new Terminator(DelimiterType.Comma), "f8c6158b-c7b5-4520-8a37-49ad141ff150"},}, new Dictionary<Terminator, ReduceInformation>{ }, "7450ae6c-4df7-49f4-918f-1d8c4cbaf24c") },
		{ "fd501094-a95c-46df-962a-ff5d3f400cf0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "1fb6a754-06f0-48fe-ab12-449bca69e36c"}, { new Terminator(DelimiterType.Comma), "61999b62-48c1-4d86-8277-f2ec86cd3141"},}, new Dictionary<Terminator, ReduceInformation>{ }, "fd501094-a95c-46df-962a-ff5d3f400cf0") },
		{ "0d65f563-2d7d-4e13-b302-f07d946e5997", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "7ce1dff2-875d-411b-be06-9a57981cbdc7"}, { new Terminator(OperatorType.Multiply), "4d79eeb4-6b1a-4ca0-97f6-1f6675aed09f"}, { new Terminator(OperatorType.Divide), "fecb6832-79a2-4f95-95c4-fc698109ba3e"}, { new Terminator(KeywordType.Divide), "784cd050-c0ae-4844-8706-7f532d156c70"}, { new Terminator(KeywordType.Mod), "f3fdd777-da4f-4c74-9196-b8c4d7b7bace"}, { new Terminator(KeywordType.And), "376ab083-df94-433e-bcac-f300687907b1"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "0d65f563-2d7d-4e13-b302-f07d946e5997") },
		{ "4d6ddd46-0a92-4f16-bb5e-21d745b4033a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "4d6ddd46-0a92-4f16-bb5e-21d745b4033a") },
		{ "a974e5c4-e289-4571-8875-a1d13bdd0868", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "a974e5c4-e289-4571-8875-a1d13bdd0868") },
		{ "9c2f583e-7b4c-44af-b1be-1690351180ee", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "aca5e55a-5613-4312-88e3-66c68fffdb07"}, { new Terminator(DelimiterType.Comma), "f8c6158b-c7b5-4520-8a37-49ad141ff150"},}, new Dictionary<Terminator, ReduceInformation>{ }, "9c2f583e-7b4c-44af-b1be-1690351180ee") },
		{ "4af346d7-3a64-465f-b754-925b9c293852", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "5bc4211e-b7d9-4134-b554-575bdddd73dd"}, { new Terminator(DelimiterType.Comma), "61999b62-48c1-4d86-8277-f2ec86cd3141"},}, new Dictionary<Terminator, ReduceInformation>{ }, "4af346d7-3a64-465f-b754-925b9c293852") },
		{ "2ea4476d-3abf-4762-b269-83ff04634928", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.Of), "a1dc1253-570e-4000-a7af-1605b4d24e81"},}, new Dictionary<Terminator, ReduceInformation>{ }, "2ea4476d-3abf-4762-b269-83ff04634928") },
		{ "d54112f1-c9d9-4628-a9ba-3063f250de16", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "d5a4c9a6-6d17-4d2b-85b5-1e0f96e78ce7"}, { new NonTerminator(NonTerminatorType.Factor), "9fd368d0-a001-4b84-aa46-8f0704f4d4b8"}, { Terminator.NumberTerminator, "be9e0c83-08b7-49d7-8b82-da11ae725966"}, { new NonTerminator(NonTerminatorType.Variable), "b87e4e27-3965-48ef-9dfe-6176fac8c44d"}, { new Terminator(DelimiterType.LeftParenthesis), "a8c2102f-0ff3-42cd-bc52-6e1fc69cda7a"}, { Terminator.IdentifierTerminator, "0a4b8e6a-3cff-40c5-bfd1-e0b1db376926"}, { new Terminator(KeywordType.Not), "b3ba0d75-1e89-4d58-aebe-ed5e196ba21e"}, { new Terminator(OperatorType.Minus), "fb5850b4-f92c-4dbd-8f57-e1443165325e"},}, new Dictionary<Terminator, ReduceInformation>{ }, "d54112f1-c9d9-4628-a9ba-3063f250de16") },
		{ "dcee0aef-7e0a-4ce1-bee4-34e8f8fcd066", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "d752afa7-d077-4887-a4d4-1cf20da135d2"}, { Terminator.NumberTerminator, "be9e0c83-08b7-49d7-8b82-da11ae725966"}, { new NonTerminator(NonTerminatorType.Variable), "b87e4e27-3965-48ef-9dfe-6176fac8c44d"}, { new Terminator(DelimiterType.LeftParenthesis), "a8c2102f-0ff3-42cd-bc52-6e1fc69cda7a"}, { Terminator.IdentifierTerminator, "0a4b8e6a-3cff-40c5-bfd1-e0b1db376926"}, { new Terminator(KeywordType.Not), "b3ba0d75-1e89-4d58-aebe-ed5e196ba21e"}, { new Terminator(OperatorType.Minus), "fb5850b4-f92c-4dbd-8f57-e1443165325e"},}, new Dictionary<Terminator, ReduceInformation>{ }, "dcee0aef-7e0a-4ce1-bee4-34e8f8fcd066") },
		{ "afbd32b4-0244-4fd5-9861-f20fb9a07435", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "073ef123-cbbf-4433-a727-5ec64b8a1a3e"},}, new Dictionary<Terminator, ReduceInformation>{ }, "afbd32b4-0244-4fd5-9861-f20fb9a07435") },
		{ "7992617e-e6c2-4aae-84e1-6ea8d61af027", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "f8145475-ec15-479b-b299-6750f629dc55"}, { new NonTerminator(NonTerminatorType.Expression), "2e7bde52-48a5-420d-a6e3-9f664510da68"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "17428d1e-ed4e-461b-ad85-a5f1d8f006eb"}, { new NonTerminator(NonTerminatorType.Term), "bab97185-47af-4ddd-8275-1863e6c37e23"}, { new NonTerminator(NonTerminatorType.Factor), "f5d56db2-25f1-4001-abce-4572f92e534a"}, { Terminator.NumberTerminator, "35cb5529-4f39-4b7c-a937-233f6efe6cdb"}, { new NonTerminator(NonTerminatorType.Variable), "ca1b9fbe-0a27-4b7d-ad6a-ee1ea8f20f5f"}, { new Terminator(DelimiterType.LeftParenthesis), "5f4ba076-b3cf-4472-848c-74d2a618eacb"}, { Terminator.IdentifierTerminator, "99071775-1827-45c4-b161-826ababf0384"}, { new Terminator(KeywordType.Not), "ed5ffdaa-b277-4c22-ad93-e4b1ed293ae4"}, { new Terminator(OperatorType.Minus), "15dc2c62-c938-4d0a-b0aa-1f0a0b73661f"},}, new Dictionary<Terminator, ReduceInformation>{ }, "7992617e-e6c2-4aae-84e1-6ea8d61af027") },
		{ "de281630-3dd6-4481-a75d-ab630d81854d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Else), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "de281630-3dd6-4481-a75d-ab630d81854d") },
		{ "f576bd7c-0d9c-42d2-a4a1-7915264de14b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "1ee54946-c6f5-4c6d-a1ec-d6335d8bce7b"}, { new NonTerminator(NonTerminatorType.Expression), "2e88b57e-7c35-4aee-9387-ff1c061cb7ab"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "089240b9-2be9-4199-97a8-41a6a9ac2f54"}, { new NonTerminator(NonTerminatorType.Term), "f46a091e-b991-43bd-877a-3c2eef645ab7"}, { new NonTerminator(NonTerminatorType.Factor), "5cb8d05c-bce3-47cc-8728-d7c037f2e9b1"}, { Terminator.NumberTerminator, "3928c790-d1f1-4d6a-8e98-6a0d0a3182e7"}, { new NonTerminator(NonTerminatorType.Variable), "08f29b35-75b5-4fe9-b677-ed18c2caa51b"}, { new Terminator(DelimiterType.LeftParenthesis), "7bb9f1fe-edd2-4812-9091-30ae6a61e59d"}, { Terminator.IdentifierTerminator, "ad126546-60a0-4982-8f68-c9a727b1c223"}, { new Terminator(KeywordType.Not), "d17a2e13-31b3-4e22-a0a6-5cf5fa910500"}, { new Terminator(OperatorType.Minus), "e8e4e781-e847-4c03-b1c8-fc377a1a86f7"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f576bd7c-0d9c-42d2-a4a1-7915264de14b") },
		{ "e8fe72ef-8bc6-4a08-a72e-4825b2645720", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "e8fe72ef-8bc6-4a08-a72e-4825b2645720") },
		{ "3c78c5e2-584c-4851-a378-0fe21fd94baf", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "3c78c5e2-584c-4851-a378-0fe21fd94baf") },
		{ "797298f3-691a-4c09-8fde-7f00b8a20224", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "797298f3-691a-4c09-8fde-7f00b8a20224") },
		{ "52403dde-9f3e-43d9-bba1-b8d2f00c3c67", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "52403dde-9f3e-43d9-bba1-b8d2f00c3c67") },
		{ "51ea81ae-034c-42a9-af9c-c1492628cf59", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.ElsePart))}, { new Terminator(KeywordType.Else), new ReduceInformation(2, new NonTerminator(NonTerminatorType.ElsePart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.ElsePart))}, }, "51ea81ae-034c-42a9-af9c-c1492628cf59") },
		{ "06410c8b-ce8a-4caf-96a2-973d1d02b6c1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.Do), "4cb466ce-1065-45c7-8f5d-124142bd656e"},}, new Dictionary<Terminator, ReduceInformation>{ }, "06410c8b-ce8a-4caf-96a2-973d1d02b6c1") },
		{ "f60b2069-d973-4564-a4a7-5df43e010dce", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "abc7ae54-f4da-4e7a-978f-1a3488585d12"}, { new NonTerminator(NonTerminatorType.Factor), "9f7de901-90af-4779-b4b9-e0606669d2a1"}, { Terminator.NumberTerminator, "63768ff0-cb59-46b5-b913-b63b40ba4e2b"}, { new NonTerminator(NonTerminatorType.Variable), "7999084a-55b1-40ae-a7b2-b51f36816c27"}, { new Terminator(DelimiterType.LeftParenthesis), "b92de3f4-1e09-44e3-8269-0843ed413914"}, { Terminator.IdentifierTerminator, "2e9e5fb3-80c4-4b84-b46b-283f085e3b3d"}, { new Terminator(KeywordType.Not), "126dac58-a406-4ff3-90ae-c9370b8bcc8c"}, { new Terminator(OperatorType.Minus), "a3403d06-c578-45d3-9e33-30cd0b356b36"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f60b2069-d973-4564-a4a7-5df43e010dce") },
		{ "93a69f85-c213-4ee3-bb14-d44235ddaffb", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "1dc4f6ae-ec48-4801-9108-e181f74f3a82"}, { Terminator.NumberTerminator, "63768ff0-cb59-46b5-b913-b63b40ba4e2b"}, { new NonTerminator(NonTerminatorType.Variable), "7999084a-55b1-40ae-a7b2-b51f36816c27"}, { new Terminator(DelimiterType.LeftParenthesis), "b92de3f4-1e09-44e3-8269-0843ed413914"}, { Terminator.IdentifierTerminator, "2e9e5fb3-80c4-4b84-b46b-283f085e3b3d"}, { new Terminator(KeywordType.Not), "126dac58-a406-4ff3-90ae-c9370b8bcc8c"}, { new Terminator(OperatorType.Minus), "a3403d06-c578-45d3-9e33-30cd0b356b36"},}, new Dictionary<Terminator, ReduceInformation>{ }, "93a69f85-c213-4ee3-bb14-d44235ddaffb") },
		{ "d1d80ce4-6c73-42af-97af-71be663d9ca1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "9ec67e1d-0dcd-4bd7-a716-5566d125bc61"},}, new Dictionary<Terminator, ReduceInformation>{ }, "d1d80ce4-6c73-42af-97af-71be663d9ca1") },
		{ "909a2e1a-6021-485c-957a-7461c8354655", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "e5c9bbb8-6604-4d10-a450-a915c0257be4"}, { new NonTerminator(NonTerminatorType.Expression), "2e7bde52-48a5-420d-a6e3-9f664510da68"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "17428d1e-ed4e-461b-ad85-a5f1d8f006eb"}, { new NonTerminator(NonTerminatorType.Term), "bab97185-47af-4ddd-8275-1863e6c37e23"}, { new NonTerminator(NonTerminatorType.Factor), "f5d56db2-25f1-4001-abce-4572f92e534a"}, { Terminator.NumberTerminator, "35cb5529-4f39-4b7c-a937-233f6efe6cdb"}, { new NonTerminator(NonTerminatorType.Variable), "ca1b9fbe-0a27-4b7d-ad6a-ee1ea8f20f5f"}, { new Terminator(DelimiterType.LeftParenthesis), "5f4ba076-b3cf-4472-848c-74d2a618eacb"}, { Terminator.IdentifierTerminator, "99071775-1827-45c4-b161-826ababf0384"}, { new Terminator(KeywordType.Not), "ed5ffdaa-b277-4c22-ad93-e4b1ed293ae4"}, { new Terminator(OperatorType.Minus), "15dc2c62-c938-4d0a-b0aa-1f0a0b73661f"},}, new Dictionary<Terminator, ReduceInformation>{ }, "909a2e1a-6021-485c-957a-7461c8354655") },
		{ "4dccbe6f-6fc1-4eef-87b5-bdaa6136ea29", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "4dccbe6f-6fc1-4eef-87b5-bdaa6136ea29") },
		{ "8f0085c3-f3ad-444b-a57d-185fb4d1d3df", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "bba7e6dd-c39d-4797-a7a7-7131eb9b2288"}, { new NonTerminator(NonTerminatorType.Expression), "2e88b57e-7c35-4aee-9387-ff1c061cb7ab"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "089240b9-2be9-4199-97a8-41a6a9ac2f54"}, { new NonTerminator(NonTerminatorType.Term), "f46a091e-b991-43bd-877a-3c2eef645ab7"}, { new NonTerminator(NonTerminatorType.Factor), "5cb8d05c-bce3-47cc-8728-d7c037f2e9b1"}, { Terminator.NumberTerminator, "3928c790-d1f1-4d6a-8e98-6a0d0a3182e7"}, { new NonTerminator(NonTerminatorType.Variable), "08f29b35-75b5-4fe9-b677-ed18c2caa51b"}, { new Terminator(DelimiterType.LeftParenthesis), "7bb9f1fe-edd2-4812-9091-30ae6a61e59d"}, { Terminator.IdentifierTerminator, "ad126546-60a0-4982-8f68-c9a727b1c223"}, { new Terminator(KeywordType.Not), "d17a2e13-31b3-4e22-a0a6-5cf5fa910500"}, { new Terminator(OperatorType.Minus), "e8e4e781-e847-4c03-b1c8-fc377a1a86f7"},}, new Dictionary<Terminator, ReduceInformation>{ }, "8f0085c3-f3ad-444b-a57d-185fb4d1d3df") },
		{ "a1c50061-d325-4b45-b30c-745956b63f7d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "a1c50061-d325-4b45-b30c-745956b63f7d") },
		{ "59be0810-6d6e-46ef-94dd-0d72553aaf53", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "59be0810-6d6e-46ef-94dd-0d72553aaf53") },
		{ "ca752cb2-e90c-4865-aba4-95cae299ce5a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "ca752cb2-e90c-4865-aba4-95cae299ce5a") },
		{ "1fb6a754-06f0-48fe-ab12-449bca69e36c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "1fb6a754-06f0-48fe-ab12-449bca69e36c") },
		{ "aca5e55a-5613-4312-88e3-66c68fffdb07", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "aca5e55a-5613-4312-88e3-66c68fffdb07") },
		{ "5bc4211e-b7d9-4134-b554-575bdddd73dd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "5bc4211e-b7d9-4134-b554-575bdddd73dd") },
		{ "a1dc1253-570e-4000-a7af-1605b4d24e81", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.BasicType), "c2212e3f-7bd8-4a10-95a5-4cd2a5cf6302"}, { new Terminator(KeywordType.Integer), "2352d3f9-6384-4719-a336-d52576da05cd"}, { new Terminator(KeywordType.Real), "f0f06df7-693c-4589-a52c-1fc0b527a0c4"}, { new Terminator(KeywordType.Boolean), "b946cd1b-56cc-44f0-a1aa-9a4204988ac3"}, { new Terminator(KeywordType.Character), "c1517388-d76a-4ff8-81ce-ee6053dae200"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a1dc1253-570e-4000-a7af-1605b4d24e81") },
		{ "d5a4c9a6-6d17-4d2b-85b5-1e0f96e78ce7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "dcee0aef-7e0a-4ce1-bee4-34e8f8fcd066"}, { new Terminator(OperatorType.Multiply), "4d79eeb4-6b1a-4ca0-97f6-1f6675aed09f"}, { new Terminator(OperatorType.Divide), "fecb6832-79a2-4f95-95c4-fc698109ba3e"}, { new Terminator(KeywordType.Divide), "784cd050-c0ae-4844-8706-7f532d156c70"}, { new Terminator(KeywordType.Mod), "f3fdd777-da4f-4c74-9196-b8c4d7b7bace"}, { new Terminator(KeywordType.And), "376ab083-df94-433e-bcac-f300687907b1"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "d5a4c9a6-6d17-4d2b-85b5-1e0f96e78ce7") },
		{ "d752afa7-d077-4887-a4d4-1cf20da135d2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "d752afa7-d077-4887-a4d4-1cf20da135d2") },
		{ "073ef123-cbbf-4433-a727-5ec64b8a1a3e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "073ef123-cbbf-4433-a727-5ec64b8a1a3e") },
		{ "f8145475-ec15-479b-b299-6750f629dc55", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "c3682599-d72e-421d-ac11-fb50e4b22a46"}, { new Terminator(DelimiterType.Comma), "f8c6158b-c7b5-4520-8a37-49ad141ff150"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f8145475-ec15-479b-b299-6750f629dc55") },
		{ "1ee54946-c6f5-4c6d-a1ec-d6335d8bce7b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "d125c2ea-8153-44ff-ae93-c113f2fa3854"}, { new Terminator(DelimiterType.Comma), "61999b62-48c1-4d86-8277-f2ec86cd3141"},}, new Dictionary<Terminator, ReduceInformation>{ }, "1ee54946-c6f5-4c6d-a1ec-d6335d8bce7b") },
		{ "4cb466ce-1065-45c7-8f5d-124142bd656e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Statement), "f52e901e-1a0a-402d-a6ae-e1c4e366e6bf"}, { new NonTerminator(NonTerminatorType.Variable), "ea208c56-799b-4542-aab9-97041337f8cd"}, { Terminator.IdentifierTerminator, "9a4e95ea-b154-4175-98e3-a2691d4eb20f"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "14f3970e-268e-4251-aa12-eaa8effa762a"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "b1815343-33db-47b4-8679-35c240abe9b0"}, { new Terminator(KeywordType.If), "e3fc592c-2252-423a-8c29-eb6efd3e239e"}, { new Terminator(KeywordType.For), "35582002-f9e8-40c5-b5fe-930d8a63d94a"}, { new Terminator(KeywordType.Begin), "09269d2d-ccde-4852-8c11-5c03f8f1f9f2"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "4cb466ce-1065-45c7-8f5d-124142bd656e") },
		{ "abc7ae54-f4da-4e7a-978f-1a3488585d12", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "93a69f85-c213-4ee3-bb14-d44235ddaffb"}, { new Terminator(OperatorType.Multiply), "4d79eeb4-6b1a-4ca0-97f6-1f6675aed09f"}, { new Terminator(OperatorType.Divide), "fecb6832-79a2-4f95-95c4-fc698109ba3e"}, { new Terminator(KeywordType.Divide), "784cd050-c0ae-4844-8706-7f532d156c70"}, { new Terminator(KeywordType.Mod), "f3fdd777-da4f-4c74-9196-b8c4d7b7bace"}, { new Terminator(KeywordType.And), "376ab083-df94-433e-bcac-f300687907b1"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "abc7ae54-f4da-4e7a-978f-1a3488585d12") },
		{ "1dc4f6ae-ec48-4801-9108-e181f74f3a82", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "1dc4f6ae-ec48-4801-9108-e181f74f3a82") },
		{ "9ec67e1d-0dcd-4bd7-a716-5566d125bc61", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "9ec67e1d-0dcd-4bd7-a716-5566d125bc61") },
		{ "e5c9bbb8-6604-4d10-a450-a915c0257be4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "1b6c8783-7597-4a1a-a0b5-0b4005f45109"}, { new Terminator(DelimiterType.Comma), "f8c6158b-c7b5-4520-8a37-49ad141ff150"},}, new Dictionary<Terminator, ReduceInformation>{ }, "e5c9bbb8-6604-4d10-a450-a915c0257be4") },
		{ "bba7e6dd-c39d-4797-a7a7-7131eb9b2288", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "3da4bee0-54fb-423a-abfd-44e9a7ef35e6"}, { new Terminator(DelimiterType.Comma), "61999b62-48c1-4d86-8277-f2ec86cd3141"},}, new Dictionary<Terminator, ReduceInformation>{ }, "bba7e6dd-c39d-4797-a7a7-7131eb9b2288") },
		{ "c2212e3f-7bd8-4a10-95a5-4cd2a5cf6302", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(6, new NonTerminator(NonTerminatorType.Type))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(6, new NonTerminator(NonTerminatorType.Type))}, }, "c2212e3f-7bd8-4a10-95a5-4cd2a5cf6302") },
		{ "c3682599-d72e-421d-ac11-fb50e4b22a46", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "c3682599-d72e-421d-ac11-fb50e4b22a46") },
		{ "d125c2ea-8153-44ff-ae93-c113f2fa3854", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "d125c2ea-8153-44ff-ae93-c113f2fa3854") },
		{ "f52e901e-1a0a-402d-a6ae-e1c4e366e6bf", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(8, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(8, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(8, new NonTerminator(NonTerminatorType.Statement))}, }, "f52e901e-1a0a-402d-a6ae-e1c4e366e6bf") },
		{ "1b6c8783-7597-4a1a-a0b5-0b4005f45109", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "1b6c8783-7597-4a1a-a0b5-0b4005f45109") },
		{ "3da4bee0-54fb-423a-abfd-44e9a7ef35e6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "3da4bee0-54fb-423a-abfd-44e9a7ef35e6") },
	};

    private GeneratedGrammarParser()
    {
        foreach(GeneratedTransformer transformer in s_transformers.Values)
        {
            transformer.ConstructShiftTable(s_transformers);
        }
    }

    private static GeneratedGrammarParser s_instance = new GeneratedGrammarParser();

    public static GeneratedGrammarParser Instance => s_instance;

	public ITransformer BeginTransformer => s_transformers["42b4bf15-b0ab-4e7a-86db-947c31c11f30"];
	public NonTerminator Begin => new NonTerminator(NonTerminatorType.StartNonTerminator);
}
